@@ -34,6 +34,7 @@ main(int argc, char **argv)
     __pmArchCtl	archctl;
     __pmPDU	*pdp;
     pmTimeval	epoch = { 0, 0 };
+    __pmTimestamp	stamp;
     int		numinst;
     int		*ilist;
     char	**nlist;
@@ -86,40 +87,49 @@ Options:\n\
     memset(&logctl, 0, sizeof(logctl));
     memset(&archctl, 0, sizeof(archctl));
     archctl.ac_log = &logctl;
-    if ((sts = __pmLogCreate("qatest", argv[optind], LOG_PDU_VERSION, &archctl)) != 0) {
+    if ((sts = __pmLogCreate("qatest", argv[optind], PM_LOG_VERS02, &archctl)) != 0) {
 	fprintf(stderr, "%s: __pmLogCreate failed: %s\n", pmGetProgname(), pmErrStr(sts));
 	exit(1);
     }
-    logctl.l_state = PM_LOG_STATE_INIT;
+    logctl.state = PM_LOG_STATE_INIT;
 
     /*
      * make the archive label deterministic
      */
-    logctl.l_label.ill_pid = 1234;
-    logctl.l_label.ill_start.tv_sec = epoch.tv_sec;
-    logctl.l_label.ill_start.tv_usec = epoch.tv_usec;
-    strcpy(logctl.l_label.ill_hostname, "happycamper");
-    strcpy(logctl.l_label.ill_tz, "UTC");
+    logctl.label.pid = 1234;
+    logctl.label.start.sec = epoch.tv_sec;
+    logctl.label.start.nsec = epoch.tv_usec * 1000;
+    if (logctl.label.hostname)
+	free(logctl.label.hostname);
+    logctl.label.hostname = strdup("happycamper");
+    if (logctl.label.timezone)
+	free(logctl.label.timezone);
+    logctl.label.timezone = strdup("UTC");
+    if (logctl.label.zoneinfo)
+	free(logctl.label.zoneinfo);
+    logctl.label.zoneinfo = NULL;
 
-    logctl.l_label.ill_vol = PM_LOG_VOL_TI;
-    if ((sts = __pmLogWriteLabel(logctl.l_tifp, &logctl.l_label)) != 0) {
+    logctl.label.vol = PM_LOG_VOL_TI;
+    if ((sts = __pmLogWriteLabel(logctl.tifp, &logctl.label)) != 0) {
 	fprintf(stderr, "%s: __pmLogWriteLabel TI failed: %s\n", pmGetProgname(), pmErrStr(sts));
 	exit(1);
     }
-    logctl.l_label.ill_vol = PM_LOG_VOL_META;
-    if ((sts = __pmLogWriteLabel(logctl.l_mdfp, &logctl.l_label)) != 0) {
+    logctl.label.vol = PM_LOG_VOL_META;
+    if ((sts = __pmLogWriteLabel(logctl.mdfp, &logctl.label)) != 0) {
 	fprintf(stderr, "%s: __pmLogWriteLabel META failed: %s\n", pmGetProgname(), pmErrStr(sts));
 	exit(1);
     }
-    logctl.l_label.ill_vol = 0;
-    if ((sts = __pmLogWriteLabel(archctl.ac_mfp, &logctl.l_label)) != 0) {
+    logctl.label.vol = 0;
+    if ((sts = __pmLogWriteLabel(archctl.ac_mfp, &logctl.label)) != 0) {
 	fprintf(stderr, "%s: __pmLogWriteLabel VOL 0 failed: %s\n", pmGetProgname(), pmErrStr(sts));
 	exit(1);
     }
 
     __pmFflush(archctl.ac_mfp);
-    __pmFflush(logctl.l_mdfp);
-    __pmLogPutIndex(&archctl, &epoch);
+    __pmFflush(logctl.mdfp);
+    stamp.sec = epoch.tv_sec;
+    stamp.nsec = epoch.tv_usec * 1000;
+    __pmLogPutIndex(&archctl, &stamp);
 
     pmids = (pmID *)malloc(nmetric*sizeof(pmID));
     assert(pmids != NULL);
@@ -142,7 +152,7 @@ Options:\n\
 		printf("pmGetInDom: %s: %s\n", pmInDomStr(desc.indom), pmErrStr(numinst));
 		exit(1);
 	    }
-	    if ((sts = __pmLogPutInDom(&archctl, desc.indom, &epoch, numinst, ilist, nlist)) < 0) {
+	    if ((sts = __pmLogPutInDom(&archctl, desc.indom, &stamp, numinst, ilist, nlist)) < 0) {
 		fprintf(stderr, "%s: __pmLogPutInDom(...,indom=%s,numinst=%d,...) failed: %s\n", pmGetProgname(), pmInDomStr(desc.indom), numinst, pmErrStr(sts));
 		exit(1);
 	    }
@@ -179,8 +189,10 @@ Options:\n\
     }
 
     __pmFflush(archctl.ac_mfp);
-    __pmFflush(logctl.l_mdfp);
-    __pmLogPutIndex(&archctl, &epoch);
+    __pmFflush(logctl.mdfp);
+    stamp.sec = epoch.tv_sec;
+    stamp.nsec = epoch.tv_usec * 1000;
+    __pmLogPutIndex(&archctl, &stamp);
 
     return 0;
 }
