@@ -10,7 +10,8 @@ main(int argc, char **argv)
     int		c;
     int		sts;
     int		errflag = 0;
-    char	*host = "localhost";
+    int		highres = 0;
+    char	*hostspec = "local:";
     int		type = PM_CONTEXT_HOST;
     char	*namespace = PM_NS_DEFAULT;
     int		iterations = 2000;
@@ -18,13 +19,14 @@ main(int argc, char **argv)
     const char	*metric;
     pmID	pmid;
     pmResult	*result;
-    struct timeval      before, after;
+    pmHighResResult *hresult;
+    struct timespec before, after;
     double	delta;
-    static char	*usage = "[-h hostname] [-L] [-n namespace] [-i iterations] metric";
+    static char	*usage = "[-h hostspec] [-H] [-L] [-n namespace] [-i iterations] metric";
 
     pmSetProgname(argv[0]);
 
-    while ((c = getopt(argc, argv, "D:h:Ln:i:")) != EOF) {
+    while ((c = getopt(argc, argv, "D:h:HLn:i:")) != EOF) {
 	switch (c) {
 
 	case 'D':	/* debug options */
@@ -38,13 +40,17 @@ main(int argc, char **argv)
 
 	case 'h':	/* hostname for PMCD to contact */
 	    type = PM_CONTEXT_HOST;
-	    host = optarg;
+	    hostspec = optarg;
+	    break;
+
+	case 'H':	/* high resolution fetching */
+	    highres = 1;
 	    break;
 
 	case 'L':	/* local, no PMCD */
 	    putenv("PMDA_LOCAL_SAMPLE=");
 	    type = PM_CONTEXT_LOCAL;
-	    host = NULL;
+	    hostspec = NULL;
 	    break;
 
 	case 'i':	/* iterations */
@@ -62,55 +68,59 @@ main(int argc, char **argv)
 	}
     }
 
-    if (errflag) {
-USAGE:
+    /* non-flag args are argv[optind] ... argv[argc-1] */
+    if (errflag || optind >= argc) {
 	fprintf(stderr, "Usage: %s %s\n", pmGetProgname(), usage);
 	exit(1);
     }
 
-    if (namespace != PM_NS_DEFAULT && (sts = pmLoadASCIINameSpace(namespace, 1)) < 0) {
-	printf("%s: Cannot load namespace from \"%s\": %s\n", pmGetProgname(), namespace, pmErrStr(sts));
+    if ((namespace != PM_NS_DEFAULT) &&
+	(sts = pmLoadASCIINameSpace(namespace, 1)) < 0) {
+	fprintf(stderr, "%s: Cannot load namespace from \"%s\": %s\n",
+			pmGetProgname(), namespace, pmErrStr(sts));
 	exit(1);
     }
 
-    if ((sts = pmNewContext(type, host)) < 0) {
+    if ((sts = pmNewContext(type, hostspec)) < 0) {
 	if (type == PM_CONTEXT_HOST)
-	    printf("%s: Cannot connect to PMCD on host \"%s\": %s\n", pmGetProgname(), host, pmErrStr(sts));
+	    fprintf(stderr, "%s: Cannot connect to PMCD on host \"%s\": %s\n",
+			pmGetProgname(), hostspec, pmErrStr(sts));
 	else
-	    printf("%s: Cannot make standalone local connection: %s\n", pmGetProgname(), pmErrStr(sts));
+	    fprintf(stderr, "%s: Cannot make local context connection: %s\n",
+			pmGetProgname(), pmErrStr(sts));
 	exit(1);
     }
 
-    /* non-flag args are argv[optind] ... argv[argc-1] */
-    if (optind >= argc)
-	goto USAGE;
-
     metric = argv[optind];
     if ((sts = pmLookupName(1, &metric, &pmid)) < 0) {
-	printf("%s: metric ``%s'' : %s\n", pmGetProgname(), metric, pmErrStr(sts));
+	fprintf(stderr, "%s: metric ``%s'' : %s\n",
+			pmGetProgname(), metric, pmErrStr(sts));
 	exit(1);
     }
 
-    gettimeofday(&before, (struct timezone *)0);
+    pmtimespecNow(&before);
     for (iter=0; iter < iterations; iter++) {
-	sts = pmFetch(1, &pmid, &result);
+	sts = highres ? pmHighResFetch(1, &pmid, &hresult) :
+			pmFetch(1, &pmid, &result);
 	if (sts < 0) {
-	    printf("%s: iteration %d : %s\n", pmGetProgname(), iter, pmErrStr(sts));
+	    fprintf(stderr, "%s: iteration %d : %s\n",
+			    pmGetProgname(), iter, pmErrStr(sts));
 	    exit(1);
 	}
-	pmFreeResult(result);
+	highres ? pmFreeHighResResult(hresult) : pmFreeResult(result);
     }
-    gettimeofday(&after, (struct timezone *)0);
+    pmtimespecNow(&after);
 
     if ((sts = pmWhichContext()) < 0) {
-	printf("%s: pmWhichContext: %s\n", pmGetProgname(), pmErrStr(sts));
+	fprintf(stderr, "%s: pmWhichContext: %s\n",
+			pmGetProgname(), pmErrStr(sts));
 	exit(1);
     }
     pmDestroyContext(sts);
 
-    delta = pmtimevalSub(&after, &before);
+    delta = pmtimespecSub(&after, &before);
     printf("%s: metric %s %.2lf fetches/second\n",
-    pmGetProgname(), metric, (double)iterations / delta);
+	    pmGetProgname(), metric, (double)iterations / delta);
 
     exit(0);
 }
