@@ -73,7 +73,7 @@ _pcp_complete()
         arg_regex="-[dpw]"
     ;;
     pmiectl)
-        all_args="acfiNpV"
+        all_args="acfimNpV"
         arg_regex="-[cip]"
     ;;
     pminfo)
@@ -93,7 +93,7 @@ _pcp_complete()
         arg_regex="-[nSTZ]"
     ;;
     pmlogctl)
-        all_args="acfiNpV"
+        all_args="acfimNpV"
         arg_regex="-[cip]"
     ;;
     pmlogextract)
@@ -101,8 +101,8 @@ _pcp_complete()
         arg_regex="-[cSsTvZ]"
     ;;
     pmlogger)
-        all_args="CcHhKLlmNnoPprsTtUuVvxy"
-        arg_regex="-[cHhKlmnpsTtUVvx]"
+        all_args="CcHIhKLlmNnoPprsTtUuVvxy"
+        arg_regex="-[cHIhKlmnpsTtUVvx]"
     ;;
     pmloglabel)
         all_args="hLlpsVvZ"
