@@ -244,6 +244,11 @@ printindom(FILE *f, pmInResult *irp)
 {
     int		i;
 
+    if (irp->numinst == 0) {
+	fprintf(f, "Instance domain is empty\n");
+	return;
+    }
+
     for (i = 0; i < irp->numinst; i++) {
 	fprintf(f, "[%3d]", i);
 	if (irp->instlist != NULL)
@@ -298,7 +303,7 @@ dohelp(int command, int full)
 	    break;
 	case DBG:
 	    puts("debug all | none");
-	    puts("debug option [ option ... ] (option is a symbolic name or a decimal number)");
+	    puts("debug [-]flag [ [-]flag ... ] (- prefix to clear)");
 	    break;
 	case DESC:
 	    puts("desc metric");
