@@ -332,7 +332,7 @@ load_pmcd_ports(void)
 }
 
 static void
-load_proxy_hostspec(pmHostSpec *proxy)
+load_proxy_hostspec(__pmHostSpec *proxy)
 {
     char	errmsg[PM_MAXERRMSGLEN];
     char	*envstr;
@@ -362,7 +362,7 @@ load_proxy_hostspec(pmHostSpec *proxy)
 }
 
 void
-__pmConnectGetPorts(pmHostSpec *host)
+__pmConnectGetPorts(__pmHostSpec *host)
 {
     PM_INIT_LOCKS();
     PM_LOCK(connect_lock);
@@ -378,7 +378,7 @@ __pmConnectGetPorts(pmHostSpec *host)
 }
 
 int
-__pmConnectPMCD(pmHostSpec *hosts, int nhosts, int ctxflags, __pmHashCtl *attrs)
+__pmConnectPMCD(__pmHostSpec *hosts, int nhosts, int ctxflags, __pmHashCtl *attrs)
 {
     int		sts = -1;
     int		fd = -1;	/* Fd for socket connection to pmcd */
@@ -387,10 +387,10 @@ __pmConnectPMCD(pmHostSpec *hosts, int nhosts, int ctxflags, __pmHashCtl *attrs)
     int		portIx;
     int		version = -1;
     int		proxyport;
-    pmHostSpec	*proxyhost;
+    __pmHostSpec *proxyhost;
 
     static int first_time = 1;
-    static pmHostSpec proxy;
+    static __pmHostSpec proxy;
 
     PM_INIT_LOCKS();
     PM_LOCK(connect_lock);
