@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2012-2018,2020 Red Hat.
+ * Copyright (c) 2012-2018,2020-2021 Red Hat.
  * Copyright (c) 2007-2008 Aconex.  All Rights Reserved.
  * Copyright (c) 1995-2002,2004,2006,2008 Silicon Graphics, Inc.  All Rights Reserved.
  * 
@@ -303,7 +303,7 @@ pmGetHostName(int handle, char *buf, int buflen)
 	    break;
 
 	case PM_CONTEXT_ARCHIVE:
-	    strncpy(buf, ctxp->c_archctl->ac_log->l_label.ill_hostname, buflen-1);
+	    strncpy(buf, ctxp->c_archctl->ac_log->label.hostname, buflen-1);
 	    break;
 	}
 
@@ -507,18 +507,18 @@ __pmFindOrOpenArchive(__pmContext *ctxp, const char *name, int multi_arch)
 
     /*
      * We're done with the current archive, if any. Close it, if necessary.
-     * If we do close it, keep the __pmLogCtl block for re-use of the l_pmns.
+     * If we do close it, keep the __pmLogCtl block for re-use of the pmns.
      */
     acp = ctxp->c_archctl;
     lcp = acp->ac_log;
     if (lcp) {
-	PM_LOCK(lcp->l_lock);
-	if (--lcp->l_refcnt == 0) {
-	    PM_UNLOCK(lcp->l_lock);
+	PM_LOCK(lcp->lc_lock);
+	if (--lcp->refcnt == 0) {
+	    PM_UNLOCK(lcp->lc_lock);
 	    __pmLogClose(acp);
 	}
 	else {
-	    PM_UNLOCK(lcp->l_lock);
+	    PM_UNLOCK(lcp->lc_lock);
 	    lcp = NULL;
 	}
     }
@@ -526,7 +526,7 @@ __pmFindOrOpenArchive(__pmContext *ctxp, const char *name, int multi_arch)
     /*
      * See if an archive by this name is already open in another context.
      * We can't share archive control structs for multi-archive contexts
-     * because, for those, there is a global l_pmns which is shared among the
+     * because, for those, there is a global pmns which is shared among the
      * archives in the context.
      *
      * We must take the contexts_lock mutex for this search, and we need
@@ -550,13 +550,13 @@ __pmFindOrOpenArchive(__pmContext *ctxp, const char *name, int multi_arch)
 	    ctxp2 = contexts[i];
 	    if (ctxp2->c_type == PM_CONTEXT_ARCHIVE) {
 		acp2 = ctxp2->c_archctl;
-		PM_LOCK(acp2->ac_log->l_lock);
-		if (! acp2->ac_log->l_multi &&
-		    strcmp (name, acp2->ac_log->l_name) == 0) {
+		PM_LOCK(acp2->ac_log->lc_lock);
+		if (! acp2->ac_log->multi &&
+		    strcmp (name, acp2->ac_log->name) == 0) {
 		    lcp2 = acp2->ac_log;
 		    break;
 		}
-		PM_UNLOCK(acp2->ac_log->l_lock);
+		PM_UNLOCK(acp2->ac_log->lc_lock);
 	    }
 	}
 
@@ -568,25 +568,25 @@ __pmFindOrOpenArchive(__pmContext *ctxp, const char *name, int multi_arch)
 	if (lcp2 != NULL) {
 	    if (lcp) {
 #ifdef PM_MULTI_THREAD
-		__pmDestroyMutex(&lcp->l_lock);
+		__pmDestroyMutex(&lcp->lc_lock);
 #endif
 		free(lcp);
 	    }
-	    ++lcp2->l_refcnt;
+	    ++lcp2->refcnt;
 	    acp->ac_log = lcp2;
-	    PM_UNLOCK(acp2->ac_log->l_lock);
+	    PM_UNLOCK(acp2->ac_log->lc_lock);
 	    PM_UNLOCK(contexts_lock);
 	    /*
 	     * Setup the per-context part of the controls ...
 	     */
 	    acp->ac_mfp = NULL;
 	    acp->ac_curvol = -1;
-	    sts = __pmLogChangeVol(acp, acp->ac_log->l_minvol);
+	    sts = __pmLogChangeVol(acp, acp->ac_log->minvol);
 	    if  (sts < 0) {
 		if (pmDebugOptions.log) {
 		    char	errmsg[PM_MAXERRMSGLEN];
 		    fprintf(stderr, "__pmFindOrOpenArchive(..., %s, ...): __pmLogChangeVol(..., %d) failed: %s\n",
-		    	name, acp->ac_log->l_minvol, pmErrStr_r(sts, errmsg, sizeof(errmsg)));
+		    	name, acp->ac_log->minvol, pmErrStr_r(sts, errmsg, sizeof(errmsg)));
 		}
 		acp->ac_log = NULL;
 		return sts;
@@ -606,27 +606,27 @@ __pmFindOrOpenArchive(__pmContext *ctxp, const char *name, int multi_arch)
 	    /* NOTREACHED */
 	}
 #ifdef PM_MULTI_THREAD
-	__pmInitMutex(&lcp->l_lock);
+	__pmInitMutex(&lcp->lc_lock);
 #endif
-	lcp->l_multi = multi_arch;
+	lcp->multi = multi_arch;
 	acp->ac_log = lcp;
     }
     sts = __pmLogOpen(name, ctxp);
     if (sts < 0) {
 #ifdef PM_MULTI_THREAD
-	__pmDestroyMutex(&lcp->l_lock);
+	__pmDestroyMutex(&lcp->lc_lock);
 #endif
 	free(lcp);
 	acp->ac_log = NULL;
     }
     else {
 	/*
-	 * Note: we don't need to l_lock here, this is a new __pmLogCtl
+	 * Note: we don't need the lc_lock here, this is a new __pmLogCtl
 	 * structure and we hold the context lock for the only context
 	 * that will point to this __pmLogCtl ... no one else can see
 	 * it yet
 	 */
-	lcp->l_refcnt = 1;
+	lcp->refcnt = 1;
     }
 
     return sts;
@@ -690,11 +690,7 @@ expandArchiveList(const char *names)
     char	*dirname;
     const char	*suffix;
     DIR		*dirp = NULL;
-#if defined(HAVE_READDIR64)
-    struct dirent64	*direntp;
-#else
     struct dirent	*direntp;
-#endif
  
     current = names;
     while (*current) {
@@ -721,11 +717,7 @@ expandArchiveList(const char *names)
 
 	/* dirp is an on-stack variable, so readdir*() is THREADSAFE */
 	if ((dirp = opendir(dirname)) != NULL) {
-#if defined(HAVE_READDIR64)
-	    while ((direntp = readdir64(dirp)) != NULL) {	/* THREADSAFE */
-#else
 	    while ((direntp = readdir(dirp)) != NULL) {		/* THREADSAFE */
-#endif
 		/*
 		 * If this file is part of an archive, then add it.
 		 * Look for names ending in .meta. These are unique to
@@ -792,11 +784,11 @@ initarchive(__pmContext	*ctxp, const char *name)
     int			multi_arch = 0;
     int			ignore;
     double		tdiff;
-    pmLogLabel		label;
-    pmTimeval		tmpTime;
+    __pmLogLabel	*lp;
+    __pmTimestamp	tmpTime;
 
     /*
-     * Catch these early. Formerly caught by __pmLogLoadLabel(), but with
+     * Catch these early. Formerly caught by __pmLogOpenFind(), but with
      * multi-archive support, things are more complex now.
      */
     if (name == NULL || *name == '\0')
@@ -852,23 +844,21 @@ initarchive(__pmContext	*ctxp, const char *name)
 	 * Obtain the start time of this archive. The end time could change
 	 * on the fly and needs to be re-checked as needed.
 	 */
-	if ((sts = __pmGetArchiveLabel(ctxp->c_archctl->ac_log, &label)) < 0)
-	    goto error;
+	lp = &ctxp->c_archctl->ac_log->label;
 
 	/*
 	 * Insert this new entry into the list in sequence by time. Check for
 	 * overlaps. Also check for duplicates.
 	 */
-	tmpTime.tv_sec = (__uint32_t)label.ll_start.tv_sec;
-	tmpTime.tv_usec = (__uint32_t)label.ll_start.tv_usec;
+	tmpTime = lp->start;
 	ignore = 0;
 	for (i = 0; i < acp->ac_num_logs; i++) {
-	    tdiff = __pmTimevalSub(&tmpTime, &acp->ac_log_list[i]->ml_starttime);
+	    tdiff = __pmTimestampSub(&tmpTime, &acp->ac_log_list[i]->starttime);
 	    if (tdiff < 0.0) /* found insertion point */
 		break;
 	    if (tdiff == 0.0) {
 		/* Is it a duplicate? */
-		if (strcmp (current, acp->ac_log_list[i]->ml_name) == 0) {
+		if (strcmp (current, acp->ac_log_list[i]->name) == 0) {
 		    ignore = 1;
 		    break;
 		}
@@ -886,29 +876,41 @@ initarchive(__pmContext	*ctxp, const char *name)
 						   (acp->ac_num_logs + 1) *
 						   sizeof(*acp->ac_log_list));
 	    if (list_new == NULL) {
-		pmNoMem("initArchive",
+		pmNoMem("initArchive: list_new",
 			  (acp->ac_num_logs + 1) * sizeof(*acp->ac_log_list),
 			  PM_FATAL_ERR);
 		/* NOTREACHED */
 	    }
 	    acp->ac_log_list = list_new;
 	    if ((mlcp = (__pmMultiLogCtl *)malloc(sizeof(__pmMultiLogCtl))) == NULL) {
-		pmNoMem("initArchive", sizeof(__pmMultiLogCtl), PM_FATAL_ERR);
+		pmNoMem("initArchive: __pmMultiLogCtl", sizeof(__pmMultiLogCtl), PM_FATAL_ERR);
 		/* NOTREACHED */
 	    }
-	    if ((mlcp->ml_name = strdup(current)) == NULL) {
-		pmNoMem("initArchive", strlen(current) + 1, PM_FATAL_ERR);
+	    if ((mlcp->name = strdup(current)) == NULL) {
+		pmNoMem("initArchive: name", strlen(current) + 1, PM_FATAL_ERR);
 		/* NOTREACHED */
 	    }
-	    if ((mlcp->ml_hostname = strdup(label.ll_hostname)) == NULL) {
-		pmNoMem("initArchive", strlen(label.ll_hostname) + 1, PM_FATAL_ERR);
+	    if ((mlcp->hostname = strdup(lp->hostname)) == NULL) {
+		pmNoMem("initArchive: hostname", strlen(lp->hostname) + 1, PM_FATAL_ERR);
 		/* NOTREACHED */
 	    }
-	    if ((mlcp->ml_tz = strdup(label.ll_tz)) == NULL) {
-		pmNoMem("initArchive", strlen(label.ll_tz) + 1, PM_FATAL_ERR);
-		/* NOTREACHED */
+	    if (lp->timezone != NULL) {
+		if ((mlcp->timezone = strdup(lp->timezone)) == NULL) {
+		    pmNoMem("initArchive: timezone", strlen(lp->timezone) + 1, PM_FATAL_ERR);
+		    /* NOTREACHED */
+		}
 	    }
-	    mlcp->ml_starttime = tmpTime;
+	    else
+		mlcp->timezone = NULL;
+	    if (lp->zoneinfo != NULL) {
+		if ((mlcp->zoneinfo = strdup(lp->zoneinfo)) == NULL) {
+		    pmNoMem("initArchive: zoneinfo", strlen(lp->zoneinfo) + 1, PM_FATAL_ERR);
+		    /* NOTREACHED */
+		}
+	    }
+	    else
+		mlcp->zoneinfo = NULL;
+	    mlcp->starttime = tmpTime;
 
 	    /*
 	     * If we found the insertion point, then make room for the current
@@ -941,16 +943,15 @@ initarchive(__pmContext	*ctxp, const char *name)
 	sts = __pmLogChangeArchive(ctxp, 0);
 	if (sts < 0)
 	    goto error;
-	sts = __pmLogChangeVol(acp, acp->ac_log->l_minvol);
+	sts = __pmLogChangeVol(acp, acp->ac_log->minvol);
 	if (sts < 0)
 	    goto error;
     }
 
     /* start after header + label record + trailer */
-    ctxp->c_origin.tv_sec = (__int32_t)acp->ac_log->l_label.ill_start.tv_sec;
-    ctxp->c_origin.tv_usec = (__int32_t)acp->ac_log->l_label.ill_start.tv_usec;
+    ctxp->c_origin = acp->ac_log->label.start;
     ctxp->c_mode = (ctxp->c_mode & 0xffff0000) | PM_MODE_FORW;
-    acp->ac_offset = sizeof(__pmLogLabel) + 2*sizeof(int);
+    acp->ac_offset = __pmLogLabelSize(acp->ac_log);
     acp->ac_vol = acp->ac_curvol;
     acp->ac_serial = 0;		/* not serial access, yet */
     acp->ac_pmid_hc.nodes = 0;	/* empty hash list */
@@ -964,12 +965,14 @@ initarchive(__pmContext	*ctxp, const char *name)
 
  error:
     if (mlcp) {
-	if (mlcp->ml_name)
-	    free (mlcp->ml_name);
-	if (mlcp->ml_hostname)
-	    free (mlcp->ml_hostname);
-	if (mlcp->ml_tz)
-	    free (mlcp->ml_tz);
+	if (mlcp->name)
+	    free (mlcp->name);
+	if (mlcp->hostname)
+	    free (mlcp->hostname);
+	if (mlcp->timezone)
+	    free (mlcp->timezone);
+	if (mlcp->zoneinfo)
+	    free (mlcp->zoneinfo);
 	free(mlcp);
     }
     if (namelist)
@@ -978,15 +981,16 @@ initarchive(__pmContext	*ctxp, const char *name)
 	while (acp->ac_num_logs > 0) {
 	    --acp->ac_num_logs;
 	    if (acp->ac_log_list[acp->ac_num_logs]) {
-		free(acp->ac_log_list[acp->ac_num_logs]->ml_name);
-		free(acp->ac_log_list[acp->ac_num_logs]->ml_hostname);
-		free(acp->ac_log_list[acp->ac_num_logs]->ml_tz);
+		free(acp->ac_log_list[acp->ac_num_logs]->name);
+		free(acp->ac_log_list[acp->ac_num_logs]->hostname);
+		free(acp->ac_log_list[acp->ac_num_logs]->timezone);
+		free(acp->ac_log_list[acp->ac_num_logs]->zoneinfo);
 		free(acp->ac_log_list[acp->ac_num_logs]);
 	    }
 	}
 	free(acp->ac_log_list);
     }
-    if (acp->ac_log && --acp->ac_log->l_refcnt == 0)
+    if (acp->ac_log && --acp->ac_log->refcnt == 0)
 	free(acp->ac_log);
     free(acp);
     ctxp->c_archctl = NULL;
@@ -1098,7 +1102,7 @@ INIT_CONTEXT:
     /* c_lock not re-initialized, created once from initcontextlock() above */
     new->c_type = (type & PM_CONTEXT_TYPEMASK);
     new->c_mode = 0;
-    new->c_origin.tv_sec = new->c_origin.tv_usec = 0;
+    new->c_origin.sec = new->c_origin.nsec = 0;
     new->c_delta = 0;
     new->c_sent = 0;
     new->c_flags = (type & ~PM_CONTEXT_TYPEMASK);
@@ -1395,7 +1399,7 @@ pmDupContext(void)
     else if (oldcon->c_type == PM_CONTEXT_LOCAL)
 	new = pmNewContext(oldtype, NULL);
     else if (oldcon->c_type == PM_CONTEXT_ARCHIVE)
-	new = pmNewContext(oldtype, oldcon->c_archctl->ac_log->l_name);
+	new = pmNewContext(oldtype, oldcon->c_archctl->ac_log->name);
     if (new < 0) {
 	/* failed to connect or out of memory */
 	sts = new;
@@ -1508,25 +1512,33 @@ pmDupContext(void)
 		oldmlcp = oldcon->c_archctl->ac_log_list[i];
 		*newmlcp = *oldmlcp;
 		/*
-		 * We need to duplicate the ml_name and the ml_hostname of each
+		 * We need to duplicate the name and the hostname of each
 		 * archive in the list.
 		 */
-		if ((newmlcp->ml_name = strdup (newmlcp->ml_name)) == NULL) {
+		if ((newmlcp->name = strdup (newmlcp->name)) == NULL) {
 		    sts = -oserror();
 		    goto done_locked;
 		}
-		if ((newmlcp->ml_hostname = strdup (newmlcp->ml_hostname)) == NULL) {
+		if ((newmlcp->hostname = strdup (newmlcp->hostname)) == NULL) {
 		    sts = -oserror();
 		    goto done_locked;
 		}
-		if ((newmlcp->ml_tz = strdup (newmlcp->ml_tz)) == NULL) {
-		    sts = -oserror();
-		    goto done_locked;
+		if (newmlcp->timezone != NULL) {
+		    if ((newmlcp->timezone = strdup (newmlcp->timezone)) == NULL) {
+			sts = -oserror();
+			goto done_locked;
+		    }
+		}
+		if (newmlcp->zoneinfo != NULL) {
+		    if ((newmlcp->zoneinfo = strdup (newmlcp->zoneinfo)) == NULL) {
+			sts = -oserror();
+			goto done_locked;
+		    }
 		}
 	    }
 	    /* We need to bump up the reference count of the ac_log. */
 	    if (newcon->c_archctl->ac_log != NULL)
-		++newcon->c_archctl->ac_log->l_refcnt;
+		++newcon->c_archctl->ac_log->refcnt;
 	}
     }
 
@@ -1775,21 +1787,21 @@ __pmDumpContext(FILE *f, int context, pmInDom indom)
 		fprintf(f, " handle %d:", contexts_map[i]);
 		for (j = 0; j < con->c_archctl->ac_num_logs; j++) {
 		    fprintf(f, " log %s:",
-			    con->c_archctl->ac_log_list[j]->ml_name);
+			    con->c_archctl->ac_log_list[j]->name);
 		    if (con->c_archctl->ac_log == NULL ||
-			con->c_archctl->ac_log->l_refcnt == 0 ||
-			strcmp (con->c_archctl->ac_log_list[j]->ml_name,
-				con->c_archctl->ac_log->l_name) != 0) {
+			con->c_archctl->ac_log->refcnt == 0 ||
+			strcmp (con->c_archctl->ac_log_list[j]->name,
+				con->c_archctl->ac_log->name) != 0) {
 			fprintf(f, " not open\n");
 			continue;
 		    }
 		    fprintf(f, " mode=%s", _mode[con->c_mode & __PM_MODE_MASK]);
 		    fprintf(f, " profile=%s tifd=%d mdfd=%d mfd=%d\nrefcnt=%d vol=%d",
 			    con->c_sent ? "SENT" : "NOT_SENT",
-			    con->c_archctl->ac_log->l_tifp == NULL ? -1 : __pmFileno(con->c_archctl->ac_log->l_tifp),
-			    __pmFileno(con->c_archctl->ac_log->l_mdfp),
+			    con->c_archctl->ac_log->tifp == NULL ? -1 : __pmFileno(con->c_archctl->ac_log->tifp),
+			    __pmFileno(con->c_archctl->ac_log->mdfp),
 			    __pmFileno(con->c_archctl->ac_mfp),
-			    con->c_archctl->ac_log->l_refcnt,
+			    con->c_archctl->ac_log->refcnt,
 			    con->c_archctl->ac_curvol);
 		    fprintf(f, " offset=%ld (vol=%d) serial=%d",
 			    (long)con->c_archctl->ac_offset,
@@ -1798,8 +1810,8 @@ __pmDumpContext(FILE *f, int context, pmInDom indom)
 		}
 	    }
 	    if (con->c_type == PM_CONTEXT_HOST || con->c_type == PM_CONTEXT_ARCHIVE) {
-		fprintf(f, " origin=%d.%06d",
-		    con->c_origin.tv_sec, con->c_origin.tv_usec);
+		fprintf(f, " origin=%" FMT_INT64 ".%09d",
+		    con->c_origin.sec, con->c_origin.nsec);
 		fprintf(f, " delta=%d\n", con->c_delta);
 	    }
 	    __pmDumpProfile(f, indom, con->c_instprof);
@@ -1827,7 +1839,7 @@ __pmIsContextLock(void *lock)
 }
 
 /*
- * return list of context handles if lock == l_lock for an associated
+ * return list of context handles if lock == lc_lock for an associated
  * __pmLogCtl ... no locking here to avoid recursion ad nauseum
  */
 char *
@@ -1850,7 +1862,7 @@ __pmIsLogCtlLock(void *lock)
 	if (ctxp->c_archctl->ac_log == NULL)
 	    /* this should not happen, just being careful */
 	    continue;
-	if ((void *)&ctxp->c_archctl->ac_log->l_lock == lock) {
+	if ((void *)&ctxp->c_archctl->ac_log->lc_lock == lock) {
 	    char	number[10];
 	    pmsprintf(number, sizeof(number), "%d", ctxp->c_handle);
 	    if (reslen == 0) {
