@@ -890,14 +890,14 @@ eval_expr(__pmContext *ctxp, node_t *np, struct timespec *stamp, int numpmid,
 			     * scaling factor (to scale metric from counter
 			     * units into seconds)
 			     */
-			    int		i;
+			    int		n;
 			    np->data.info->time_scale = 1;
 			    if (np->left->desc.units.scaleTime > PM_TIME_SEC) {
-				for (i = PM_TIME_SEC; i < np->left->desc.units.scaleTime; i++)
+				for (n = PM_TIME_SEC; n < np->left->desc.units.scaleTime; n++)
 				    np->data.info->time_scale *= 60;
 			    }
 			    else {
-				for (i = np->left->desc.units.scaleTime; i < PM_TIME_SEC; i++)
+				for (n = np->left->desc.units.scaleTime; n < PM_TIME_SEC; n++)
 				    np->data.info->time_scale /= 1000;
 			    }
 			}
@@ -1503,7 +1503,7 @@ eval_expr(__pmContext *ctxp, node_t *np, struct timespec *stamp, int numpmid,
 		    char		*q;
 
 		    if ((hp = __pmHashSearch(np->right->data.info->ivlist[i].inst, &np->left->data.pattern->hash)) == NULL) {
-			pmTimeval	save_origin;
+			__pmTimestamp	save_origin;
 
 			/* first time we've seen this inst for this expr node */
 			if ((ip = (instctl_t *)malloc(sizeof(instctl_t))) == NULL) {
@@ -1519,8 +1519,8 @@ eval_expr(__pmContext *ctxp, node_t *np, struct timespec *stamp, int numpmid,
 			     * back.
 			     */
 			    save_origin = ctxp->c_origin;	/* struct assignment */
-			    ctxp->c_origin.tv_sec = stamp->tv_sec;
-			    ctxp->c_origin.tv_usec = stamp->tv_nsec / 1000;
+			    ctxp->c_origin.sec = stamp->tv_sec;
+			    ctxp->c_origin.nsec = stamp->tv_nsec;
 			}
 			sts = pmNameInDom_ctx(ctxp, np->right->desc.indom, ip->inst, &iname);
 			if (ctxp->c_type == PM_CONTEXT_ARCHIVE)
@@ -2045,8 +2045,14 @@ void
 __dmpostfetch(__pmContext *ctxp, pmResult **result)
 {
     struct timespec	timestamp;
-    pmResult		*newrp, *rp = *result;
-    size_t		need;
+#if 0	// TODO when result -> __pmResult
+    __pmResult		*newrp;
+    __pmResult		*rp = *result;
+#else
+    pmResult		*newrp;
+    pmResult		*rp = *result;
+    __pmResult		*__newrp;
+#endif
     ctl_t		*cp = (ctl_t *)ctxp->c_dm;
     int			fails;
 
@@ -2054,11 +2060,11 @@ __dmpostfetch(__pmContext *ctxp, pmResult **result)
     if (cp == NULL || cp->fetch_has_dm == 0)
 	return;
 
-    need = sizeof(pmResult) + (cp->numpmid - 1) * sizeof(pmValueSet *);
-    if ((newrp = (pmResult *)malloc(need)) == NULL) {
-	pmNoMem("__dmpostfetch: newrp", need, PM_FATAL_ERR);
-	/*NOTREACHED*/
+    if ((__newrp = __pmAllocResult(cp->numpmid)) == NULL) {
+	pmNoMem("__dmpostfetch: newrp", sizeof(__pmResult) + (cp->numpmid - 1) * sizeof(pmValueSet *), PM_FATAL_ERR);
+	/* NOTREACHED */
     }
+    newrp = __pmOffsetResult(__newrp);
     newrp->numpmid = cp->numpmid;
     newrp->timestamp = rp->timestamp;
 
