@@ -192,18 +192,12 @@ pmfg_lookup_item(const char *metric, const char *instance, pmFGI item)
 	(item->u.item.metric_desc.indom != PM_INDOM_NULL && !instance))
 	return PM_ERR_INDOM;
 
-    /* Clear instance profile if this is an opaque derived metric. */
-    if (IS_DERIVED(item->u.item.metric_desc.pmid)) {
-        (void) pmAddProfile(PM_INDOM_NULL, 0, NULL);
-    }
-
     /* Add given instance to profile. */
     if (item->u.item.metric_desc.indom != PM_INDOM_NULL) {
 	sts = pmLookupInDom(item->u.item.metric_desc.indom, instance);
 	if (sts < 0)
 	    return sts;
 	item->u.item.metric_inst = sts;
-        /* Moot & harmless if IS_DERIVED. */
 	sts = pmAddProfile(item->u.item.metric_desc.indom, 1,
 			   &item->u.item.metric_inst);
     }
@@ -226,11 +220,6 @@ pmfg_lookup_indom(const char *metric, pmFGI item)
     if (sts < 0)
 	return sts;
 
-    /* Clear instance profile if this is an opaque derived metric. */
-    if (IS_DERIVED(item->u.item.metric_desc.pmid)) {
-        (void) pmAddProfile(PM_INDOM_NULL, 0, NULL);
-    }
-
     /* As a convenience to users, we also accept non-indom'd metrics */
     if (item->u.indom.metric_desc.indom == PM_INDOM_NULL)
 	return 0;
@@ -238,7 +227,6 @@ pmfg_lookup_indom(const char *metric, pmFGI item)
     /*
      * Add all instances; this will override any other past or future
      * piecemeal instance requests from __pmExtendFetchGroup_lookup.
-     * Moot & harmless if IS_DERIVED.
      */
     return pmAddProfile(item->u.indom.metric_desc.indom, 0, NULL);
 }
@@ -509,13 +497,13 @@ __pmStuffDoubleValue(double val, pmAtomValue *oval, int otype)
 		oval->ul = val;
 	    break;
 	case PM_TYPE_64:
-	    if (val > LLONG_MAX || val < LLONG_MIN)
+	    if (val > (double)LLONG_MAX || val < (double)LLONG_MIN)
 		sts = PM_ERR_TRUNC;
 	    else
 		oval->ll = val;
 	    break;
 	case PM_TYPE_U64:
-	    if (val > ULLONG_MAX)
+	    if (val > (double)ULLONG_MAX)
 		sts = PM_ERR_TRUNC;
 	    else if (val < 0.0)
 		sts = PM_ERR_SIGN;
@@ -1214,17 +1202,25 @@ static int
 pmfg_clear_profile(pmFG pmfg)
 {
     int sts;
+    pmFGI item;
 
     sts = pmUseContext(pmfg->ctx);
     if (sts != 0)
 	return sts;
 
     /*
-     * Wipe clean all instances; we'll add them back incrementally as
-     * the fetchgroup is extended.  This cannot fail for the manner in
-     * which we call it - see pmDelProfile(3) man page discussion.
+     * Wipe clean all instances profiles; we'll add them back incrementally
+     * as the fetchgroup is extended.
+     * Walk the fetchgroup, just clearing the profile for any indoms
+     * that are not PM_INDOM_NULL.
+     * Any errors here are benign (or rather there's nothing we can do
+     * about 'em), so ignore pmDelProfile() return value.
      */
-    return pmDelProfile(PM_INDOM_NULL, 0, NULL);
+    for (item = pmfg->items; item; item = item->next) {
+	if (item->u.item.metric_desc.indom != PM_INDOM_NULL)
+	    (void)pmDelProfile(item->u.item.metric_desc.indom, 0, NULL);
+    }
+    return 0;
 }
 
 
@@ -1318,18 +1314,30 @@ pmExtendFetchGroup_item(pmFG pmfg,
 
 	if (ctxp->c_type == PM_CONTEXT_ARCHIVE) {
 	    struct timeval saved_origin;
+#if 0	// when pmSetHighResMode() exists
+	    struct timespec archive_end;
+#else
 	    struct timeval archive_end;
+#endif
+	    __pmTimestamp	stamp;
 	    int saved_mode, saved_delta;
 
-	    saved_origin.tv_sec = ctxp->c_origin.tv_sec;
-	    saved_origin.tv_usec = ctxp->c_origin.tv_usec;
+	    saved_origin.tv_sec = ctxp->c_origin.sec;
+	    saved_origin.tv_usec = ctxp->c_origin.nsec / 1000;
 	    saved_mode = ctxp->c_mode;
 	    saved_delta = ctxp->c_delta;
-	    sts = pmGetArchiveEnd_ctx(ctxp, &archive_end);
+	    sts = pmGetArchiveEnd_ctx(ctxp, &stamp);
 	    PM_UNLOCK(ctxp->c_lock);
 	    if (sts < 0)
 		goto out;
+	    archive_end.tv_sec = stamp.sec;
+#if 0	// when pmSetHighResMode() exists
+	    archive_end.tv_nsec = stamp.nsec;
+	    sts = pmSetHighResMode(PM_MODE_BACK, &archive_end, 0);
+#else
+	    archive_end.tv_usec = stamp.nsec / 1000;
 	    sts = pmSetMode(PM_MODE_BACK, &archive_end, 0);
+#endif
 	    if (sts < 0)
 		goto out;
 	    /* try again */
@@ -1499,19 +1507,32 @@ pmExtendFetchGroup_event(pmFG pmfg,
 	__pmContext *ctxp = __pmHandleToPtr(pmfg->ctx);
 
 	if (ctxp->c_type == PM_CONTEXT_ARCHIVE) {
-	    struct timeval saved_origin;
+#if 0	// when pmSetHighResMode() exists
+	    struct timespec archive_end;
+	    struct timespec saved_origin;
+#else
 	    struct timeval archive_end;
+	    struct timeval saved_origin;
+#endif
+	    __pmTimestamp	stamp;
 	    int saved_mode, saved_delta;
 
-	    saved_origin.tv_sec = ctxp->c_origin.tv_sec;
-	    saved_origin.tv_usec = ctxp->c_origin.tv_usec;
+	    saved_origin.tv_sec = ctxp->c_origin.sec;
+	    saved_origin.tv_usec = ctxp->c_origin.nsec / 1000;
 	    saved_mode = ctxp->c_mode;
 	    saved_delta = ctxp->c_delta;
-	    sts = pmGetArchiveEnd_ctx(ctxp, &archive_end);
+	    sts = pmGetArchiveEnd_ctx(ctxp, &stamp);
 	    PM_UNLOCK(ctxp->c_lock);
 	    if (sts < 0)
 		goto out;
+	    archive_end.tv_sec = stamp.sec;
+#if 0	// when pmSetHighResMode() exists
+	    archive_end.tv_nsec = stamp.nsec;
+	    sts = pmSetHighResMode(PM_MODE_BACK, &archive_end, 0);
+#else
+	    archive_end.tv_usec = stamp.nsec / 1000;
 	    sts = pmSetMode(PM_MODE_BACK, &archive_end, 0);
+#endif
 	    if (sts < 0)
 		goto out;
 	    /* try again */
