@@ -18,7 +18,7 @@
 #include "libpcp.h"
 #include "pmda.h"
 #include "internal.h"
-#include "jsonsl.h"
+#include <jsonsl/jsonsl.h>
 #include "sha256.h"
 #include "sort_r.h"
 #include "fault.h"
@@ -29,10 +29,10 @@
 static int __pmMergeLabels(const char *, const char *, char *, int, int);
 static int __pmParseLabels(const char *, int, pmLabel *, int, __pmHashCtl *, int, char *, int *);
 
-typedef int (*filter)(const pmLabel *, const char *, void *);
+typedef int (*filter_labels)(const pmLabel *, const char *, void *);
 static int __pmMergeLabelSets(pmLabel *, const char *, __pmHashCtl *, int,
 		pmLabel *, const char *, __pmHashCtl *, int,
-		pmLabel *, char *, int *, int, filter, void *);
+		pmLabel *, char *, int *, int, filter_labels, void *);
 
 /*
  * Parse JSONB labels string, building up the array index as we go.
@@ -183,6 +183,9 @@ pmFreeLabelSets(pmLabelSet *sets, int nsets)
 {
     int		i;
 
+    if (sets == NULL)
+	return;
+
     for (i = 0; i < nsets; i++) {
 	if (sets[i].nlabels > 0)
 	    free(sets[i].labels);
@@ -195,10 +198,7 @@ pmFreeLabelSets(pmLabelSet *sets, int nsets)
 	    sets[i].hash = NULL;
 	}
     }
-    if (sets != NULL) {
-	assert(nsets > 0);
-	free(sets);
-    }
+    free(sets);
 }
 
 pmLabelSet *
@@ -434,7 +434,7 @@ static int
 stash_label(const pmLabel *lp, const char *json, __pmHashCtl *lc,
 	    pmLabel *olabel, const char *obuffer, int *no,
 	    char **buffer, int *buflen,
-	    filter filter, void *arg)
+	    filter_labels filter, void *arg)
 {
     const char		*name;
     char		*bp = *buffer;
@@ -846,22 +846,23 @@ static int
 __pmMergeLabelSets(pmLabel *alabels, const char *abuf, __pmHashCtl *ac, int na,
 		   pmLabel *blabels, const char *bbuf, __pmHashCtl *bc, int nb,
 		   pmLabel *olabels, char *output, int *no, int buflen,
-		   filter filter, void *arg)
+		   filter_labels filter, void *arg)
 {
     char		*bp = output;
     int			sts, i, j;
 
-    if (no)
-	*no = 0;	/* number of output labels */
-
     /* integrity check */
-    if ((na > 0 && alabels == NULL) || (nb > 0 && blabels == NULL)) {
+    if ((na > 0 && alabels == NULL) || (nb > 0 && blabels == NULL) ||
+        (olabels == NULL && no != NULL) || (olabels != NULL && no == NULL)) {
 	if (pmDebugOptions.labels)
 	    fprintf(stderr, "__pmMergeLabelSets: invalid or corrupt arguments\n");
 	sts = -EINVAL;
 	goto done;
     }
 
+    if (no)
+	*no = 0;	/* number of output labels */
+
     /* Walk over both label sets inserting all names into the output
      * buffer, but prefering b-group values over those in the a-group.
      * As we go, check for duplicates between a-group & b-group (since
@@ -968,7 +969,7 @@ __pmMergeLabels(const char *a, const char *b, char *buffer, int buflen, int flag
  */
 int
 pmMergeLabelSets(pmLabelSet **sets, int nsets, char *buffer, int buflen,
-		filter filter, void *arg)
+		filter_labels filter, void *arg)
 {
     __pmHashCtl		*compound, bhash = {0};
     pmLabel		olabels[MAXLABELSET];
@@ -1123,12 +1124,12 @@ archive_host_labels(__pmContext *ctxp, char *buffer, int buflen)
     /*
      * Backward compatibility fallback, for archives created before
      * labels support is added to pmlogger.
-     * Once that's implemented (TYPE_LABEL in .meta) fields will be
+     * Once that's implemented (TYPE_LABEL_V2 in .meta) fields will be
      * added to the context structure and we'll be able to read 'em
      * here to provide complete archive label support.
      */
     pmsprintf(buffer, buflen, "{\"hostname\":\"%s\"}",
-	     ctxp->c_archctl->ac_log->l_label.ill_hostname);
+		ctxp->c_archctl->ac_log->label.hostname);
     buffer[buflen-1] = '\0';
     return buffer;
 }
@@ -1368,7 +1369,7 @@ getlabels(int ident, int type, pmLabelSet **sets, int *nsets)
 		    sts = __pmMapErrno(sts);
 		else {
 		    int x_ident = ident, x_type = type;
-		    PM_FAULT_POINT("libpcp/" __FILE__ ":1", PM_FAULT_TIMEOUT);
+		    PM_FAULT_POINT("libpcp/" __FILE__ ":1", PM_FAULT_CALL);
 		    sts = __pmRecvLabel(fd, ctxp, tout, &x_ident, &x_type, sets, nsets);
 		}
 	    }
@@ -1406,8 +1407,9 @@ getlabels(int ident, int type, pmLabelSet **sets, int *nsets)
 	}
     }
     else {
-	if ((sts = __pmLogLookupLabel(ctxp->c_archctl, type,
-				ident, sets, &ctxp->c_origin)) < 0) {
+	sts = __pmLogLookupLabel(ctxp->c_archctl, type, ident, sets, &ctxp->c_origin);
+
+	if (sts < 0) {
 	    /* supply context labels for archives lacking label support */
 	    if (type & PM_LABEL_CONTEXT)
 		sts = archive_context_labels(ctxp, sets);
