@@ -1,7 +1,7 @@
 /*
  * General Utility Routines
  *
- * Copyright (c) 2012-2018 Red Hat.
+ * Copyright (c) 2012-2018,2021 Red Hat.
  * Copyright (c) 2009 Aconex.  All Rights Reserved.
  * Copyright (c) 1995-2002,2004 Silicon Graphics, Inc.  All Rights Reserved.
  *
@@ -745,7 +745,6 @@ dump_valueset(__pmContext *ctxp, FILE *f, pmValueSet *vsp)
     if ((n = pmNameAll_ctx(ctxp, vsp->pmid, &names)) < 0)
 	fprintf(f, "  %s (%s):", pmid, "<noname>");
     else {
-	int	j;
 	fprintf(f, "  %s (", pmid);
 	for (j = 0; j < n; j++) {
 	    if (j > 0)
@@ -829,6 +828,34 @@ __pmDumpResult(FILE *f, const pmResult *resp)
     __pmDumpResult_ctx(NULL, f, resp);
 }
 
+/* Internal variant of __pmPrintResult() with current context. */
+void
+__pmPrintResult_ctx(__pmContext *ctxp, FILE *f, const __pmResult *resp)
+{
+    int		i;
+    struct timespec ts;
+
+    if (ctxp != NULL)
+	PM_ASSERT_IS_LOCKED(ctxp->c_lock);
+
+    save_debug();
+    fprintf(f, "__pmResult dump from " PRINTF_P_PFX "%p timestamp: %" FMT_INT64 ".%06d ",
+	resp, resp->timestamp.sec, resp->timestamp.nsec);
+    ts.tv_sec = resp->timestamp.sec;
+    ts.tv_nsec = resp->timestamp.nsec;
+    pmPrintHighResStamp(f, &ts);
+    fprintf(f, " numpmid: %d\n", resp->numpmid);
+    for (i = 0; i < resp->numpmid; i++)
+	dump_valueset(ctxp, f, resp->vset[i]);
+    restore_debug();
+}
+
+void
+__pmPrintResult(FILE *f, const __pmResult *resp)
+{
+    __pmPrintResult_ctx(NULL, f, resp);
+}
+
 /* Internal variant of __pmDumpHighResResult() with current context. */
 void
 __pmDumpHighResResult_ctx(__pmContext *ctxp, FILE *f, const pmHighResResult *hresp)
@@ -1220,6 +1247,20 @@ __pmTimevalCmp(const pmTimeval *a, const pmTimeval *b)
     return res;
 }
 
+/*
+ * a : b for __pmTimestamp ... <0 for a<b, ==0 for a==b, >0 for a>b
+ */
+int
+__pmTimestampCmp(const __pmTimestamp *a, const __pmTimestamp *b)
+{
+    int res = (int)(a->sec - b->sec);
+
+    if (res == 0)
+	res = (int)(a->nsec - b->nsec);
+
+    return res;
+}
+
 /*
  * Difference for two of the internal timestamps ...
  * Same as pmtimevalSub() in tv.c, just with pmTimeval args
@@ -1231,6 +1272,56 @@ __pmTimevalSub(const pmTimeval *ap, const pmTimeval *bp)
      return (double)(ap->tv_sec - bp->tv_sec + (long double)(ap->tv_usec - bp->tv_usec) / (long double)1000000);
 }
 
+/*
+ * Difference for two of the internal highres timestamps ...
+ * Same as pmtimespecSub() in tv.c, just with pmTimespec args
+ * rather than struct timespec args.
+ */
+double
+__pmTimespecSub(const pmTimespec *ap, const pmTimespec *bp)
+{
+     return (double)(ap->tv_sec - bp->tv_sec + (long double)(ap->tv_nsec - bp->tv_nsec) / (long double)1000000000);
+}
+
+/*
+ * Difference for two of the universal timestamps ...
+ * Same as pmtimespecSub() in tv.c, just with __pmTimestamp args
+ * rather than struct timespec args.
+ */
+double
+__pmTimestampSub(const __pmTimestamp *ap, const __pmTimestamp *bp)
+{
+     return (double)(ap->sec - bp->sec + (long double)(ap->nsec - bp->nsec) / (long double)1000000000);
+}
+
+/*
+ * Increment a universal timestamps ...
+ */
+void
+__pmTimestampInc(__pmTimestamp *ap, const __pmTimestamp *bp)
+{
+     ap->sec += bp->sec;
+     ap->nsec += bp->nsec;
+     if (ap->nsec > 1000000000) {
+	 ap->sec++;
+	 ap->nsec -= 1000000000;
+    }
+}
+
+/*
+ * Decrement a universal timestamps ...
+ */
+void
+__pmTimestampDec(__pmTimestamp *ap, const __pmTimestamp *bp)
+{
+     ap->sec -= bp->sec;
+     ap->nsec -= bp->nsec;
+     if (ap->nsec < 0) {
+	 ap->sec--;
+	 ap->nsec += 1000000000;
+    }
+}
+
 /*
  * print timeval timestamp in HH:MM:SS.XXX format
  * Note: to minimize ABI surprises, this one still reports to msec precision,
@@ -1264,7 +1355,7 @@ pmPrintHighResStamp(FILE *f, const struct timespec *tp)
 
 /*
  * print pmTimeval timestamp in HH:MM:SS.XXXXXX format (usec precision)
- * (pmTimeval variant used in PDUs, archives and internally)
+ * (pmTimeval variant used in historical PDUs, archives and internally)
  */
 void
 __pmPrintTimeval(FILE *f, const pmTimeval *tp)
@@ -1277,6 +1368,35 @@ __pmPrintTimeval(FILE *f, const pmTimeval *tp)
     fprintf(f, "%02d:%02d:%02d.%06d", tmp.tm_hour, tmp.tm_min, tmp.tm_sec, (int)tp->tv_usec);
 }
 
+/*
+ * print pmTimespec timestamp in HH:MM:SS.XXXXXXXXX format (nsec precision)
+ * (pmTimespec variant used in the latest PDUs, archives and internally)
+ */
+void
+__pmPrintTimespec(FILE *f, const pmTimespec *tp)
+{
+    struct tm	tmp;
+    time_t	now;
+
+    now = (time_t)tp->tv_sec;
+    pmLocaltime(&now, &tmp);
+    fprintf(f, "%02d:%02d:%02d.%09d", tmp.tm_hour, tmp.tm_min, tmp.tm_sec, (int)tp->tv_nsec);
+}
+
+/*
+ * print universal __pmTimestamp timestamp in HH:MM:SS.XXXXXXXXX format
+ */
+void
+__pmPrintTimestamp(FILE *f, const __pmTimestamp *tsp)
+{
+    struct tm	tmp;
+    time_t	now;
+
+    now = (time_t)tsp->sec;
+    pmLocaltime(&now, &tmp);
+    fprintf(f, "%02d:%02d:%02d.%09d", tmp.tm_hour, tmp.tm_min, tmp.tm_sec, (int)tsp->nsec);
+}
+
 /*
  * must be in agreement with ordinal values for PM_TYPE_* #defines
  */
