@@ -294,14 +294,12 @@ dump(FILE *fp, hdr_t *h, int do_hash)
     }
 
     if (h->ctl_inst != NULL) {
-	int	i;
 	fprintf(fp, "inst hash\n");
 	for (i = 0; i < h->hsize; i++) {
 	    dump_hash_list(fp, h, 0, i);
 	}
     }
     if (h->ctl_name != NULL) {
-	int	i;
 	fprintf(fp, "name hash\n");
 	for (i = 0; i < h->hsize; i++) {
 	    dump_hash_list(fp, h, 1, i);
@@ -330,6 +328,9 @@ find_inst(hdr_t *h, int inst)
 {
     entry_t	*e;
 
+    if ((e = h->last) != NULL && e->inst < inst)
+	return NULL;
+
     for (e = h->first; e != NULL; e = e->next) {
 	if (e->inst == inst && e->state != PMDA_CACHE_EMPTY)
 	    break;
@@ -382,7 +383,7 @@ find_entry(hdr_t *h, const char *name, int inst, int *sts)
 /*
  * optionally resize the hash table first (if resize == 1)
  *
- * then re-order each hash chain so that active entires are
+ * then re-order each hash chain so that active entries are
  * before inactive entries, and culled entries dropped
  *
  * applies to _both_ the inst and name hashes
@@ -623,7 +624,11 @@ insert_cache(hdr_t *h, const char *name, int inst, int *sts)
 	    *sts = PM_ERR_INST;
 	    return e;
 	}
-	for (e = h->first; e != NULL; e = e->next) {
+	/* if this entry is beyond the (sorted) list end, avoid linear scan */
+	if ((e = h->last) == NULL || e->inst > inst)
+	    e = h->first;
+	/* linear search over linked list, starting at either first or last */
+	for (; e != NULL; e = e->next) {
 	    if (e->inst < inst)
 		last_e = e;
 	    else if (e->inst > inst)
@@ -713,7 +718,11 @@ retry:
 	h->last = e;
     h->nentry++;
 
-    if (h->hsize > 0 && h->hsize < 1024 && h->nentry > 4 * h->hsize)
+    /*
+     * try to keep hash table sized so there are on average no
+     * more than 4 entries per hash chain
+     */
+    if (h->hsize > 0 && h->nentry > 4 * h->hsize)
 	redo_hash(h, 1);
 
     /* link into the inst hash list, if any */
@@ -759,7 +768,10 @@ load_cache(hdr_t *h)
 	    return PM_ERR_GENERIC;
 	pmsprintf(filename, sizeof(filename),
 		"%s%c" "config" "%c" "pmda", vdp, sep, sep);
-	mkdir2(filename, 0755);
+	if (mkdir2(filename, 0755) < 0) {
+	    /* failure here is not fatal ... dir may already exist */
+	    ;
+	}
     }
 
     pmsprintf(filename, sizeof(filename), "%s%cconfig%cpmda%c%s",
@@ -920,7 +932,10 @@ save_cache(hdr_t *h, int hstate)
 	    return PM_ERR_GENERIC;
 	pmsprintf(filename, sizeof(filename),
 		"%s%c" "config" "%c" "pmda", vdp, sep, sep);
-	mkdir2(filename, 0755);
+	if (mkdir2(filename, 0755) < 0) {
+	    /* failure here is not fatal ... dir may already exist */
+	    ;
+	}
     }
 
     pmsprintf(filename, sizeof(filename), "%s%cconfig%cpmda%c%s",
