@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2017-2020 Red Hat.
+ * Copyright (c) 2017-2021 Red Hat.
  *
  * This library is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -162,7 +162,7 @@ new_metric(seriesLoadBaton *baton, pmValueSet *vsp)
     pmDesc		desc;
     char		errmsg[PM_MAXERRMSGLEN], idbuf[64];
     char		**nameall = NULL;
-    int			count, sts, i;
+    int			count = 0, sts, i;
 
     if ((sts = pmUseContext(context->context)) < 0) {
 	fprintf(stderr, "%s: failed to use context for PMID %s: %s\n",
@@ -189,7 +189,10 @@ new_metric(seriesLoadBaton *baton, pmValueSet *vsp)
     if (sts < 0)
 	return NULL;
 
-    if ((metric = pmwebapi_new_metric(context, NULL, &desc, count, nameall)) == NULL)
+    metric = pmwebapi_new_metric(context, NULL, &desc, count, nameall);
+    if (count > 0 && nameall != NULL)
+	free(nameall);
+    if (metric == NULL)
 	return NULL;
     get_metric_metadata(baton, metric);
 
@@ -901,6 +904,7 @@ connect_redis_source_service(seriesLoadBaton *baton)
     pmSeriesModule	*module = (pmSeriesModule *)baton->module;
     seriesModuleData	*data = getSeriesModuleData(module);
     redisSlotsFlags	flags;
+    sds			option;
 
     /* attempt to re-use existing slots connections */
     if (data == NULL) {
@@ -909,26 +913,41 @@ connect_redis_source_service(seriesLoadBaton *baton)
 	baton->slots = data->slots;
 	series_load_end_phase(baton);
     } else {
-	flags = SLOTS_VERSION;
-	if ((baton->flags & PM_SERIES_FLAG_TEXT))
-	    flags |= SLOTS_SEARCH;
-	baton->slots = data->slots =
-	    redisSlotsConnect(
-		data->config, flags, baton->info,
-		series_load_end_phase, baton->userdata,
-		data->events, (void *)baton);
+	option = pmIniFileLookup(data->config, "redis", "enabled");
+	if (option && strcmp(option, "false") == 0) {
+	    baton->error = -ENOTSUP;
+	} else {
+	    flags = SLOTS_VERSION;
+	    if ((baton->flags & PM_SERIES_FLAG_TEXT))
+		flags |= SLOTS_SEARCH;
+	    baton->slots = data->slots =
+		redisSlotsConnect(
+		    data->config, flags, baton->info,
+		    series_load_end_phase, baton->userdata,
+		    data->events, (void *)baton);
+	}
     }
 }
 
 static void
-setup_source_services(void *arg)
+setup_pmapi_source_service(void *arg)
 {
     seriesLoadBaton	*baton = (seriesLoadBaton *)arg;
 
-    seriesBatonCheckMagic(baton, MAGIC_LOAD, "setup_source_services");
-    seriesBatonReferences(baton, 2, "setup_source_services");
+    seriesBatonCheckMagic(baton, MAGIC_LOAD, "setup_pmapi_source_service");
+    seriesBatonReferences(baton, 1, "setup_pmapi_source_service");
 
     connect_pmapi_source_service(baton);
+}
+
+static void
+setup_redis_source_service(void *arg)
+{
+    seriesLoadBaton	*baton = (seriesLoadBaton *)arg;
+
+    seriesBatonCheckMagic(baton, MAGIC_LOAD, "setup_redis_source_service");
+    seriesBatonReferences(baton, 1, "setup_redis_source_service");
+
     connect_redis_source_service(baton);
 }
 
@@ -1011,7 +1030,9 @@ series_load(pmSeriesSettings *settings,
     /* ordering of async operations */
     i = 0;
     baton->current = &baton->phases[i];
-    baton->phases[i++].func = setup_source_services;
+    /* Coverity CID341699 - split pmapi and redis setup to avoid use after free */
+    baton->phases[i++].func = setup_pmapi_source_service;
+    baton->phases[i++].func = setup_redis_source_service;
     /* assign source/host string map (series_source_mapping) */
     baton->phases[i++].func = series_source_mapping;
     /* write source info into schema (series_cache_source) */
@@ -1057,7 +1078,7 @@ pmSeriesDiscoverSource(pmDiscoverEvent *event, void *arg)
     sds			msg;
     int			i;
 
-    if (data == NULL || data->slots == NULL || data->slots->setup == 0)
+    if (data == NULL || data->slots == NULL || data->slots->state != SLOTS_READY)
 	return;
 
     baton = (seriesLoadBaton *)calloc(1, sizeof(seriesLoadBaton));
@@ -1138,7 +1159,7 @@ pmSeriesDiscoverLabels(pmDiscoverEvent *event,
     sds			msg;
     int			i, id;
 
-    if (baton == NULL || baton->slots == NULL || baton->slots->setup == 0)
+    if (baton == NULL || baton->slots == NULL || baton->slots->state != SLOTS_READY)
 	return;
 
     switch (type) {
@@ -1277,7 +1298,7 @@ pmSeriesDiscoverMetric(pmDiscoverEvent *event,
 			i + 1, numnames, pmIDStr(desc->pmid), names[i]);
     }
 
-    if (baton == NULL || baton->slots == NULL || baton->slots->setup == 0)
+    if (baton == NULL || baton->slots == NULL || baton->slots->state != SLOTS_READY)
 	return;
 
     if ((metric = pmwebapi_add_metric(&baton->pmapi.context,
@@ -1300,7 +1321,13 @@ pmSeriesDiscoverValues(pmDiscoverEvent *event, pmResult *result, void *arg)
     if (pmDebugOptions.discovery)
 	fprintf(stderr, "%s: result numpmids=%d\n", "pmSeriesDiscoverValues", result->numpmid);
 
-    if (baton == NULL || baton->slots == NULL || baton->slots->setup == 0)
+    if (data == NULL) {
+	/* calloc failed in getDiscoverModuleData() */
+    	baton->error = -ENOMEM;
+	return;
+    }
+
+    if (baton == NULL || baton->slots == NULL || baton->slots->state != SLOTS_READY)
 	return;
 
     seriesBatonReference(context, "pmSeriesDiscoverValues");
@@ -1325,19 +1352,21 @@ pmSeriesDiscoverInDom(pmDiscoverEvent *event, pmInResult *in, void *arg)
     if (pmDebugOptions.discovery)
 	fprintf(stderr, "%s: %s\n", "pmSeriesDiscoverInDom", pmInDomStr(id));
 
-    if (baton == NULL || baton->slots == NULL || baton->slots->setup == 0)
+    if (baton == NULL || baton->slots == NULL || baton->slots->state != SLOTS_READY)
 	return;
 
     if ((domain = pmwebapi_add_domain(context, pmInDom_domain(id))) == NULL) {
 	infofmt(msg, "%s: failed indom discovery (domain %u)",
 			"pmSeriesDiscoverInDom", pmInDom_domain(id));
 	moduleinfo(event->module, PMLOG_ERROR, msg, arg);
+	baton->error = -ENOMEM;
 	return;
     }
     if ((indom = pmwebapi_add_indom(context, domain, id)) == NULL) {
 	infofmt(msg, "%s: failed indom discovery (indom %s)",
 			"pmSeriesDiscoverInDom", pmInDomStr(id));
 	moduleinfo(event->module, PMLOG_ERROR, msg, arg);
+	baton->error = -ENOMEM;
 	return;
     }
     for (i = 0; i < in->numinst; i++) {
@@ -1347,6 +1376,8 @@ pmSeriesDiscoverInDom(pmDiscoverEvent *event, pmInResult *in, void *arg)
 			"pmSeriesDiscoverInDom", pmInDomStr(id),
 			in->instlist[i], in->namelist[i]);
 	moduleinfo(event->module, PMLOG_ERROR, msg, arg);
+	/* Coverity CID:328046 - resource leak from earlier pmwebapi_add_instance calls. */
+	baton->error = -ENOMEM;
 	return;
     }
 }
