@@ -43,16 +43,28 @@ static sds EMPTYSTRING, LOCALHOST, WORK_TIMER, POLL_TIMEOUT, BATCHSIZE;
 enum matches { MATCH_EXACT, MATCH_GLOB, MATCH_REGEX };
 enum profile { PROFILE_ADD, PROFILE_DEL };
 
+enum webgroup_metric {
+    CONTEXT_MAP_SIZE,
+    NAMES_MAP_SIZE,
+    LABELS_MAP_SIZE,
+    INST_MAP_SIZE,
+    NUM_WEBGROUP_METRIC
+};
+
 typedef struct webgroups {
     struct dict		*contexts;
-    mmv_registry_t	*metrics;
-    void		*metrics_handle;
     struct dict		*config;
+
+    mmv_registry_t	*registry;
+    pmAtomValue		*metrics[NUM_WEBGROUP_METRIC];
+    void		*map;
+
     uv_loop_t		*events;
-    unsigned int	active;
     uv_timer_t		timer;
     uv_mutex_t		mutex;
-    int			stats_timer;
+
+    unsigned int	active;
+    int			timerid;
 } webgroups;
 
 static struct webgroups *
@@ -64,7 +76,7 @@ webgroups_lookup(pmWebGroupModule *module)
 	module->privdata = calloc(1, sizeof(struct webgroups));
 	groups = (struct webgroups *)module->privdata;
 	uv_mutex_init(&groups->mutex);
-	groups->stats_timer = -1;
+	groups->timerid = -1;
     }
     return groups;
 }
@@ -311,15 +323,17 @@ refresh_maps_metrics(void *data)
 {
     struct webgroups	*groups = (struct webgroups *)data;
 
-    if (groups->metrics_handle) {
-	mmv_stats_set(groups->metrics_handle, "contextmap.size",
-	    NULL, dictSize(contextmap));
-	mmv_stats_set(groups->metrics_handle, "namesmap.size",
-	    NULL, dictSize(namesmap));
-	mmv_stats_set(groups->metrics_handle, "labelsmap.size",
-	    NULL, dictSize(labelsmap));
-	mmv_stats_set(groups->metrics_handle, "instmap.size",
-	    NULL, dictSize(instmap));
+    if (groups->metrics) {
+	unsigned int	value;
+
+	value = dictSize(contextmap);
+	mmv_set(groups->map, groups->metrics[CONTEXT_MAP_SIZE], &value);
+	value = dictSize(namesmap);
+	mmv_set(groups->map, groups->metrics[NAMES_MAP_SIZE], &value);
+	value = dictSize(labelsmap);
+	mmv_set(groups->map, groups->metrics[LABELS_MAP_SIZE], &value);
+	value = dictSize(instmap);
+	mmv_set(groups->map, groups->metrics[INST_MAP_SIZE], &value);
     }
 }
 
@@ -392,7 +406,7 @@ webgroup_lookup_context(pmWebGroupSettings *sp, sds *id, dict *params,
 	uv_timer_start(&groups->timer, webgroup_worker,
 			default_worker, default_worker);
 	/* timer for map stats refresh */
-	groups->stats_timer = pmWebTimerRegister(refresh_maps_metrics, (void *)groups);
+	groups->timerid = pmWebTimerRegister(refresh_maps_metrics, groups);
     }
 
     if (*id == NULL) {
@@ -1581,15 +1595,15 @@ pmWebGroupMetric(pmWebGroupSettings *settings, sds id, dict *params, void *arg)
     lookup.context = cp;
     lookup.arg = arg;
 
-    /* allocate uninit'd space for strings */
-    metric->series = sdsnewlen(SDS_NOINIT, 42); sdsclear(metric->series);
-    metric->name = sdsnewlen(SDS_NOINIT, 16); sdsclear(metric->name);
-    metric->sem = sdsnewlen(SDS_NOINIT, 20); sdsclear(metric->sem);
-    metric->type = sdsnewlen(SDS_NOINIT, 20); sdsclear(metric->type);
-    metric->units = sdsnewlen(SDS_NOINIT, 64); sdsclear(metric->units);
-    metric->labels = sdsnewlen(SDS_NOINIT, 128); sdsclear(metric->labels);
-    metric->oneline = sdsnewlen(SDS_NOINIT, 128); sdsclear(metric->oneline);
-    metric->helptext = sdsnewlen(SDS_NOINIT, 128); sdsclear(metric->helptext);
+    /* allocate zero'd space for strings */
+    metric->series = sdsnewlen(NULL, 42); sdsclear(metric->series);
+    metric->name = sdsnewlen(NULL, 16); sdsclear(metric->name);
+    metric->sem = sdsnewlen(NULL, 20); sdsclear(metric->sem);
+    metric->type = sdsnewlen(NULL, 20); sdsclear(metric->type);
+    metric->units = sdsnewlen(NULL, 64); sdsclear(metric->units);
+    metric->labels = sdsnewlen(NULL, 128); sdsclear(metric->labels);
+    metric->oneline = sdsnewlen(NULL, 128); sdsclear(metric->oneline);
+    metric->helptext = sdsnewlen(NULL, 128); sdsclear(metric->helptext);
 
     if (prefix == NULL || *prefix == '\0') {
 	sts = pmTraversePMNS_r("", webmetric_lookup, &lookup);
@@ -1664,6 +1678,8 @@ scrape_metric_labelsets(metric_t *metric, pmWebLabelSet *labels)
 	labels->sets[nsets++] = metric->labelset;
     labels->nsets = nsets;
     sdsclear(labels->buffer);
+    labels->instid = PM_IN_NULL;
+    labels->instname = NULL;
 }
 
 static void
@@ -1689,6 +1705,8 @@ scrape_instance_labelsets(metric_t *metric, indom_t *indom, instance_t *inst,
 	labels->sets[nsets++] = inst->labelset;
     labels->nsets = nsets;
     sdsclear(labels->buffer);
+    labels->instid = inst->inst;
+    labels->instname = inst->name.sds;
 }
 
 static int
@@ -1708,10 +1726,10 @@ webgroup_scrape(pmWebGroupSettings *settings, context_t *cp,
     int			i, j, k, sts, type;
 
     /* pre-allocate buffers for metric metadata */
-    sems = sdsnewlen(SDS_NOINIT, 20); sdsclear(sems);
-    types = sdsnewlen(SDS_NOINIT, 20); sdsclear(types);
-    units = sdsnewlen(SDS_NOINIT, 64); sdsclear(units);
-    labels.buffer = sdsnewlen(SDS_NOINIT, PM_MAXLABELJSONLEN);
+    sems = sdsnewlen(NULL, 20); sdsclear(sems);
+    types = sdsnewlen(NULL, 20); sdsclear(types);
+    units = sdsnewlen(NULL, 64); sdsclear(units);
+    labels.buffer = sdsnewlen(NULL, PM_MAXLABELJSONLEN);
     sdsclear(labels.buffer);
 
     if ((sts = pmFetch(numpmid, pmidlist, &result)) >= 0) {
@@ -2263,10 +2281,10 @@ pmWebGroupSetup(pmWebGroupModule *module)
 int
 pmWebGroupSetEventLoop(pmWebGroupModule *module, void *events)
 {
-    struct webgroups	*webgroups = webgroups_lookup(module);
+    struct webgroups	*groups = webgroups_lookup(module);
 
-    if (webgroups) {
-	webgroups->events = (uv_loop_t *)events;
+    if (groups) {
+	groups->events = (uv_loop_t *)events;
 	return 0;
     }
     return -ENOMEM;
@@ -2275,7 +2293,7 @@ pmWebGroupSetEventLoop(pmWebGroupModule *module, void *events)
 int
 pmWebGroupSetConfiguration(pmWebGroupModule *module, dict *config)
 {
-    struct webgroups	*webgroups = webgroups_lookup(module);
+    struct webgroups	*groups = webgroups_lookup(module);
     char		*endnum;
     sds			value;
 
@@ -2303,8 +2321,8 @@ pmWebGroupSetConfiguration(pmWebGroupModule *module, dict *config)
 	    default_batchsize = DEFAULT_BATCHSIZE;
     }
 
-    if (webgroups) {
-	webgroups->config = config;
+    if (groups) {
+	groups->config = config;
 	return 0;
     }
     return -ENOMEM;
@@ -2313,47 +2331,54 @@ pmWebGroupSetConfiguration(pmWebGroupModule *module, dict *config)
 static void
 pmWebGroupSetupMetrics(pmWebGroupModule *module)
 {
-    struct webgroups	*webgroups = webgroups_lookup(module);
-    pmUnits            nounits = MMV_UNITS(0,0,0,0,0,0);
-    pmInDom            noindom = MMV_INDOM_NULL;
+    struct webgroups	*groups = webgroups_lookup(module);
+    pmAtomValue		**ap;
+    pmUnits		nounits = MMV_UNITS(0,0,0,0,0,0);
+    void		*map;
 
-    if (webgroups == NULL || webgroups->metrics == NULL)
+    if (groups == NULL || groups->registry == NULL)
 	return; /* no metric registry has been set up */
 
     /*
      * Reverse mapping dict metrics
      */
-    mmv_stats_add_metric(webgroups->metrics, "contextmap.size", 1,
-	MMV_TYPE_U32, MMV_SEM_INSTANT, nounits, noindom,
+    mmv_stats_add_metric(groups->registry, "contextmap.size", 1,
+	MMV_TYPE_U32, MMV_SEM_INSTANT, nounits, MMV_INDOM_NULL,
 	"context map dictionary size",
-	"number of entries in the context map dictionary");
+	"Number of entries in the context map dictionary");
 
-    mmv_stats_add_metric(webgroups->metrics, "namesmap.size", 2,
-	MMV_TYPE_U32, MMV_SEM_INSTANT, nounits, noindom,
+    mmv_stats_add_metric(groups->registry, "namesmap.size", 2,
+	MMV_TYPE_U32, MMV_SEM_INSTANT, nounits, MMV_INDOM_NULL,
 	"metric names map dictionary size",
-	"number of entries in the metric names map dictionary");
+	"Number of entries in the metric names map dictionary");
 
-    mmv_stats_add_metric(webgroups->metrics, "labelsmap.size", 3,
-	MMV_TYPE_U32, MMV_SEM_INSTANT, nounits, noindom,
+    mmv_stats_add_metric(groups->registry, "labelsmap.size", 3,
+	MMV_TYPE_U32, MMV_SEM_INSTANT, nounits, MMV_INDOM_NULL,
 	"labels map dictionary size",
-	"number of entries in the labels map dictionary");
+	"Number of entries in the labels map dictionary");
 
-    mmv_stats_add_metric(webgroups->metrics, "instmap.size", 4,
-	MMV_TYPE_U32, MMV_SEM_INSTANT, nounits, noindom,
+    mmv_stats_add_metric(groups->registry, "instmap.size", 4,
+	MMV_TYPE_U32, MMV_SEM_INSTANT, nounits, MMV_INDOM_NULL,
 	"instance name map dictionary size",
-	"number of entries in the instance name map dictionary");
+	"Number of entries in the instance name map dictionary");
 
-    webgroups->metrics_handle = mmv_stats_start(webgroups->metrics);
+    groups->map = map = mmv_stats_start(groups->registry);
+
+    ap = groups->metrics;
+    ap[CONTEXT_MAP_SIZE] = mmv_lookup_value_desc(map, "contextmap.size", NULL);
+    ap[NAMES_MAP_SIZE] = mmv_lookup_value_desc(map, "namesmap.size", NULL);
+    ap[LABELS_MAP_SIZE] = mmv_lookup_value_desc(map, "labelsmap.size", NULL);
+    ap[INST_MAP_SIZE] = mmv_lookup_value_desc(map, "instmap.size", NULL);
 }
 
 
 int
 pmWebGroupSetMetricRegistry(pmWebGroupModule *module, mmv_registry_t *registry)
 {
-    struct webgroups	*webgroups = webgroups_lookup(module);
+    struct webgroups	*groups = webgroups_lookup(module);
 
-    if (webgroups) {
-	webgroups->metrics = registry;
+    if (groups) {
+	groups->registry = registry;
 	pmWebGroupSetupMetrics(module);
 	return 0;
     }
@@ -2372,8 +2397,8 @@ pmWebGroupClose(pmWebGroupModule *module)
 	if (groups->active) {
 	    groups->active = 0;
 	    uv_timer_stop(&groups->timer);
-	    pmWebTimerRelease(groups->stats_timer);
-	    groups->stats_timer = -1;
+	    pmWebTimerRelease(groups->timerid);
+	    groups->timerid = -1;
 	}
 	iterator = dictGetIterator(groups->contexts);
 	while ((entry = dictNext(iterator)) != NULL)
