@@ -8,7 +8,7 @@
 ** of the 'atop'-framework.
 ** 
 ** Copyright (C) 2007-2010 Gerlof Langeveld
-** Copyright (C) 2015-2019 Red Hat.
+** Copyright (C) 2015-2021 Red Hat.
 **
 ** This program is free software; you can redistribute it and/or modify it
 ** under the terms of the GNU General Public License as published by the
@@ -99,25 +99,23 @@ static time_t	daylimit(time_t);
 static char *
 saroptions(void)
 {
-	char		*flaglist;
+	char		*flags;
 	int		i;
 
 	/* 
 	** gather all flags for the print-functions
 	*/
-	flaglist = malloc(pricnt+32);
+	flags = calloc(1, pricnt+32);
 
-	ptrverify(flaglist, "Malloc failed for %d flags\n", pricnt+32);
+	ptrverify(flags, "Malloc failed for %d flags\n", pricnt+32);
 
 	for (i=0; i < pricnt; i++)
-		flaglist[i] = pridef[i].flag;
-
-	flaglist[i] = 0;
+		flags[i] = pridef[i].flag;
 
 	/*
 	** add generic flags
 	*/
-	return strncat(flaglist, sarflags, pricnt+32);
+	return strncat(flags, sarflags, pricnt+32 - 1);
 }
 
 int
@@ -161,6 +159,7 @@ atopsar(int argc, char *argv[])
 
                            case 'b':		/* begin time ?          */
 				opts.start_optarg = abstime(opts.optarg);
+				opts.origin_optarg = opts.start_optarg;
 				break;
 
                            case 'e':		/* end   time ?          */
@@ -266,17 +265,17 @@ atopsar(int argc, char *argv[])
 					pratopsaruse(pmGetProgname(), &opts);
 			}
 		}
-		/* if no interval specified, read from logfile */
+		/* if no interval specified, read from midnight in todays logfile */
 		else if (!rawreadflag)
 		{
-			__pmAddOptArchivePath(&opts);
+			rawarchive_from_midnight(&opts);
 			rawreadflag++;
 		}
 	}
-	/* if no flags specified at all, read from logfile */
+	/* if no flags specified at all, read midnight in todays from logfile */
 	else if (!rawreadflag)
 	{
-		__pmAddOptArchivePath(&opts);
+		rawarchive_from_midnight(&opts);
 		rawreadflag++;
 	}
 
@@ -352,7 +351,7 @@ atopsar(int argc, char *argv[])
 ** report function to print a new sample in case of live measurements
 */
 static char
-reportlive(double curtime, double numsecs,
+reportlive(double timenow, double numsecs,
          	struct devtstat *devtstat, struct sstat *ss,
 		int nexit, unsigned int noverflow, int flags)
 {
@@ -385,7 +384,7 @@ reportlive(double curtime, double numsecs,
 		if (sampcnt == 0)
 			return '\0';
 
-		printf(datemsg, convdate(curtime, datebuf, sizeof(datebuf)-1));
+		printf(datemsg, convdate(timenow, datebuf, sizeof(datebuf)-1));
 
 		for (i=0; i < pricnt && nr > 0; i++)
 		{
@@ -402,9 +401,9 @@ reportlive(double curtime, double numsecs,
 			if (usecolors)
 				printf(COLSETHEAD);
 
-			printf("%s  ", convtime(curtime-numsecs, timebuf, sizeof(timebuf)-1));
+			printf("%s  ", convtime(timenow-numsecs, timebuf, sizeof(timebuf)-1));
 	
-			(pridef[i].prihead)(osvers, osrel, ossub);
+			(pridef[i].prihead)(os_vers, os_rel, os_sub);
 	
 			if (usecolors)
 				printf(COLRESET);
@@ -414,11 +413,11 @@ reportlive(double curtime, double numsecs,
 			/*
 			** print line with statistical counters
 			*/
-			printf("%s  ", convtime(curtime, timebuf, sizeof(timebuf)-1));
+			printf("%s  ", convtime(timenow, timebuf, sizeof(timebuf)-1));
 	
 			if ( !(pridef[i].priline)(ss, (struct tstat *)0, 0, 0,
 				numsecs, numsecs*hertz, hertz,
-				osvers, osrel, ossub,
+				os_vers, os_rel, os_sub,
 				stampalways ? timebuf : "        ",
 	                        0, 0, 0, 0, 0, 0) )
 			{
@@ -447,10 +446,10 @@ reportlive(double curtime, double numsecs,
 		/*
 		** verify if we have passed midnight of some day
 		*/
-		if (curtime > daylim)
+		if (timenow > daylim)
 		{
-			printf(datemsg, convdate(curtime, datebuf, sizeof(datebuf)-1));
-			daylim = daylimit(curtime);
+			printf(datemsg, convdate(timenow, datebuf, sizeof(datebuf)-1));
+			daylim = daylimit(timenow);
 			curline++;
 		}
 
@@ -467,9 +466,9 @@ reportlive(double curtime, double numsecs,
 			if (usecolors)
 				printf(COLSETHEAD);
 
-			printf("%s  ", convtime(curtime, timebuf, sizeof(timebuf)-1));
+			printf("%s  ", convtime(timenow, timebuf, sizeof(timebuf)-1));
 	
-			(pridef[i].prihead)(osvers, osrel, ossub);
+			(pridef[i].prihead)(os_vers, os_rel, os_sub);
 
 			if (usecolors)
 				printf(COLRESET);
@@ -485,11 +484,11 @@ reportlive(double curtime, double numsecs,
 		/*
 		** print line with statistical counters
 		*/
-		printf("%s  ", convtime(curtime, timebuf, sizeof(timebuf)-1));
+		printf("%s  ", convtime(timenow, timebuf, sizeof(timebuf)-1));
 	
 		if ( !(rv = (pridef[i].priline)(ss, (struct tstat *)0, 0, 0,
 					numsecs, numsecs*hertz, hertz,
-					osvers, osrel, ossub, 
+					os_vers, os_rel, os_sub, 
 		                        stampalways ? timebuf : "        ",
 	                        	0, 0, 0, 0, 0, 0) ) )
 		{
@@ -514,9 +513,9 @@ reportlive(double curtime, double numsecs,
 			if (usecolors)
 				printf(COLSETHEAD);
 
-			printf("%s  ", convtime(curtime, timebuf, sizeof(timebuf)-1));
+			printf("%s  ", convtime(timenow, timebuf, sizeof(timebuf)-1));
 	
-			(pridef[i].prihead)(osvers, osrel, ossub);
+			(pridef[i].prihead)(os_vers, os_rel, os_sub);
 
 			if (usecolors)
 				printf(COLRESET);
@@ -570,7 +569,7 @@ prep()
 ** report function to print a new sample in case of logged measurements
 */
 static char
-reportraw(double curtime, double numsecs,
+reportraw(double timenow, double numsecs,
          	struct devtstat *devtstat, struct sstat *sstat,
 		int nexit, unsigned int noverflow, int flags)
 {
@@ -603,10 +602,10 @@ reportraw(double curtime, double numsecs,
 	/*
 	** verify if we have passed midnight
 	*/
-	if (curtime > daylim)
+	if (timenow > daylim)
 	{
-		printf(datemsg, convdate(curtime, datebuf, sizeof(datebuf)-1));
-		daylim = daylimit(curtime);
+		printf(datemsg, convdate(timenow, datebuf, sizeof(datebuf)-1));
+		daylim = daylimit(timenow);
 		curline++;
 	}
 
@@ -619,7 +618,7 @@ reportraw(double curtime, double numsecs,
 		/*
 		** initialize variables for new report
 		*/
-		pmtimevalFromReal(curtime, &pretime);
+		pmtimevalFromReal(timenow, &pretime);
 
 		curline   = 1;
 		headline  = 0;
@@ -650,7 +649,7 @@ reportraw(double curtime, double numsecs,
 		timed = pmtimevalToReal(&pretime);
 		printf("%s  ", convtime(timed, timebuf, sizeof(timebuf)-1));
 
-		(pridef[prinow].prihead)(osvers, osrel, ossub);
+		(pridef[prinow].prihead)(os_vers, os_rel, os_sub);
 
 		if (usecolors)
 			printf(COLRESET);
@@ -677,7 +676,7 @@ reportraw(double curtime, double numsecs,
 			rv = (pridef[prinow].priline)(&totsyst,
 				(struct tstat *)0, 0, 0,
 				totalsec, totalsec*hertz, hertz,
-			        osvers, osrel, ossub,
+			        os_vers, os_rel, os_sub,
 		                stampalways ? timebuf : "        ",
 				lastnpres, lastntrun, lastntslpi, lastntslpu,
 	                        totalexit, lastnzomb);
@@ -701,12 +700,12 @@ reportraw(double curtime, double numsecs,
 		*/
 		if (flags & RRMARK)
 		{
-			printf("%s  ", convtime(curtime, timebuf, sizeof(timebuf)-1));
+			printf("%s  ", convtime(timenow, timebuf, sizeof(timebuf)-1));
 
 			printf("......................... logging restarted "
 			       ".........................\n");
 		}
-		pmtimevalFromReal(curtime, &pretime);
+		pmtimevalFromReal(timenow, &pretime);
 		curline++;
 
 		/*
@@ -726,12 +725,12 @@ reportraw(double curtime, double numsecs,
 	*/
 	if (summarycnt == 1)
 	{
-		printf("%s  ", convtime(curtime, timebuf, sizeof(timebuf)-1));
+		printf("%s  ", convtime(timenow, timebuf, sizeof(timebuf)-1));
 
 		rv = (pridef[prinow].priline) (sstat, devtstat->taskall,
 				devtstat->procall, devtstat->nprocall,
 				numsecs, numsecs*hertz, hertz,
-				osvers, osrel, ossub,
+				os_vers, os_rel, os_sub,
 	               		stampalways ? timebuf : "        ",
 				devtstat->ntaskall, devtstat->totrun,
 				devtstat->totslpi, devtstat->totslpu,
@@ -770,7 +769,7 @@ reportraw(double curtime, double numsecs,
 		** remember some values in case the next record
 		** contains the log-restart indicator
 		*/
-		lasttime   = curtime;
+		lasttime   = timenow;
 		lastnpres  = devtstat->nprocall;
 		lastntrun  = devtstat->totrun;
 		lastntslpi = devtstat->totslpi;
@@ -785,12 +784,12 @@ reportraw(double curtime, double numsecs,
 			/*
 			** print output line for required report
 			*/
-			printf("%s  ", convtime(curtime, timebuf, sizeof(timebuf)-1));
+			printf("%s  ", convtime(timenow, timebuf, sizeof(timebuf)-1));
 
 			rv = (pridef[prinow].priline) (&totsyst,
 					(struct tstat *)0, 0, 0,
 					totalsec, totalsec*hertz, hertz,
-					osvers, osrel, ossub,
+					os_vers, os_rel, os_sub,
 					stampalways ? timebuf : "        ",
 					devtstat->ntaskall, devtstat->totrun,
 					devtstat->totslpi, devtstat->totslpu,
@@ -832,7 +831,7 @@ reportraw(double curtime, double numsecs,
 			cleanstop(1);
 	}
 
-	pmtimevalFromReal(curtime, &pretime);
+	pmtimevalFromReal(timenow, &pretime);
 
 	return '\0';
 }
@@ -841,15 +840,12 @@ reportraw(double curtime, double numsecs,
 ** print overall header
 */
 static void
-reportheader(struct sysname *sysname, time_t mtime)
+reportheader(struct sysname *sys, time_t mtime)
 {
         char            cdate[16];
 
         printf("\n%s  %s  %s  %s  %s\n\n",
-                sysname->nodename,
-                sysname->release,
-                sysname->version,
-                sysname->machine,
+                sys->nodename, sys->release, sys->version, sys->machine,
         	convdate(mtime, cdate, sizeof(cdate)-1));
 }
 
@@ -862,7 +858,7 @@ pratopsaruse(char *myname, pmOptions *opts)
 	int	i;
 
 	fprintf(stderr,
-		"Usage: %s [-flags] [-r file|date|y...] [-R cnt] [-b hh:mm] [-e hh:mm]\n",
+		"Usage: %s [-flags] [-r file|date|y...] [-R cnt] [-b date|time] [-e date|time]\n",
 								myname);
 	fprintf(stderr, "\t\tor\n");
 	fprintf(stderr,
@@ -880,9 +876,9 @@ pratopsaruse(char *myname, pmOptions *opts)
 	fprintf(stderr,
 		"\t  -R  summarize <cnt> samples into one sample\n");
 	fprintf(stderr,
-		"\t  -b  begin  showing data from  specified time\n");
+		"\t  -b  begin  showing data from  specified date/time\n");
 	fprintf(stderr,
-		"\t  -e  finish showing data after specified time\n");
+		"\t  -e  finish showing data after specified date/time\n");
 	fprintf(stderr,
 		"\t  -S  print timestamp on every line in case of more "
 		"resources\n");
@@ -1434,7 +1430,7 @@ swapline(struct sstat *ss, struct tstat *ts, struct tstat **ps, int nactproc,
 }
 
 /*
-** Pressure Stall Information statistics
+** PSI statistics
 */
 static void
 psihead(int osvers, int osrel, int ossub)
@@ -1449,25 +1445,52 @@ psiline(struct sstat *ss, struct tstat *ts, struct tstat **ps, int nactproc,
         int osvers, int osrel, int ossub, char *tstamp,
         int ppres,  int ntrun, int ntslpi, int ntslpu, int pexit, int pzombie)
 {
+	// calculate pressure percentages for entire interval
+	unsigned int	csperc  = ss->psi.cpusome.total/(deltatic*10000/hz);
+	unsigned int	msperc  = ss->psi.memsome.total/(deltatic*10000/hz);
+	unsigned int	mfperc  = ss->psi.memfull.total/(deltatic*10000/hz);
+	unsigned int	isperc  = ss->psi.iosome.total /(deltatic*10000/hz);
+	unsigned int	ifperc  = ss->psi.iofull.total /(deltatic*10000/hz);
+	unsigned int	badness = 0;
+
 	if (!ss->psi.present)
 	{
-		printf("no PSI stats available for this system .....\n");
-		return 0;
+		printf("no PSI stats available for this interval...\n");
+		return 1;
 	}
 
-	printf("%3.0f%%%3.0f%%%3.0f%%   %3.0f%%%3.0f%%%3.0f%% "
-	       "%3.0f%%%3.0f%%%3.0f%%   %3.0f%%%3.0f%%%3.0f%% "
-	       "%3.0f%%%3.0f%%%3.0f%%\n",
-		ss->psi.cpusome.avg10, ss->psi.cpusome.avg60,
-		ss->psi.cpusome.avg300,
-		ss->psi.memsome.avg10, ss->psi.memsome.avg60,
-		ss->psi.memsome.avg300,
-		ss->psi.memfull.avg10, ss->psi.memfull.avg60,
-		ss->psi.memfull.avg300,
-		ss->psi.iosome.avg10, ss->psi.iosome.avg60,
-		ss->psi.iosome.avg300,
-		ss->psi.iofull.avg10, ss->psi.iofull.avg60,
-		ss->psi.iofull.avg300);
+	// correct percentages if needed
+	if (csperc > 100)
+		csperc = 100;
+
+	if (msperc > 100)
+		msperc = 100;
+
+	if (mfperc > 100)
+		mfperc = 100;
+
+	if (isperc > 100)
+		isperc = 100;
+
+	if (ifperc > 100)
+		ifperc = 100;
+
+	// consider a 'some' percentage > 0 as almost critical
+	// (I/O full tends to increase rapidly as well)
+	if (csperc || msperc || isperc || ifperc)
+		badness = 80;
+
+	// consider a memory 'full' percentage > 0 as critical
+	if (mfperc)
+		badness = 100;
+
+	// show results
+	preprint(badness);
+
+	printf("   %3u%%       %3u%%     %3u%%      %3u%%    %3u%%",
+		csperc, msperc, mfperc, isperc, ifperc);
+
+	postprint(badness);
 
 	return 1;
 }
@@ -1564,7 +1587,7 @@ gendskline(struct sstat *ss, char *tstamp, char selector)
 			pn = dp->name;
 
 		printf("%-14s %3.0lf%% %6.1lf %7.1lf %7.1lf %7.1lf "
-		       "%5.1lf %6.2lf ms",
+		       "%5.1lf %9.5lf ms",
 		    	pn,
 			(double)dp->io_ms  *  100.0 / mstot,
 			(double)dp->nread  * 1000.0 / mstot,
