@@ -14,7 +14,7 @@
 #
 # pylint: disable=line-too-long,bad-continuation,broad-except,bare-except
 # pylint: disable=missing-docstring,multiple-imports,unused-import
-# pylint: disable=too-many-boolean-expressions
+# pylint: disable=too-many-boolean-expressions,consider-using-dict-items
 
 """ Display socket statistics """
 
@@ -22,8 +22,8 @@ import os, sys, argparse, errno
 
 # PCP Python PMAPI
 from pcp import pmapi
-from cpmapi import PM_CONTEXT_ARCHIVE, PM_CONTEXT_HOST, PM_CONTEXT_LOCAL
-from cpmapi import PM_ERR_EOL, PM_MODE_FORW, PM_MODE_INTERP
+from cpmapi import PM_CONTEXT_ARCHIVE, PM_CONTEXT_HOST
+from cpmapi import PM_ERR_EOL, PM_MODE_INTERP
 
 if sys.version >= '3':
     long = int
@@ -155,8 +155,8 @@ class SS(object):
                     start = options.pmGetOptionStart()
                     self.context.pmSetMode(PM_MODE_INTERP, start, 0)
 
-        except pmapi.pmErr as error:
-            sys.stderr.write("%s: %s '%s'\n" % (error.progname(), error.message(), self.source))
+        except pmapi.pmErr as pmerr:
+            sys.stderr.write("%s: %s '%s'\n" % (pmerr.progname(), pmerr.message(), self.source))
             return False
 
         # check network.persocket metrics are available
@@ -228,7 +228,7 @@ class SS(object):
             s = self.valuesD[metric][inst]
             if s is not None:
                 return fmt % s
-        except:
+        except Exception:
             pass
         return fmt % default
 
@@ -238,7 +238,7 @@ class SS(object):
             s = self.valuesD[metric][inst]
             if s is not None:
                 return fmt % s
-        except:
+        except Exception:
             pass
         return fmt % default
 
@@ -248,7 +248,7 @@ class SS(object):
             s = self.valuesD[metric][inst]
             if s is not None and s != 0:
                 return field
-        except:
+        except Exception:
             pass
         return default
 
