@@ -1582,14 +1582,18 @@ ConnectSocketAgent(AgentInfo *aPtr)
 	     addr = __pmHostEntGetSockAddr(host, &enumIx)) {
 	    if (__pmSockAddrIsInet(addr)) {
 		/* Only consider addresses of the chosen family. */
-		if (aPtr->ipc.socket.addrDomain != AF_INET)
+		if (aPtr->ipc.socket.addrDomain != AF_INET) {
+		    __pmSockAddrFree(addr);
 		    continue;
+		}
 	        fd = __pmCreateSocket();
 	    }
 	    else if (__pmSockAddrIsIPv6(addr)) {
 		/* Only consider addresses of the chosen family. */
-		if (aPtr->ipc.socket.addrDomain != AF_INET6)
+		if (aPtr->ipc.socket.addrDomain != AF_INET6) {
+		    __pmSockAddrFree(addr);
 		    continue;
+		}
 	        fd = __pmCreateIPv6Socket();
 	    }
 	    else {
@@ -1630,7 +1634,7 @@ ConnectSocketAgent(AgentInfo *aPtr)
 	}
 	memset(&addr, 0, sizeof(addr));
 	addr.sun_family = AF_UNIX;
-	strcpy(addr.sun_path, aPtr->ipc.socket.name);
+	pmstrncpy(addr.sun_path, sizeof(addr.sun_path), aPtr->ipc.socket.name);
 	len = (int)offsetof(struct sockaddr_un, sun_path) + (int)strlen(addr.sun_path);
 	sts = connect(fd, (struct sockaddr *) &addr, len);
 #else
@@ -1639,7 +1643,7 @@ ConnectSocketAgent(AgentInfo *aPtr)
 	goto error;
 #endif
     }
-    if (sts < 0) {
+    if (sts < 0 || fd < 0) {
 	fprintf(stderr, "pmcd: Error connecting to \"%s\" agent : %s\n",
 		     aPtr->pmDomainLabel, netstrerror());
 	goto error;
