@@ -1,12 +1,12 @@
 /*
- * Copyright (c) 2012-2019 Red Hat.
+ * Copyright (c) 2012-2019,2021 Red Hat.
  * Copyright (c) 1995-2001 Silicon Graphics, Inc.  All Rights Reserved.
- * 
+ *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
  * Free Software Foundation; either version 2 of the License, or (at your
  * option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
@@ -204,8 +204,10 @@ extern int pmcd_load_libpcp_pmda(void);
  * PDU handling routines
  */
 extern int DoFetch(ClientInfo *, __pmPDU *);
+extern int DoHighResFetch(ClientInfo *, __pmPDU *);
 extern int DoProfile(ClientInfo *, __pmPDU *);
 extern int DoDesc(ClientInfo *, __pmPDU *);
+extern int DoDescIDs(ClientInfo *, __pmPDU *);
 extern int DoLabel(ClientInfo *, __pmPDU *);
 extern int DoInstance(ClientInfo *, __pmPDU *);
 extern int DoText(ClientInfo *, __pmPDU *);
