@@ -322,7 +322,7 @@ setup_proc_net_all(proc_net_all_t *all)
 	pmNotifyErr(LOG_ERR, "%s: ignoring \"%s\" pattern from %s: %s\n",
 		    pmGetProgname(), pattern, filename, buffer);
 defaults:
-	regcomp(&all->regex, default_pattern, REG_EXTENDED|REG_NOSUB);
+	(void)regcomp(&all->regex, default_pattern, REG_EXTENDED|REG_NOSUB);
 	if (pmDebugOptions.libpmda)
 	    fprintf(stderr, "%s: %s interface regular expression:\n%s\n",
 		    "setup_proc_net_all", "default", default_pattern);
@@ -335,13 +335,13 @@ defaults:
 	free(pattern);
 }
 
-int
+void
 refresh_proc_net_all(pmInDom indom, proc_net_all_t *all)
 {
-    net_interface_t	*netip;
     static int		setup;
-    int			sts = 0;
+    net_interface_t	*netip;
     char		*name;
+    int			sts;
 
     if (!setup) {
 	setup_proc_net_all(all);
@@ -359,13 +359,12 @@ refresh_proc_net_all(pmInDom indom, proc_net_all_t *all)
             continue;
 	refresh_net_all(all, netip, name);
     }
-    return sts;
 }
 
-int
+void
 refresh_proc_net_dev(pmInDom indom, linux_container_t *container)
 {
-    static uint32_t	gen;	/* refresh generation number */
+    static int		setup;		/* first pass through */
     static uint32_t	cache_err;	/* throttle messages */
     char		buf[1024];
     FILE		*fp;
@@ -373,18 +372,20 @@ refresh_proc_net_dev(pmInDom indom, linux_container_t *container)
     int			j, sts;
     net_interface_t	*netip;
 
-    if ((fp = linux_statsfile("/proc/net/dev", buf, sizeof(buf))) == NULL)
-    	return -oserror();
-
-    if (gen == 0) {
-	/*
-	 * first time, reload cache from external file, and force any
-	 * subsequent changes to be saved
-	 */
+    /*
+     * first time, reload cache from external file, and force any
+     * subsequent changes to be saved
+     */
+    if (!setup) {
 	pmdaCacheOp(indom, PMDA_CACHE_LOAD);
-	gen++;
+	setup = 1;
     }
 
+    pmdaCacheOp(indom, PMDA_CACHE_INACTIVE);
+
+    if ((fp = linux_statsfile("/proc/net/dev", buf, sizeof(buf))) == NULL)
+	return;
+
     /*
 Inter-|   Receive                                                |  Transmit
  face |bytes    packets errs drop fifo frame compressed multicast|bytes    packets errs drop fifo colls carrier compressed
@@ -392,8 +393,6 @@ Inter-|   Receive                                                |  Transmit
   eth0:       0  337614    0    0    0     0          0         0        0  267537    0    0    0 27346      62          0
      */
 
-    pmdaCacheOp(indom, PMDA_CACHE_INACTIVE);
-
     while (fgets(buf, sizeof(buf), fp) != NULL) {
 	if ((p = v = strchr(buf, ':')) == NULL)
 	    continue;
@@ -436,7 +435,6 @@ Inter-|   Receive                                                |  Transmit
 
     if (!container)
 	pmdaCacheOp(indom, PMDA_CACHE_SAVE);
-    return 0;
 }
 
 int
@@ -457,12 +455,12 @@ refresh_net_sysfs(pmInDom indom, int *need_refresh)
     return sts;
 }
 
-int
+void
 refresh_net_ioctl(pmInDom indom, linux_container_t *cp, int *need_refresh)
 {
     net_interface_t	*netip;
     char		*p;
-    int			sts = 0;
+    int			sts;
 
     for (pmdaCacheOp(indom, PMDA_CACHE_WALK_REWIND);;) {
 	if ((sts = pmdaCacheOp(indom, PMDA_CACHE_WALK_NEXT)) < 0)
@@ -471,10 +469,9 @@ refresh_net_ioctl(pmInDom indom, linux_container_t *cp, int *need_refresh)
 	    continue;
 	refresh_net_dev_ioctl(p, netip, cp, need_refresh);
     }
-    return sts;
 }
 
-static int
+static void
 refresh_net_dev_ipv4_addr(pmInDom indom, linux_container_t *container)
 {
     int n, fd, sts, numreqs = 30;
@@ -484,7 +481,7 @@ refresh_net_dev_ipv4_addr(pmInDom indom, linux_container_t *container)
     static uint32_t cache_err;
 
     if ((fd = refresh_inet_socket(container)) < 0)
-	return fd;
+	return;
 
     ifc.ifc_buf = NULL;
     for (;;) {
@@ -493,7 +490,7 @@ refresh_net_dev_ipv4_addr(pmInDom indom, linux_container_t *container)
 
 	if (ioctl(fd, SIOCGIFCONF, &ifc) < 0) {
 	    free(ifc.ifc_buf);
-	    return -oserror();
+	    return;
 	}
 	if (ifc.ifc_len == sizeof(struct ifreq) * numreqs) {
 	    /* assume it overflowed and try again */
@@ -532,7 +529,6 @@ refresh_net_dev_ipv4_addr(pmInDom indom, linux_container_t *container)
 	refresh_net_ipv4_addr(ifr->ifr_name, netip, container);
     }
     free(ifc.ifc_buf);
-    return 0;
 }
 
 static int
@@ -583,7 +579,7 @@ refresh_net_dev_hw_addr(pmInDom indom)
     return 0;
 }
 
-static int
+static void
 refresh_net_dev_ipv6_addr(pmInDom indom)
 {
     FILE *fp;
@@ -597,7 +593,7 @@ refresh_net_dev_ipv6_addr(pmInDom indom)
     static uint32_t cache_err;
 
     if ((fp = linux_statsfile("/proc/net/if_inet6", buf, sizeof(buf))) == NULL)
-	return 0;
+	return;
 
     /*
      * expecting something like ...
@@ -645,7 +641,6 @@ refresh_net_dev_ipv6_addr(pmInDom indom)
 	netip->has_ipv6 = 1;
     }
     fclose(fp);
-    return 0;
 }
 
 /*
