@@ -134,6 +134,10 @@ static pmDesc	desctab[] = {
     { PMDA_PMID(1,21), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) },
 /* pdu_in.highres_result */
     { PMDA_PMID(1,22), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) },
+/* pdu_in.desc_ids */
+    { PMDA_PMID(1,23), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) },
+/* pdu_in.descs */
+    { PMDA_PMID(1,24), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) },
 
 /* pdu_out.error */
     { PMDA_PMID(2,0), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) },
@@ -179,6 +183,10 @@ static pmDesc	desctab[] = {
     { PMDA_PMID(2,21), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) },
 /* pdu_out.highres_result */
     { PMDA_PMID(2,22), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) },
+/* pdu_out.desc_ids */
+    { PMDA_PMID(2,23), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) },
+/* pdu_out.descs */
+    { PMDA_PMID(2,24), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_COUNTER, PMDA_PMUNITS(0,0,1,0,0,PM_COUNT_ONE) },
 
 /* pmlogger.port */
     { PMDA_PMID(3,0), PM_TYPE_U32, PM_INDOM_NULL, PM_SEM_DISCRETE, PMDA_PMUNITS(0,0,0,0,0,0) },
@@ -1123,175 +1131,6 @@ tzinfo(void)
     return __pmTimezone();
 }
 
-#define ZONEINFO "/zoneinfo/"
-#define LOCALTIME "/etc/localtime"
-
-/*
- * Plan B
- *
- * Get the size for /etc/localtime if possible.
- *
- * Descend /usr/share/zoneinfo looking for a file with the same size
- * and then compare file contents ... if equal, consider it a match.
- *
- * If more than one file matches, pick the first but emit a warning.
- */
-char *
-getzoneinfo_plan_b(void)
-{
-    FILE	*fp;		/* find ... pipe */
-    FILE	*f1;		/* /etc/localtime */
-    FILE	*f2;		/* candidate file */
-    int		c1;
-    int		c2;
-    char	*p;
-    char	*path = NULL;
-    struct stat	sbuf;		/* sbuf.st_size is all that matters */
-    char	tmp[MAXPATHLEN+2];	/* shell command and line buffer */
-
-    if ((f1 = fopen(LOCALTIME, "r")) == NULL) {
-	fprintf(stderr, "getzoneinfo_plan_b: cannot open %s: %s\n", LOCALTIME, pmErrStr(-oserror()));
-	return NULL;
-    }
-
-    if (fstat(fileno(f1), &sbuf) < 0) {
-	fprintf(stderr, "getzoneinfo_plan_b: cannot stat %s: %s\n", LOCALTIME, pmErrStr(-oserror()));
-	fclose(f1);
-	return NULL;
-    }
-    sprintf(tmp, "find /usr/share/zoneinfo -type f -a -size %ldc", sbuf.st_size);
-    if ((fp = popen(tmp, "r")) == NULL) {
-	fprintf(stderr, "getzoneinfo_plan_b: pipe(%s) failed: %s\n", tmp, pmErrStr(-oserror()));
-	fclose(f1);
-	return NULL;
-    }
-    /* start at reading at tmp[1], so ':' can be inserted at tmp[0] */
-    while (fgets(&tmp[1], sizeof(tmp)-1, fp) != NULL) {
-	/* strip \n at end of line */
-	for (p = &tmp[1]; *p != '\n'; p++)
-	    ;
-	*p = '\0';
-	if ((f2 = fopen(&tmp[1], "r")) == NULL) {
-	    fprintf(stderr, "getzoneinfo_plan_b: cannot open %s: %s\n", &tmp[1], pmErrStr(-oserror()));
-	    fclose(f1);
-	    pclose(fp);
-	    if (path != NULL)
-		free(path);
-	    return NULL;
-	}
-	rewind(f1);
-
-	for ( ; ; ) {
-	    c1 = fgetc(f1);
-	    c2 = fgetc(f2);
-	    if (c1 == EOF && c2 == EOF) {
-		/* contents match */
-		if (path == NULL) {
-		    tmp[0] = ':';
-		    path = strdup(tmp);
-		    if (path == NULL) {
-			fprintf(stderr, "getzoneinfo_plan_b: match %s but strdup failed\n", &tmp[1]);
-			fclose(f2);
-			fclose(f1);
-			pclose(fp);
-			return NULL;
-		    }
-		}
-		else {
-		    /*
-		     * Duplicates ... pick shortest path, as this will favour, for
-		     * example, Australia/Melbourne over posix/Australia/Melbourne
-		     */
-		    if (strlen(&path[1]) <= strlen(&tmp[1]))
-			fprintf(stderr, "getzoneinfo_plan_b: Warning: match %s and %s, choosing first one\n", &path[1], &tmp[1]);
-		    else {
-			fprintf(stderr, "getzoneinfo_plan_b: Warning: match %s and %s, choosing second one\n", &path[1], &tmp[1]);
-			free(path);
-			tmp[0] = ':';
-			path = strdup(tmp);
-			if (path == NULL) {
-			    fprintf(stderr, "getzoneinfo_plan_b: strdup failed\n");
-			    fclose(f2);
-			    fclose(f1);
-			    pclose(fp);
-			    return NULL;
-			}
-		    }
-		}
-		break;
-	    }
-	    if (c1 != c2)
-		break;
-	}
-	fclose(f2);
-    }
-    fclose(f1);
-    pclose(fp);
-
-    return path;
-}
-
-/*
- * Get the local timezone tzfile identification
- *
- * We'd like this to return something like ":Australia/Melbourne"
- * that can be used as a $TZ setting.
- *
- * Plan A
- *   If /etc/localtime is a symbolic link, get the pathname it points
- *   to and strip anything up to (and including) the string "/zoneinfo/".
- *
- * Return NULL on failure.
- */
-char *
-getzoneinfo(void)
-{
-    ssize_t	sts;
-    char	*buf;
-    char	*tmp_buf;
-    char	*p;
-    char	*q;
-
-    /* +1 for : +1 for NULL */
-    buf = (char *)malloc(MAXPATHLEN+2);
-    if (buf == NULL)
-	return NULL;
-
-    sts = readlink(LOCALTIME, &buf[1], MAXPATHLEN);
-    if (sts < 0) {
-	/*
-	 * Hmm, not a symlink. Now try Plan B.
-	 */
-	free(buf);
-	buf = getzoneinfo_plan_b();
-	if (buf == NULL)
-	    return NULL;
-    }
-    else
-	buf[sts+1] = '\0';
-
-    /* try to find prefix .../zoneinfo/... */
-    p = strstr(buf, ZONEINFO);
-    if (p != NULL) {
-	/* found it! */
-	q = &p[strlen(ZONEINFO)-1];
-	tmp_buf = strdup(q);
-	if (tmp_buf != NULL) {
-	    free(buf);
-	    buf = tmp_buf;
-	}
-    }
-    else {
-	/* no prefix, truncate ... nice to have, not necessary */
-	tmp_buf = realloc(buf, sts+2);
-	if (tmp_buf != NULL)
-	    buf = tmp_buf;
-    }
-    buf[0] = ':';
-
-    return buf;
-}
-
 static char *
 services(void)
 {
@@ -1715,7 +1554,7 @@ pmcd_fetch(int numpmid, pmID pmidlist[], pmResult **resp, pmdaExt *pmda)
 
 			case 26:	/* zoneinfo */
 				if (zoneinfo == NULL) {
-				    zoneinfo = getzoneinfo();
+				    zoneinfo = __pmZoneinfo();
 				    if (zoneinfo == NULL) {
 					/* value is an empty string if not available */
 					zoneinfo = "";
