@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014-2018 Red Hat.
+ * Copyright (c) 2014-2018,2021 Red Hat.
  * Copyright (c) 1995-2002 Silicon Graphics, Inc.  All Rights Reserved.
  * 
  * This program is free software; you can redistribute it and/or modify it
@@ -22,7 +22,6 @@
 #include <errno.h>
 #include "../libpcp/src/internal.h"
 
-static struct timeval	tv;
 static char		timebuf[32];	/* for pmCtime result + .xxx */
 static int		numpmid;
 static pmID		*pmid;
@@ -31,6 +30,7 @@ static pmID		pmid_missed;
 static int		sflag;
 static int		xflag;		/* for -x (long timestamps) */
 static pmLogLabel	label;
+static int		version;	/* of input archive */
 
 static pmLongOptions longopts[] = {
     PMAPI_OPTIONS_HEADER("Options"),
@@ -88,30 +88,56 @@ tvcmp(struct timeval a, struct timeval b)
 }
 
 static void
-mytimestamp(struct timeval *stamp)
+dump_timeval(struct timeval *stamp)
 {
-    struct pmTimeval	pmtv;
-
-    pmtv.tv_sec = stamp->tv_sec;
-    pmtv.tv_usec = stamp->tv_usec;
+    time_t	time = stamp->tv_sec;
+    int		usec = (int)stamp->tv_usec;
+    char	*yr = NULL;
+    char       	*ddmm;
+    struct tm	tm;
 
     if (xflag) {
-	char	       	*ddmm;
-	char	       	*yr;
-	time_t		time;
-
-	time = stamp->tv_sec;
 	ddmm = pmCtime(&time, timebuf);
 	ddmm[10] = '\0';
 	yr = &ddmm[20];
 	printf("%s ", ddmm);
-	__pmPrintTimeval(stdout, &pmtv);
-	printf(" %4.4s", yr);
-	if (xflag >= 2)
-	    printf(" (%.6f)", pmtimevalSub(stamp, &label.ll_start));
     }
-    else {
-	__pmPrintTimeval(stdout, &pmtv);
+    pmLocaltime(&time, &tm);
+    printf("%02d:%02d:%02d.%06d", tm.tm_hour, tm.tm_min, tm.tm_sec, usec);
+    if (xflag && yr)
+	printf(" %4.4s", yr);
+    if (xflag >= 2)
+	printf(" (%.6f)", pmtimevalSub(stamp, &label.ll_start));
+}
+
+static void
+dump_pmTimestamp(__pmTimestamp *tsp)
+{
+    time_t	time = tsp->sec;
+    int		nsec = (int)tsp->nsec;
+    char	*yr = NULL;
+    char       	*ddmm;
+    struct tm	tm;
+
+    if (xflag) {
+	ddmm = pmCtime(&time, timebuf);
+	ddmm[10] = '\0';
+	yr = &ddmm[20];
+	printf("%s ", ddmm);
+    }
+    pmLocaltime(&time, &tm);
+    printf("%02d:%02d:%02d.%09d", tm.tm_hour, tm.tm_min, tm.tm_sec, nsec);
+    if (xflag && yr)
+	printf(" %4.4s", yr);
+    if (xflag >= 2) {
+#if 0	// TODO use this when log label => __pmTimestamp
+	printf(" (%.9f)", __pmTimestampSub(tsp, &label.ll_start));
+#else
+	__pmTimestamp	start;
+	start.sec = label.ll_start.tv_sec;
+	start.nsec = label.ll_start.tv_usec * 1000;
+	printf(" (%.9f)", __pmTimestampSub(tsp, &start));
+#endif
     }
 }
 
@@ -199,17 +225,17 @@ dump_nrecords(int nrecords, int nmissed)
 }
 
 static int
-dump_nparams(int numpmid)
+dump_nparams(int npmids)
 {
-    if (numpmid == 0) {
+    if (npmids == 0) {
 	printf(" ---\n");
 	printf("	          No parameters\n");
 	return -1;
     }
-    if (numpmid < 0) {
+    if (npmids < 0) {
 	printf(" ---\n");
 	printf("	          Error: illegal number of parameters (%d)\n",
-		numpmid);
+		npmids);
 	return -1;
     }
     return 0;
@@ -339,7 +365,7 @@ dump_event(int numnames, char **names, pmValueSet *vsp, int index, int indom, in
 
 	for (r = 0; r < nrecords; r++) {
 	    printf("        --- event record [%d] timestamp ", r);
-	    mytimestamp(&res[r]->timestamp);
+	    dump_timeval(&res[r]->timestamp);
 	    if (dump_nparams(res[r]->numpmid) < 0)
 		continue;
 	    flags = 0;
@@ -397,7 +423,7 @@ dump_result(pmResult *resp)
 	printf("[%d bytes]\n", nbyte);
     }
 
-    mytimestamp(&resp->timestamp);
+    dump_timeval(&resp->timestamp);
 
     if (resp->numpmid == 0) {
 	printf("  <mark>\n");
@@ -458,8 +484,8 @@ dumpDesc(__pmContext *ctxp)
     pmDesc		*dp;
 
     printf("\nDescriptions for Metrics in the Log ...\n");
-    for (i = 0; i < ctxp->c_archctl->ac_log->l_hashpmid.hsize; i++) {
-	for (hp = ctxp->c_archctl->ac_log->l_hashpmid.hash[i]; hp != NULL; hp = hp->next) {
+    for (i = 0; i < ctxp->c_archctl->ac_log->hashpmid.hsize; i++) {
+	for (hp = ctxp->c_archctl->ac_log->hashpmid.hash[i]; hp != NULL; hp = hp->next) {
 	    dp = (pmDesc *)hp->data;
 	    names = NULL; /* silence coverity */
 	    sts = pmNameAll(dp->pmid, &names);
@@ -483,11 +509,12 @@ dumpDiskInDom(__pmContext *ctxp)
     size_t	rlen;
     __int32_t	check;
     __pmLogHdr	hdr;
-    __pmFILE	*f = ctxp->c_archctl->ac_log->l_mdfp;
+    __pmLogCtl	*lcp = ctxp->c_archctl->ac_log;
+    __pmFILE	*f = lcp->mdfp;
 
     printf("\nInstance Domains on-disk ...\n");
 
-    __pmFseek(f, (long)(sizeof(__pmLogLabel) + 2*sizeof(int)), SEEK_SET);
+    __pmFseek(f, (long)__pmLogLabelSize(lcp), SEEK_SET);
     for ( ; ; ) {
 	n = __pmFread(&hdr, 1, sizeof(__pmLogHdr), f);
 	hdr.len = ntohl(hdr.len);
@@ -509,51 +536,32 @@ dumpDiskInDom(__pmContext *ctxp)
 	    return;
 	}
 	rlen = (size_t)hdr.len - sizeof(__pmLogHdr) - sizeof(__int32_t);
-	if (hdr.type == TYPE_INDOM) {
-	    pmTimeval		*tvp;
+	if (hdr.type == TYPE_INDOM || hdr.type == TYPE_INDOM_V2) {
+	    __pmTimestamp	stamp;
 	    pmInResult		in;
-	    char		*namebase;
-	    __int32_t		*rbuf;
-	    __int32_t		*stridx;
+	    __int32_t		*buf;
+	    int			allinbuf;
 	    int			i;
-	    int			k;
 
-	    if ((rbuf = (__int32_t *)malloc(rlen)) == NULL) {
-		fprintf(stderr, "dumpDiskInDom: tbuf: malloc failed: %zd\n", rlen);
+	    if ((allinbuf = __pmLogLoadInDom(ctxp->c_archctl, rlen, hdr.type, &in, &stamp, &buf)) < 0) {
+		fprintf(stderr, "dumpDiskInDom: __pmLogLoadInDom failed: %s\n", pmErrStr(allinbuf));
 		exit(1);
 	    }
-	    if ((n = __pmFread(rbuf, 1, rlen, f)) != rlen) {
-		fprintf(stderr, "dumpDiskInDom: indom __pmFread(%zd) -> %zd\n",
-			    rlen, n);
-		free(rbuf);
-		return;
-	    }
-
-	    k = 0;
-	    tvp = (pmTimeval *)&rbuf[k];
-	    tv.tv_sec = ntohl(tvp->tv_sec);
-	    tv.tv_usec = ntohl(tvp->tv_usec);
-	    k += sizeof(pmTimeval)/sizeof(int);
-	    in.indom = __ntohpmInDom((unsigned int)rbuf[k++]);
-	    in.numinst = ntohl(rbuf[k++]);
-	    mytimestamp(&tv);
+	    dump_pmTimestamp(&stamp);
 	    printf(" InDom: %s", pmInDomStr(in.indom));
 	    printf(" %d instances\n", in.numinst);
 	    if (in.numinst > 0) {
-		in.instlist = &rbuf[k];
-		k += in.numinst;
-		stridx = &rbuf[k];
-		k += in.numinst;
-		namebase = (char *)&rbuf[k];
 	        for (i = 0; i < in.numinst; i++) {
 		    printf("   %d or \"%s\"\n",
-			ntohl(in.instlist[i]), &namebase[ntohl(stridx[i])]);
+			in.instlist[i], in.namelist[i]);
 		}
 	    }
-	    free(rbuf);
+	    free(buf);
+	    if (in.namelist != NULL && !allinbuf)
+		free(in.namelist);
 	}
 	else {
-	    /* skip this record, not TYPE_INDOM */
+	    /* skip this record, not TYPE_INDOM nor TYPE_INDOM_V2 */
 	    __pmFseek(f, (off_t)rlen, SEEK_CUR);
 	}
 	/* trailer check */
@@ -578,9 +586,10 @@ dumpInDom(__pmContext *ctxp)
     __pmHashNode	*hp;
     __pmLogInDom	*idp;
     __pmLogInDom	*ldp;
+
     printf("\nInstance Domains in the Log ...\n");
-    for (i = 0; i < ctxp->c_archctl->ac_log->l_hashindom.hsize; i++) {
-	for (hp = ctxp->c_archctl->ac_log->l_hashindom.hash[i]; hp != NULL; hp = hp->next) {
+    for (i = 0; i < ctxp->c_archctl->ac_log->hashindom.hsize; i++) {
+	for (hp = ctxp->c_archctl->ac_log->hashindom.hash[i]; hp != NULL; hp = hp->next) {
 	    printf("InDom: %s\n", pmInDomStr((pmInDom)hp->key));
 	    /*
 	     * in reverse chronological order, so iteration is a bit funny
@@ -589,9 +598,7 @@ dumpInDom(__pmContext *ctxp)
 	    for ( ; ; ) {
 		for (idp = (__pmLogInDom *)hp->data; idp->next != ldp; idp =idp->next)
 			;
-		tv.tv_sec = idp->stamp.tv_sec;
-		tv.tv_usec = idp->stamp.tv_usec;
-		mytimestamp(&tv);
+		dump_pmTimestamp(&idp->stamp);
 		printf(" %d instances\n", idp->numinst);
 		for (j = 0; j < idp->numinst; j++) {
 		    printf("   %d or \"%s\"\n",
@@ -612,7 +619,7 @@ dumpHelpText(__pmContext *ctxp)
     unsigned int	type;
     unsigned int	class;
     unsigned int	ident;
-    __pmHashCtl		*l_hashtext;
+    __pmHashCtl		*hashtext;
     const __pmHashCtl	*l_hashtype;
     const __pmHashNode	*hp, *tp;
     const __pmHashNode	*this_item[2], *prev_item[2];
@@ -629,7 +636,7 @@ dumpHelpText(__pmContext *ctxp)
      *   identifier, then by
      *   class (PM_TEXT_ONELINE, PM_TEXT_HELP).
      */
-    l_hashtext = &ctxp->c_archctl->ac_log->l_hashtext;
+    hashtext = &ctxp->c_archctl->ac_log->hashtext;
     for (tix = 0; tix < 2; ++tix) {
 	type = textTypes[tix];
 
@@ -644,7 +651,7 @@ dumpHelpText(__pmContext *ctxp)
 		this_item[cix] = NULL;
 
 		class = textClasses[cix];
-		hp = __pmHashSearch(type | class, l_hashtext);
+		hp = __pmHashSearch(type | class, hashtext);
 		if (hp == NULL)
 		    continue;
 
@@ -701,13 +708,13 @@ dumpLabelSets(__pmContext *ctxp)
     int				tix;
     unsigned int		type;
     unsigned int		ident;
-    __pmHashCtl			*l_hashlabels;
+    __pmHashCtl			*hashlabels;
     const __pmHashCtl		*l_hashtype;
     const __pmHashNode		*hp, *tp;
     const __pmHashNode		*this_item, *prev_item;
     const __pmLogLabelSet	*p;
     double			tdiff, min_diff;
-    pmTimeval			this_stamp, prev_stamp;
+    __pmTimestamp		this_stamp, prev_stamp;
     /* Print the label types in this order. */
     unsigned int labelTypes[] = {
 	PM_LABEL_CONTEXT,
@@ -727,19 +734,19 @@ dumpLabelSets(__pmContext *ctxp)
      *   type, then by
      *   identifier
      */
-    l_hashlabels = &ctxp->c_archctl->ac_log->l_hashlabels;
-    prev_stamp.tv_sec = 0;
-    prev_stamp.tv_usec = 0;
+    hashlabels = &ctxp->c_archctl->ac_log->hashlabels;
+    prev_stamp.sec = 0;
+    prev_stamp.nsec = 0;
     for (;;) {
 	/* find the next earliest time stamp. */
 	min_diff = DBL_MAX;
-	for (lix = 0; lix < l_hashlabels->hsize; ++lix) {
-	    for (hp = l_hashlabels->hash[lix]; hp != NULL; hp = hp->next) {
+	for (lix = 0; lix < hashlabels->hsize; ++lix) {
+	    for (hp = hashlabels->hash[lix]; hp != NULL; hp = hp->next) {
 		l_hashtype = (__pmHashCtl *)hp->data;
 		for (tix = 0; tix < l_hashtype->hsize; tix++) {
 		    for (tp = l_hashtype->hash[tix]; tp != NULL; tp = tp->next) {
 			for (p = (__pmLogLabelSet *)tp->data; p != NULL; p = p->next) {
-			    tdiff = __pmTimevalSub(&p->stamp, &prev_stamp);
+			    tdiff = __pmTimestampSub(&p->stamp, &prev_stamp);
 			    /*
 			     * The chains are sorted in reverse chronological
 			     * order so, if this time stamp is less than or
@@ -765,14 +772,12 @@ dumpLabelSets(__pmContext *ctxp)
 	 * Now print all the label sets at this time stamp.
 	 * Sort by type and then identifier.
 	 */
-	tv.tv_sec = this_stamp.tv_sec;
-	tv.tv_usec = this_stamp.tv_usec;
-	mytimestamp(&tv);
+	dump_pmTimestamp(&this_stamp);
 	putchar('\n');
 	for (lix = 0; lix < sizeof(labelTypes) / sizeof(*labelTypes); ++lix) {
 	    /* Are there labels of this type? */
 	    type = labelTypes[lix];
-	    hp = __pmHashSearch(type, l_hashlabels);
+	    hp = __pmHashSearch(type, hashlabels);
 	    if (hp == NULL)
 		continue;
 
@@ -821,8 +826,13 @@ dumpLabelSets(__pmContext *ctxp)
 		 */
 		; 
 		for (p = (__pmLogLabelSet *)this_item->data; p != NULL; p = p->next){
-		    if (__pmTimevalSub(&p->stamp, &this_stamp) != 0.0)
+#if 0	// TODO use this when labels => __pmTimestamp
+		    if (__pmTimestampSub(&p->stamp, &this_stamp) != 0.0)
 			continue;
+#else
+		    if (__pmTimestampSub(&p->stamp, &this_stamp) != 0.0)
+			continue;
+#endif
 		    ident = (unsigned int)this_item->key;
 		    pmPrintLabelSets(stdout, ident, type, p->labelsets, p->nsets);
 		}
@@ -837,49 +847,49 @@ dumpLabelSets(__pmContext *ctxp)
 static void
 dumpTI(__pmContext *ctxp)
 {
-    int		i;
-    char	path[MAXPATHLEN];
-    off_t	meta_size = -1;		/* initialize to pander to gcc */
-    off_t	log_size = -1;		/* initialize to pander to gcc */
-    struct stat	sbuf;
-    __pmLogTI	*tip;
-    __pmLogTI	*lastp;
-    __pmLogCtl  *lcp;
-    
-    lcp = ctxp->c_archctl->ac_log;
+    __pmLogCtl	*lcp = ctxp->c_archctl->ac_log;
 
     printf("\nTemporal Index\n");
     if (xflag)
 	printf("\t\t");
-    printf("\t\tLog Vol    end(meta)     end(log)\n");
-    lastp = NULL;
-    for (i = 0; i < ctxp->c_archctl->ac_log->l_numti; i++) {
-	tip = &ctxp->c_archctl->ac_log->l_ti[i];
-	tv.tv_sec = tip->ti_stamp.tv_sec;
-	tv.tv_usec = tip->ti_stamp.tv_usec;
-	mytimestamp(&tv);
-	printf("\t  %5d  %11d  %11d\n", tip->ti_vol, tip->ti_meta, tip->ti_log);
+    if (xflag >= 2)
+	printf("\t\t");
+    printf("\t\t\tLog Vol    end(meta)     end(log)\n");
+    char		path[MAXPATHLEN];
+    off_t		meta_size = -1;
+    off_t		log_size = -1;
+    struct stat		sbuf;
+    __pmLogTI		*tip;
+    __pmLogTI		*lastp = NULL;
+    int			i;
+
+    for (i = 0; i < lcp->numti; i++) {
+	tip = &lcp->ti[i];
+	dump_pmTimestamp(&tip->stamp);
+	printf("\t  %5d  %11lld  %11lld\n", tip->vol,
+		(long long)tip->off_meta, (long long)tip->off_data);
 	if (i == 0) {
-	    pmsprintf(path, sizeof(path), "%s.meta", lcp->l_name);
+	    pmsprintf(path, sizeof(path), "%s.meta", lcp->name);
 	    if (__pmStat(path, &sbuf) == 0)
 		meta_size = sbuf.st_size;
 	    else
 		meta_size = -1;
 	}
-	if (lastp == NULL || tip->ti_vol != lastp->ti_vol) { 
-	    pmsprintf(path, sizeof(path), "%s.%d", lcp->l_name, tip->ti_vol);
+	if (lastp == NULL || tip->vol != lastp->vol) { 
+	    pmsprintf(path, sizeof(path), "%s.%d", lcp->name, tip->vol);
 	    if (__pmStat(path, &sbuf) == 0)
 		log_size = sbuf.st_size;
 	    else {
 		log_size = -1;
-		printf("\t\tWarning: file missing for log volume %d\n", tip->ti_vol);
+		printf("\t\tWarning: file missing for log volume %d\n",
+				tip->vol);
 	    }
 	}
 	/*
 	 * Integrity Errors
 	 *
-	 * this(tv_sec) < 0
-	 * this(tv_usec) < 0 || this(tv_usec) > 999999
+	 * this(sec) < 0
+	 * this(nsec) < 0 || this(nsec) > 999999999
 	 * this(timestamp) < last(timestamp)
 	 * this(vol) < last(vol)
 	 * this(vol) == last(vol) && this(meta) <= last(meta)
@@ -892,28 +902,29 @@ dumpTI(__pmContext *ctxp)
 	 *
 	 * this(vol) != last(vol) && !file_exists(<base>.this(vol))
 	 */
-	if (tip->ti_stamp.tv_sec < 0 ||
-	    tip->ti_stamp.tv_usec < 0 || tip->ti_stamp.tv_usec > 999999)
-	    printf("\t\tError: illegal timestamp value (%d sec, %d usec)\n",
-		tip->ti_stamp.tv_sec, tip->ti_stamp.tv_usec);
-	if (meta_size != -1 && tip->ti_meta > meta_size)
-	    printf("\t\tError: offset to meta file past end of file (%ld)\n",
-		(long)meta_size);
-	if (log_size != -1 && tip->ti_log > log_size)
-	    printf("\t\tError: offset to log file past end of file (%ld)\n",
-		(long)log_size);
+	if (tip->stamp.sec < 0 ||
+	    tip->stamp.nsec < 0 || tip->stamp.nsec > 999999999)
+	    printf("\t\tError: illegal timestamp value (%" FMT_INT64 " sec, %d nsec)\n",
+		tip->stamp.sec, tip->stamp.nsec);
+	if (meta_size != -1 && tip->off_meta > meta_size)
+	    printf("\t\tError: offset to meta file past end of file (%lld)\n",
+		(long long)meta_size);
+	if (log_size != -1 && tip->off_data > log_size)
+	    printf("\t\tError: offset to log file past end of file (%lld)\n",
+		(long long)log_size);
 	if (i > 0) {
-	    if (tip->ti_stamp.tv_sec < lastp->ti_stamp.tv_sec ||
-	        (tip->ti_stamp.tv_sec == lastp->ti_stamp.tv_sec &&
-	         tip->ti_stamp.tv_usec < lastp->ti_stamp.tv_usec))
-		printf("\t\tError: timestamp went backwards in time %d.%06d -> %d.%06d\n",
-			(int)lastp->ti_stamp.tv_sec, (int)lastp->ti_stamp.tv_usec,
-			(int)tip->ti_stamp.tv_sec, (int)tip->ti_stamp.tv_usec);
-	    if (tip->ti_vol < lastp->ti_vol)
+	    if (tip->stamp.sec < lastp->stamp.sec ||
+	        (tip->stamp.sec == lastp->stamp.sec &&
+	         tip->stamp.nsec < lastp->stamp.nsec))
+		printf("\t\tError: timestamp went backwards in time "
+			"%" FMT_INT64 ".%09d -> %" FMT_INT64 ".%09d\n",
+			lastp->stamp.sec, lastp->stamp.nsec,
+			tip->stamp.sec, tip->stamp.nsec);
+	    if (tip->vol < lastp->vol)
 		printf("\t\tError: volume number decreased\n");
-	    if (tip->ti_vol == lastp->ti_vol && tip->ti_meta < lastp->ti_meta)
+	    if (tip->vol == lastp->vol && tip->off_meta < lastp->off_meta)
 		printf("\t\tError: offset to meta file decreased\n");
-	    if (tip->ti_vol == lastp->ti_vol && tip->ti_log < lastp->ti_log)
+	    if (tip->vol == lastp->vol && tip->off_data < lastp->off_data)
 		printf("\t\tError: offset to log file decreased\n");
 	}
 	lastp = tip;
@@ -928,7 +939,7 @@ dumpLabel(int verbose)
     pmTimeval	stamp;
     time_t	time;
 
-    printf("Log Label (Log Format Version %d)\n", label.ll_magic & 0xff);
+    printf("Log Label (Log Format Version %d)\n", version);
     printf("Performance metrics from host %s\n", label.ll_hostname);
 
     time = label.ll_start.tv_sec;
@@ -966,14 +977,14 @@ dumpLabel(int verbose)
 static void
 rawdump(FILE *f)
 {
-    long	old;
+    off_t	old;
     int		len;
     int		check;
     int		i;
     int		sts;
 
     if ((old = ftell(f)) < 0) {
-	fprintf(stderr, "rawdump: Botch: ftell(%p) -> %ld (%s)\n", f, old, pmErrStr(-errno));
+	fprintf(stderr, "rawdump: Botch: ftell(%p) -> %lld (%s)\n", f, (long long)old, pmErrStr(-errno));
 	return;
     }
 
@@ -982,7 +993,7 @@ rawdump(FILE *f)
 
     while ((sts = fread(&len, 1, sizeof(len), f)) == sizeof(len)) {
 	len = ntohl(len);
-	printf("Dump ... record len: %d @ offset: %" FMT_UINT64, len, (__uint64_t)(ftell(f) - sizeof(len)));
+	printf("Dump ... record len: %d @ offset: %lld", len, (long long)(ftell(f) - sizeof(len)));
 	len -= 2 * sizeof(len);
 	for (i = 0; i < len; i++) {
 	    check = fgetc(f);
@@ -1010,7 +1021,7 @@ rawdump(FILE *f)
     if (sts < 0)
 	printf("fread fails: %s\n", osstrerror());
     if (fseek(f, old, SEEK_SET) < 0)
-	fprintf(stderr, "Warning: fseek(..., %ld, ...) failed: %s\n", old, pmErrStr(-oserror()));
+	fprintf(stderr, "Warning: fseek(..., %lld, ...) failed: %s\n", (long long)old, pmErrStr(-oserror()));
 }
 
 static void
@@ -1031,8 +1042,9 @@ dometric(const char *name)
 }
 
 static int
-overrides(int opt, pmOptions *opts)
+overrides(int opt, pmOptions *options)
 {
+    (void)options;
     if (opt == 'a' || opt == 'h' || opt == 'L' || opt == 's' || opt == 't')
 	return 1;
     return 0;
@@ -1232,6 +1244,7 @@ main(int argc, char *argv[])
 		pmGetProgname(), pmErrStr(sts));
 	exit(1);
     }
+    version = label.ll_magic & 0xff;
 
     if (numpmid > 0) {
 	/*
@@ -1350,9 +1363,7 @@ main(int argc, char *argv[])
 		skel_result->numpmid = picked;
 		if (picked != numpmid) {
 		    /* did not find 'em all ... shuffle time */
-		    int		j;
-		    i = 0;
-		    for (j = 0; j < numpmid; j++) {
+		    for (i = j = 0; j < numpmid; j++) {
 			if (skel_result->vset[j] != NULL)
 			    skel_result->vset[i++] = skel_result->vset[j];
 		    }
