@@ -51,7 +51,7 @@ static char	help[] = \
     " 12. abort     discard changes then exit";
 
 #define MAXARGS	4
-static char	inbuf[MAXARGS][MAXBUFLEN+1];	/* input buffer */
+static char	inbuf[MAXARGS+1][MAXBUFLEN+1];	/* input buffer */
 static char	previous[MAXVARLEN+1];		/* buffer for last rule name */
 
 static symbol_t commands[] = {
@@ -269,7 +269,7 @@ print_grouplist(int argcount)
 	if ((j = strchr(rulelist[i].self.name, '.')) != NULL) {
 	    *j = '\0';	/* mark end of group name */
 	    if (strcmp(rulelist[i].self.name, lastgroup) != 0) {
-		strcpy(lastgroup, rulelist[i].self.name);
+		pmstrncpy(lastgroup, sizeof(lastgroup), rulelist[i].self.name);
 		pprintf("  %s\n", lastgroup);
 	    }
 	    *j = '.';	/* repair the rule name */
@@ -420,7 +420,7 @@ command_list(int argcount)
     }
 
     if (strcmp(".", inbuf[1]) == 0)
-	strcpy(inbuf[1], previous);
+	pmstrncpy(inbuf[1], sizeof(inbuf[1]), previous);
     if (strcmp("all", inbuf[1]) == 0)
 	all = 1;
     if ((msg = lookup_rules(inbuf[1], &rptr, &rcount, all)) != NULL) {
@@ -441,7 +441,7 @@ command_list(int argcount)
 	}
     }
     free(rptr);
-    strcpy(previous, inbuf[1]);
+    pmstrncpy(previous, sizeof(previous), inbuf[1]);
     return sts;
 }
 
@@ -465,7 +465,7 @@ command_undo(int argcount)
     }
 
     if (strcmp(".", inbuf[1]) == 0)
-	strcpy(inbuf[1], previous);
+	pmstrncpy(inbuf[1], sizeof(inbuf[1]), previous);
     if (strcmp("all", inbuf[1]) == 0)
 	all = 1;
     if ((msg = lookup_rules(inbuf[1], &rptr, &rcount, all)) != NULL) {
@@ -483,7 +483,7 @@ command_undo(int argcount)
 	error(msg);
 	return -1;
     }
-    strcpy(previous, inbuf[1]);
+    pmstrncpy(previous, sizeof(previous), inbuf[1]);
     pmiefile_modified = 1;
     return 0;
 }
@@ -514,8 +514,8 @@ command_modify(int command, int argcount)
 		argcount < 1? "few":"many");
 	    return -1;
 	}
-	strcpy(inbuf[2], "enabled");
-	strcpy(inbuf[3], "yes");
+	pmstrncpy(inbuf[2], sizeof(inbuf[2]), "enabled");
+	pmstrncpy(inbuf[3], sizeof(inbuf[3]), "yes");
     }
     else { /* (command == COMMAND_DISABLE) */
 	if (argcount != 1) {
@@ -523,12 +523,12 @@ command_modify(int command, int argcount)
 		argcount < 1? "few":"many");
 	    return -1;
 	}
-	strcpy(inbuf[2], "enabled");
-	strcpy(inbuf[3], "no");
+	pmstrncpy(inbuf[2], sizeof(inbuf[2]), "enabled");
+	pmstrncpy(inbuf[3], sizeof(inbuf[3]), "no");
     }
 
     if (strcmp(".", inbuf[1]) == 0)
-	strcpy(inbuf[1], previous);
+	pmstrncpy(inbuf[1], sizeof(inbuf[1]), previous);
     if (strcmp("all", inbuf[1]) == 0)
 	all = 1;
     if ( ((c = is_attribute(inbuf[2])) != -1) && c != ATTRIB_ENABLED ) {
@@ -549,7 +549,7 @@ command_modify(int command, int argcount)
     }
     pmiefile_modified = 1;
     free(rptr);
-    strcpy(previous, inbuf[1]);
+    pmstrncpy(previous, sizeof(previous), inbuf[1]);
     return 0;
 }
 
@@ -573,7 +573,7 @@ command_help(int argcount)
     }
 
     if (strcmp(".", inbuf[1]) == 0)
-	strcpy(inbuf[1], previous);
+	pmstrncpy(inbuf[1], sizeof(inbuf[1]), previous);
     if (strcmp("all", inbuf[1]) == 0)
 	all = 1;
     if ((msg = lookup_rules(inbuf[1], &rptr, &rcount, all)) != NULL) {
@@ -595,7 +595,7 @@ command_help(int argcount)
 	}
     }
     free(rptr);
-    strcpy(previous, inbuf[1]);
+    pmstrncpy(previous, sizeof(previous), inbuf[1]);
     return sts;
 }
 
@@ -694,7 +694,7 @@ interact(void)
 	printf("Updates will be made to %s\n", get_pmiefile());
     do {
 	sts = 0;
-	for (n = 0; n < MAXARGS; n++)
+	for (n = 0; n <= MAXARGS; n++)
 	    inbuf[n][0] = '\0';
 	if (interactive) {
 	    setio(0);
@@ -796,7 +796,7 @@ main(int argc, char **argv)
 	opts.errors++;
     }
 
-    for (c = 0; opts.optind < argc && c < MAXARGS; c++) {
+    for (c = 0; opts.optind < argc && c <= MAXARGS; c++) {
 	strncpy(inbuf[c], argv[opts.optind++], MAXBUFLEN);
 	inbuf[c][MAXBUFLEN] = '\0';
 	interactive = 0;
