@@ -30,7 +30,6 @@ int		meta_state = STATE_MISSING;
 int		log_state = STATE_MISSING;
 int		mark_count;
 int		result_count;
-__pmLogLabel	log_label;
 
 static char	*archbasename;	/* after basename() */
 
@@ -313,7 +312,7 @@ main(int argc, char *argv[])
 
     if (index_state == STATE_BAD) {
 	/* prevent subsequent use of bad temporal index */
-	ctxp->c_archctl->ac_log->l_numti = 0;
+	ctxp->c_archctl->ac_log->numti = 0;
     }
 
     sts = pass2(ctxp, archname);
