@@ -1,7 +1,7 @@
 /*
  * pmlogextract - extract desired metrics from PCP archive logs
  *
- * Copyright (c) 2014-2018 Red Hat.
+ * Copyright (c) 2014-2018,2021 Red Hat.
  * Copyright (c) 1997-2002 Silicon Graphics, Inc.  All Rights Reserved.
  *
  * This program is free software; you can redistribute it and/or modify it
@@ -58,22 +58,22 @@ static pmOptions opts = {
  * extract metric name(s) from metadata pdu buffer
  */
 void
-printmetricnames(FILE *f, __pmPDU *pdubuf)
+printmetricnames(FILE *f, __int32_t *pdubuf)
 {
     if (ntohl(pdubuf[0]) > 8) {
 	/*
 	 * have at least one name ... names are packed
 	 * <len><name>... at the end of the buffer
 	 */
-	int	numnames = ntohl(pdubuf[7]);
-	char	*p = (char *)&pdubuf[8];
-	int	i;
-	__pmPDU	len;
+	int		numnames = ntohl(pdubuf[7]);
+	char		*p = (char *)&pdubuf[8];
+	int		i;
+	__int32_t	len;
 
 	for (i = 0; i < numnames; i++) {
-	    memmove((void *)&len, (void *)p, sizeof(__pmPDU));
+	    memmove((void *)&len, (void *)p, sizeof(__int32_t));
 	    len = ntohl(len);
-	    p += sizeof(__pmPDU);
+	    p += sizeof(__int32_t);
 	    if (i > 0) fprintf(f, ", ");
 	    fprintf(f, "%*.*s", len, len, p);
 	    p += len;
@@ -99,7 +99,7 @@ printmetricnames(FILE *f, __pmPDU *pdubuf)
  * returns MATCH_EQUAL {a names} is identical to {b names}
  */
 int
-matchnames(__pmPDU *a, __pmPDU *b)
+matchnames(__int32_t *a, __int32_t *b)
 {
     int		num_a = 0;
     int		num_b = 0;
@@ -119,17 +119,17 @@ matchnames(__pmPDU *a, __pmPDU *b)
      */
     p_a = (char *)&a[8];
     for (i_a = 0; i_a < num_a; i_a++) {
-	__pmPDU		len_a;
+	__int32_t		len_a;
 
-	memmove((void *)&len_a, (void *)p_a, sizeof(__pmPDU));
+	memmove((void *)&len_a, (void *)p_a, sizeof(__int32_t));
 	len_a = ntohl(len_a);
-	p_a += sizeof(__pmPDU);
+	p_a += sizeof(__int32_t);
 	p_b = (char *)&b[8];
 	for (i_b = 0; i_b < num_b; i_b++) {
-	    __pmPDU		len_b;
-	    memmove((void *)&len_b, (void *)p_b, sizeof(__pmPDU));
+	    __int32_t		len_b;
+	    memmove((void *)&len_b, (void *)p_b, sizeof(__int32_t));
 	    len_b = ntohl(len_b);
-	    p_b += sizeof(__pmPDU);
+	    p_b += sizeof(__int32_t);
 
 	    if (len_a == len_b && strncmp(p_a, p_b, len_a) == 0) {
 		num_a_eq++;
@@ -150,17 +150,17 @@ matchnames(__pmPDU *a, __pmPDU *b)
      */
     p_b = (char *)&b[8];
     for (i_b = 0; i_b < num_b; i_b++) {
-	__pmPDU		len_b;
+	__int32_t		len_b;
 
-	memmove((void *)&len_b, (void *)p_b, sizeof(__pmPDU));
+	memmove((void *)&len_b, (void *)p_b, sizeof(__int32_t));
 	len_b = ntohl(len_b);
-	p_b += sizeof(__pmPDU);
+	p_b += sizeof(__int32_t);
 	p_a = (char *)&a[8];
 	for (i_a = 0; i_a < num_a; i_a++) {
-	    __pmPDU		len_a;
-	    memmove((void *)&len_a, (void *)p_a, sizeof(__pmPDU));
+	    __int32_t		len_a;
+	    memmove((void *)&len_a, (void *)p_a, sizeof(__int32_t));
 	    len_a = ntohl(len_a);
-	    p_a += sizeof(__pmPDU);
+	    p_a += sizeof(__int32_t);
 
 	    if (len_b == len_a && strncmp(p_b, p_a, len_b) == 0) {
 		num_b_eq++;
@@ -253,17 +253,25 @@ typedef struct {
     __pmValue_PDU	vlist[1];	/* zero or more */
 } vlist_t;
 
+// TODO ... this only works for V2 archives ... need V3 struct
+// or alternate logic
 /*
  *  Mark record
  */
 typedef struct {
-    __pmPDU		len;
-    __pmPDU		type;
-    __pmPDU		from;
+    __int32_t		len;
+    __int32_t		type;
+    __int32_t		from;
     pmTimeval		timestamp;	/* when returned */
     int			numpmid;	/* zero PMIDs to follow */
 } mark_t;
 
+#ifdef __PCP_EXPERIMENTAL_ARCHIVE_VERSION3
+// TODO I expect we need output version to track input version, else
+// a command line option in which case only V2 (in) -> v3 (out) is the
+// only sane thing (i.e. pmlogconv!)
+// NEEDS THOUGHT.
+#endif
 
 /*
  *  Global variables
@@ -433,8 +441,8 @@ newvolume(char *base, pmTimeval *tvp)
 	struct timeval	stamp;
 	__pmFclose(archctl.ac_mfp);
 	archctl.ac_mfp = newfp;
-	logctl.l_label.ill_vol = archctl.ac_curvol = nextvol;
-	__pmLogWriteLabel(archctl.ac_mfp, &logctl.l_label);
+	logctl.label.vol = archctl.ac_curvol = nextvol;
+	__pmLogWriteLabel(archctl.ac_mfp, &logctl.label);
 	__pmFflush(archctl.ac_mfp);
 	stamp.tv_sec = ntohl(tvp->tv_sec);
 	stamp.tv_usec = ntohl(tvp->tv_usec);
@@ -459,11 +467,11 @@ newvolume(char *base, pmTimeval *tvp)
 static void
 newlabel(void)
 {
-    int		indx;
-    inarch_t	*f_iap = NULL;		/* first non-empty archive */
-    inarch_t	*l_iap = NULL;		/* last non-empty archive */
-    inarch_t	*iap;
-    __pmLogLabel	*lp = &logctl.l_label;
+    int			indx;
+    inarch_t		*f_iap = NULL;		/* first non-empty archive */
+    inarch_t		*l_iap = NULL;		/* last non-empty archive */
+    inarch_t		*iap;
+    __pmLogLabel	*lp = &logctl.label;
 
     /*
      * set outarch to inarch[indx] of first non-empty archive to start
@@ -494,16 +502,17 @@ newlabel(void)
     }
 
     /* copy magic number, pid, host and timezone */
-    lp->ill_magic = f_iap->label.ll_magic;
-    lp->ill_pid = (int)getpid();
-    strncpy(lp->ill_hostname, f_iap->label.ll_hostname, PM_LOG_MAXHOSTLEN);
-    lp->ill_hostname[PM_LOG_MAXHOSTLEN-1] = '\0';
+    lp->magic = f_iap->label.ll_magic;
+    lp->pid = (int)getpid();
+    free(lp->hostname);
+    lp->hostname = strdup(f_iap->label.ll_hostname);
     if (farg) {
 	/*
 	 * use timezone from _first_ non-empty archive ...
 	 * this is the OLD default
 	 */
-	strcpy(lp->ill_tz, f_iap->label.ll_tz);
+	free(lp->timezone);
+	lp->timezone = strdup(f_iap->label.ll_tz);
     }
     else {
 	/*
@@ -513,11 +522,15 @@ newlabel(void)
 	for (indx=inarchnum-1; indx>=0; indx--) {
 	    if (inarch[indx].ctx != PM_ERR_NODATA) {
 		l_iap = &inarch[indx];
-		strcpy(lp->ill_tz, l_iap->label.ll_tz);
+		free(lp->timezone);
+		lp->timezone = strdup(l_iap->label.ll_tz);
 		break;
 	    }
 	}
     }
+    /* TODO: v3 archive zoneinfo */
+    free(lp->zoneinfo);
+    lp->zoneinfo = NULL;
 
     /* reset outarch as appropriate, depending on other input archives */
     for (indx=0; indx<inarchnum; indx++) {
@@ -541,7 +554,7 @@ newlabel(void)
         }
 
 	/* Ensure all archives of the same host */
-	if (strcmp(lp->ill_hostname, iap->label.ll_hostname) != 0) {
+	if (strcmp(lp->hostname, iap->label.ll_hostname) != 0) {
 	    fprintf(stderr,"%s: Error: host name mismatch for input archives\n",
 		    pmGetProgname());
 	    fprintf(stderr, "archive: %s host: %s\n",
@@ -553,14 +566,22 @@ newlabel(void)
 	}
 
 	/* Ensure all archives of the same timezone */
-	if (strcmp(lp->ill_tz, iap->label.ll_tz) != 0) {
+	if (strcmp(lp->timezone, iap->label.ll_tz) != 0) {
+	    const char	*log_used;
+
+	    if (farg)
+		log_used = f_iap->name;
+	    else if (l_iap)
+		log_used = l_iap->name;
+	    else
+		log_used = "?";
 	    fprintf(stderr,
-		"%s: Warning: timezone mismatch for input archives\n",
+		    "%s: Warning: timezone mismatch for input archives\n",
 		    pmGetProgname());
 	    fprintf(stderr, "archive: %s timezone: %s [will be used]\n",
-		farg ? f_iap->name : l_iap->name, lp->ill_tz);
+		    log_used, lp->timezone);
 	    fprintf(stderr, "archive: %s timezone: %s [will be ignored]\n",
-		iap->name, iap->label.ll_tz);
+		    iap->name, iap->label.ll_tz);
 	}
     } /*for(indx)*/
 }
@@ -572,13 +593,13 @@ newlabel(void)
 void
 writelabel_metati(int do_rewind)
 {
-    if (do_rewind) __pmRewind(logctl.l_tifp);
-    logctl.l_label.ill_vol = PM_LOG_VOL_TI;
-    __pmLogWriteLabel(logctl.l_tifp, &logctl.l_label);
+    if (do_rewind) __pmRewind(logctl.tifp);
+    logctl.label.vol = PM_LOG_VOL_TI;
+    __pmLogWriteLabel(logctl.tifp, &logctl.label);
 
-    if (do_rewind) __pmRewind(logctl.l_mdfp);
-    logctl.l_label.ill_vol = PM_LOG_VOL_META;
-    __pmLogWriteLabel(logctl.l_mdfp, &logctl.l_label);
+    if (do_rewind) __pmRewind(logctl.mdfp);
+    logctl.label.vol = PM_LOG_VOL_META;
+    __pmLogWriteLabel(logctl.mdfp, &logctl.label);
 }
 
 
@@ -588,8 +609,8 @@ writelabel_metati(int do_rewind)
 void
 writelabel_data(void)
 {
-    logctl.l_label.ill_vol = 0;
-    __pmLogWriteLabel(archctl.ac_mfp, &logctl.l_label);
+    logctl.label.vol = 0;
+    __pmLogWriteLabel(archctl.ac_mfp, &logctl.label);
 }
 
 
@@ -868,7 +889,7 @@ append_indomreclist(int indx)
     reclist_t		*curr;
     reclist_t		*rec;
     __pmHashNode	*hp;
-    __pmPDU		*pdu;
+    __int32_t		*pdu;
     int			indom;
 
     iap = &inarch[indx];
@@ -1167,7 +1188,7 @@ write_rec(reclist_t *rec)
 	    len = ntohl(h->len);
 	    type = ntohl(h->type);
 	    fprintf(stderr, "write_rec: record len=%d, type=%d @ offset=%d\n",
-	    	len, type, (int)(__pmFtell(logctl.l_mdfp) - sizeof(__pmLogHdr)));
+	    	len, type, (int)(__pmFtell(logctl.mdfp) - sizeof(__pmLogHdr)));
 	    if (type == TYPE_DESC) {
 		pmDesc	*dp;
 		pmDesc	desc;
@@ -1185,39 +1206,37 @@ write_rec(reclist_t *rec)
 		fprintf(stderr, "PMID: %s name: %*.*s\n", pmIDStr(desc.pmid), len, len, name);
 		pmPrintDesc(stderr, &desc);
 	    }
-	    else if (type == TYPE_INDOM) {
-		pmTimeval	*tvp;
-		pmTimeval	when;
-		int		k = 2;
-		pmInDom		indom;
-		int		numinst;
-		int		*instlist;
-		int		inst;
+	    else if (type == TYPE_INDOM || type == TYPE_INDOM_V2) {
+		__int32_t	*buf;
+		pmInResult	in;
+		__pmTimestamp	stamp;
+		int		allinbuf;
 
-		tvp = (pmTimeval *)&rec->pdu[k];
-		when.tv_sec = ntohl(tvp->tv_sec);
-		when.tv_usec = ntohl(tvp->tv_usec);
-		k += sizeof(pmTimeval)/sizeof(rec->pdu[0]);
-		indom = ntoh_pmInDom((unsigned int)rec->pdu[k++]);
-		fprintf(stderr, "INDOM: %s when: ", pmInDomStr(indom));
-		__pmPrintTimeval(stderr, &when);
-		numinst = ntohl(rec->pdu[k++]);
-		fprintf(stderr, " numinst: %d", numinst);
-		if (numinst > 0) {
-		    int		i;
-		    instlist = (int *)&rec->pdu[k];
-		    for (i = 0; i < numinst; i++) {
-			inst = ntohl(instlist[i]);
-			fprintf(stderr, " [%d] %d", i, inst);
-		    }
+		buf = &rec->pdu[2];
+		allinbuf = __pmLogLoadInDom(NULL, 0, type, &in, &stamp, &buf);
+		if (allinbuf < 0) {
+		    fprintf(stderr, "write_rec: __pmLogLoadInDom(type=%d): failed: %s\n", type, pmErrStr(allinbuf));
 		}
-		fputc('\n', stderr);
+		else {
+		    fprintf(stderr, "INDOM: %s when: ", pmInDomStr(in.indom));
+		    __pmPrintTimestamp(stderr, &stamp);
+		    fprintf(stderr, " numinst: %d", in.numinst);
+		    if (in.numinst > 0) {
+			int		i;
+			for (i = 0; i < in.numinst; i++) {
+			    fprintf(stderr, " [%d] %d", i, in.instlist[i]);
+			}
+		    }
+		    fputc('\n', stderr);
+		}
+		if (!allinbuf)
+		    free(in.namelist);
 	    }
-	    else if (type == TYPE_LABEL) {
+	    else if (type == TYPE_LABEL_V2) {
 		pmTimeval	*tvp;
 		pmTimeval	when;
 		int		k = 2;
-		int		type;
+		int		label_type;
 		int		ident;
 		char		buf[1024];
 
@@ -1225,40 +1244,40 @@ write_rec(reclist_t *rec)
 		when.tv_sec = ntohl(tvp->tv_sec);
 		when.tv_usec = ntohl(tvp->tv_usec);
 		k += sizeof(pmTimeval)/sizeof(rec->pdu[0]);
-		type = ntoh_pmLabelType((unsigned int)rec->pdu[k++]);
+		label_type = ntoh_pmLabelType((unsigned int)rec->pdu[k++]);
 		ident = ntoh_pmInDom((unsigned int)rec->pdu[k++]);
 		fprintf(stderr, "LABELSET: %s when: ",
-			__pmLabelIdentString(ident, type, buf, sizeof(buf)));
+			__pmLabelIdentString(ident, label_type, buf, sizeof(buf)));
 		__pmPrintTimeval(stderr, &when);
 		fputc('\n', stderr);
 	    }
 	    else if (type == TYPE_TEXT) {
 		int		k = 2;
-		int		type;
+		int		text_type;
 		int		ident;
 
-		type = ntoh_pmTextType((unsigned int)rec->pdu[k++]);
+		text_type = ntoh_pmTextType((unsigned int)rec->pdu[k++]);
 		fprintf(stderr, "TEXT: type: %s ",
-			((type & PM_TEXT_ONELINE)) ? "oneline" : "help");
-		if ((type & PM_TEXT_PMID)) {
+			((text_type & PM_TEXT_ONELINE)) ? "oneline" : "help");
+		if ((text_type & PM_TEXT_PMID)) {
 		    ident = ntoh_pmID((unsigned int)rec->pdu[k++]);
 		    fprintf(stderr, "TEXT: PMID: %s", pmIDStr(ident));
 		}
-		else { /* (type & PM_TEXT_PMINDOM) */
+		else { /* (text_type & PM_TEXT_PMINDOM) */
 		    ident = ntoh_pmInDom((unsigned int)rec->pdu[k++]);
 		    fprintf(stderr, "TEXT: INDOM: %s", pmInDomStr(ident));
 		}
-		if ((type & PM_TEXT_DIRECT))
+		if ((text_type & PM_TEXT_DIRECT))
 		    fprintf(stderr, " DIRECT");
 		fputc('\n', stderr);
 	    }
 	    else {
-		fprintf(stderr, "Botch: bad type\n");
+		fprintf(stderr, "Botch: bad type: %d\n", type);
 	    }
 	}
 
 	/* write out the pdu ; exit if write failed */
-	if ((sts = _pmLogPut(logctl.l_mdfp, rec->pdu)) < 0) {
+	if ((sts = _pmLogPut(logctl.mdfp, rec->pdu)) < 0) {
 	    fprintf(stderr, "%s: Error: _pmLogPut: meta data : %s\n",
 		    pmGetProgname(), pmErrStr(sts));
 	    abandon_extract();
@@ -1559,7 +1578,7 @@ write_metareclist(pmResult *result, int *needti)
 /*
  *  create a mark record
  */
-__pmPDU *
+__int32_t *
 _createmark(void)
 {
     mark_t	*markp;
@@ -1589,7 +1608,7 @@ _createmark(void)
 	markp->timestamp.tv_sec++;
     }
     markp->numpmid = 0;
-    return((__pmPDU *)markp);
+    return((__int32_t *)markp);
 }
 
 void
@@ -1619,7 +1638,7 @@ nextmeta(void)
     int		numeof = 0;
     int		sts;
     pmID	pmid;			/* pmid for TYPE_DESC */
-    pmInDom	indom;			/* indom for TYPE_INDOM */
+    pmInDom	indom;			/* indom for TYPE_INDOM_V2 */
     __pmLogCtl	*lcp;
     __pmContext	*ctxp;
     inarch_t	*iap;			/* pointer to input archive control */
@@ -1667,7 +1686,7 @@ againmeta:
 	    if (sts != PM_ERR_EOL) {
 		fprintf(stderr, "%s: Error: _pmLogGet[meta %s]: %s\n",
 			pmGetProgname(), iap->name, pmErrStr(sts));
-		_report(lcp->l_mdfp);
+		_report(lcp->mdfp);
 		abandon_extract();
 		/*NOTREACHED*/
 	    }
@@ -1725,7 +1744,7 @@ againmeta:
 		goto againmeta;
 	    }
 	}
-	else if (type == TYPE_INDOM) {
+	else if (type == TYPE_INDOM_V2) {
 	    /*
 	     * if ml is defined, then look for instance domain in the list
 	     * if indom is not in the list then discard it immediately
@@ -1756,7 +1775,7 @@ againmeta:
 	        goto againmeta;
 	    }
 	}
-	else if (type == TYPE_LABEL) {
+	else if (type == TYPE_LABEL_V2) {
 	    /* Decide which label sets we want to keep. */
 	    want = 0;
 	    if (ml == NULL) {
@@ -2245,7 +2264,8 @@ checkwinend(pmTimeval now)
     int		sts;
     pmTimeval	tmptime;
     inarch_t	*iap;
-    __pmPDU	*markpdu;	/* mark b/n time windows */
+    __int32_t	*markpdu;	/* mark b/n time windows */
+    mark_t      *p;
 
     if (winend.tv_sec < 0 || tvcmp(&now, &winend) <= 0)
 	return(0);
@@ -2307,7 +2327,10 @@ checkwinend(pmTimeval now)
     /* must create "mark" record and write it out */
     /* (need only one mark record) */
     markpdu = _createmark();
-    if ((sts = __pmLogPutResult2(&archctl, markpdu)) < 0) {
+    p = (mark_t *)markpdu;
+    p->timestamp.tv_sec = htonl(p->timestamp.tv_sec);
+    p->timestamp.tv_usec = htonl(p->timestamp.tv_usec);
+    if ((sts = __pmLogPutResult2(&archctl, (__pmPDU *)markpdu)) < 0) {
 	fprintf(stderr, "%s: Error: __pmLogPutResult2: log data: %s\n",
 		pmGetProgname(), pmErrStr(sts));
 	abandon_extract();
@@ -2328,7 +2351,7 @@ writerlist(rlist_t **rlready, pmTimeval mintime)
     pmTimeval	restime;	/* time of result */
     struct timeval tstamp;	/* temporary time stamp */
     rlist_t	*elm;		/* element of rlready to be written out */
-    __pmPDU	*pb;		/* pdu buffer */
+    __int32_t	*pb;		/* pdu buffer */
     unsigned long	peek_offset;
 
     while (*rlready != NULL) {
@@ -2353,9 +2376,9 @@ fprintf(stderr, " break!\n");
 	 */
 	if (first_datarec) {
 	    first_datarec = 0;
-	    logctl.l_label.ill_start.tv_sec = elm->res->timestamp.tv_sec;
-	    logctl.l_label.ill_start.tv_usec = elm->res->timestamp.tv_usec;
-            logctl.l_state = PM_LOG_STATE_INIT;
+	    logctl.label.start.sec = elm->res->timestamp.tv_sec;
+	    logctl.label.start.nsec = elm->res->timestamp.tv_usec * 1000;
+            logctl.state = PM_LOG_STATE_INIT;
             writelabel_data();
         }
 
@@ -2374,7 +2397,7 @@ fprintf(stderr, " break!\n");
 	write_priorlabelset(PM_LABEL_CONTEXT, PM_IN_NULL, &tstamp);
 
 	/* convert log record to a pdu */
-	sts = __pmEncodeResult(PDU_OVERRIDE2, elm->res, &pb);
+	sts = __pmEncodeResult(PDU_OVERRIDE2, elm->res, (__pmPDU **)&pb);
 	if (sts < 0) {
 	    fprintf(stderr, "%s: Error: __pmEncodeResult: %s\n",
 		    pmGetProgname(), pmErrStr(sts));
@@ -2404,7 +2427,7 @@ fprintf(stderr, " break!\n");
 	/* write out log record */
 	old_log_offset = __pmFtell(archctl.ac_mfp);
 	assert(old_log_offset >= 0);
-	if ((sts = __pmLogPutResult2(&archctl, pb)) < 0) {
+	if ((sts = __pmLogPutResult2(&archctl, (__pmPDU *)pb)) < 0) {
 	    fprintf(stderr, "%s: Error: __pmLogPutResult2: log data: %s\n",
 		    pmGetProgname(), pmErrStr(sts));
 	    abandon_extract();
@@ -2415,7 +2438,7 @@ fprintf(stderr, " break!\n");
 
 	/* check whether we need to write TI (temporal index) */
 	if (old_log_offset == 0 ||
-	    old_log_offset == sizeof(__pmLogLabel)+2*sizeof(int) ||
+	    old_log_offset == __pmLogLabelSize(&logctl) ||
 	    __pmFtell(archctl.ac_mfp) > flushsize)
 		needti = 1;
 
@@ -2428,30 +2451,33 @@ fprintf(stderr, " break!\n");
 
 	/* flush/update */
 	if (needti) {
+	    __pmTimestamp	stamp;
 	    titime = restime;
 
 	    __pmFflush(archctl.ac_mfp);
-	    __pmFflush(logctl.l_mdfp);
+	    __pmFflush(logctl.mdfp);
 
 	    if (old_log_offset == 0)
-		old_log_offset = sizeof(__pmLogLabel)+2*sizeof(int);
+		old_log_offset = __pmLogLabelSize(&logctl);
 
             new_log_offset = __pmFtell(archctl.ac_mfp);
 	    assert(new_log_offset >= 0);
-            new_meta_offset = __pmFtell(logctl.l_mdfp);
+            new_meta_offset = __pmFtell(logctl.mdfp);
 	    assert(new_meta_offset >= 0);
 
             __pmFseek(archctl.ac_mfp, (long)old_log_offset, SEEK_SET);
-            __pmFseek(logctl.l_mdfp, (long)old_meta_offset, SEEK_SET);
+            __pmFseek(logctl.mdfp, (long)old_meta_offset, SEEK_SET);
 
-            __pmLogPutIndex(&archctl, &restime);
+	    stamp.sec = restime.tv_sec;
+	    stamp.nsec = restime.tv_usec * 1000;
+            __pmLogPutIndex(&archctl, &stamp);
 
             __pmFseek(archctl.ac_mfp, (long)new_log_offset, SEEK_SET);
-            __pmFseek(logctl.l_mdfp, (long)new_meta_offset, SEEK_SET);
+            __pmFseek(logctl.mdfp, (long)new_meta_offset, SEEK_SET);
 
             old_log_offset = __pmFtell(archctl.ac_mfp);
 	    assert(old_log_offset >= 0);
-            old_meta_offset = __pmFtell(logctl.l_mdfp);
+            old_meta_offset = __pmFtell(logctl.mdfp);
 	    assert(old_meta_offset >= 0);
 
             flushsize = __pmFtell(archctl.ac_mfp) + 100000;
@@ -2495,7 +2521,7 @@ writemark(inarch_t *iap)
     p->timestamp.tv_sec = htonl(p->timestamp.tv_sec);
     p->timestamp.tv_usec = htonl(p->timestamp.tv_usec);
 
-    if ((sts = __pmLogPutResult2(&archctl, iap->pb[LOG])) < 0) {
+    if ((sts = __pmLogPutResult2(&archctl, (__pmPDU *)iap->pb[LOG])) < 0) {
 	fprintf(stderr, "%s: Error: __pmLogPutResult2: log data: %s\n",
 		pmGetProgname(), pmErrStr(sts));
 	abandon_extract();
@@ -2700,8 +2726,8 @@ main(int argc, char **argv)
 	exit(1);
     }
 
-    logctl.l_label.ill_start.tv_sec = logstart_tval.tv_sec;
-    logctl.l_label.ill_start.tv_usec = logstart_tval.tv_usec;
+    logctl.label.start.sec = logstart_tval.tv_sec;
+    logctl.label.start.nsec = logstart_tval.tv_usec * 1000;
 
     /*
      * process config file
@@ -2736,10 +2762,10 @@ main(int argc, char **argv)
 	printf("Note: timezone set to \"TZ=%s\"\n\n", tz);
     }
     else {
-	char	*tz;
-        tz = __pmTimezone();
+	char	*local_tz;
+        local_tz = __pmTimezone();
 	/* use TZ from local host */
-	if ((sts = pmNewZone(tz)) < 0) {
+	if ((sts = pmNewZone(local_tz)) < 0) {
 	    fprintf(stderr, "%s: Cannot set local host's timezone: %s\n",
 		    pmGetProgname(), pmErrStr(sts));
 	    exit_status = 1;
@@ -2825,7 +2851,7 @@ main(int argc, char **argv)
 	ilog = -1;
 	curlog.tv_sec = 0;
 	curlog.tv_usec = 0;
-	old_meta_offset = __pmFtell(logctl.l_mdfp);
+	old_meta_offset = __pmFtell(logctl.mdfp);
 	assert(old_meta_offset >= 0);
 
 	/* nextlog() resets ilog, and curlog (to the smallest timestamp) */
@@ -2970,15 +2996,17 @@ cleanup:
     }
     else {
 	/* write the last time stamp */
+	__pmTimestamp	stamp;
+
 	__pmFflush(archctl.ac_mfp);
-	__pmFflush(logctl.l_mdfp);
+	__pmFflush(logctl.mdfp);
 
 	if (old_log_offset == 0)
-	    old_log_offset = sizeof(__pmLogLabel)+2*sizeof(int);
+	    old_log_offset = __pmLogLabelSize(&logctl);
 
 	new_log_offset = __pmFtell(archctl.ac_mfp);
 	assert(new_log_offset >= 0);
-	new_meta_offset = __pmFtell(logctl.l_mdfp);
+	new_meta_offset = __pmFtell(logctl.mdfp);
 	assert(new_meta_offset >= 0);
 
 #if 0
@@ -2987,8 +3015,9 @@ cleanup:
 #endif
 
 	__pmFseek(archctl.ac_mfp, old_log_offset, SEEK_SET);
-	__pmLogPutIndex(&archctl, &current);
-
+	stamp.sec = current.tv_sec;
+	stamp.nsec = current.tv_usec * 1000;
+	__pmLogPutIndex(&archctl, &stamp);
 
 	/* need to fix up label with new start-time */
 	writelabel_metati(1);
