@@ -1212,7 +1212,26 @@ s/^\([A-Za-z][A-Za-z0-9_]*\)=/export \1; \1=/p
 	    then
 		echo "+ $KILL -s USR2 $pid"
 	    else
+		$VERY_VERBOSE && echo >&2 "Sending SIGUSR2 to $pid"
 		$KILL -s USR2 "$pid"
+		if $VERY_VERBOSE
+		then
+		    # We have seen in qa/793, but never in a "real"
+		    # deployment, cases where the pmlogger process
+		    # vanishes a short time after the SIGUSR2 has been
+		    # sent and caught, the process has called exec() and
+		    # main() has restarted.  This check is intended
+		    # detect and report when this happens if -VV is
+		    # in play.
+		    #
+		    sleep 1
+		    if $PCP_PS_PROG -p "$pid" 2>&1 | grep "^$pid[ 	]" >/dev/null
+		    then
+			: still alive
+		    else
+			echo >&2 "Error: pmlogger process $pid has vanished"
+		    fi
+		fi
 	    fi
 	fi
 
@@ -1485,16 +1504,18 @@ END	{ if (inlist != "") print lastdate,inlist }' >$tmp/list
 			# pmlc may race with pmlogger starting up here - a timeout is required
 			# to avoid pmlc blocking forever and hanging pmlogger_daily. RHBZ#1892326
 			[ -z "$PMLOGGER_REQUEST_TIMEOUT" ] && export PMLOGGER_REQUEST_TIMEOUT=2
-			echo status | pmlc $pid >$tmp/out 2>&1
-			if egrep "Connection refused|Transport endpoint is not connected|Timeout, closed connection" <$tmp/out >/dev/null
+			if pmlc "$pid" </dev/null 2>&1 | tee $tmp/out \
+				| grep "^Connected to .*pmlogger" >/dev/null
 			then
+			    # pmlogger socket has been set up ...
+			    break
+			else
 			    [ $i -eq 20 ] && break
 			    i=`expr $i + 1`
 			    sleep 1
-			else
-			    break
 			fi
 		    done
+		    echo status | pmlc "$pid" >$tmp/out 2>&1
 		    current_vol=`sed -n <$tmp/out -e '/^log volume/s/.*[^0-9]\([0-9][0-9]*\)$/\1/p'`
 		    if [ -z "$current_vol" ]
 		    then
@@ -1685,7 +1706,12 @@ fi
 #
 if ! $COMPRESSONLY && [ -f $PCP_LOG_DIR/pmlogger/.NeedRewrite ]
 then
-    rm -f $PCP_LOG_DIR/pmlogger/.NeedRewrite
+    if $SHOWME
+    then
+	echo + rm -f $PCP_LOG_DIR/pmlogger/.NeedRewrite
+    else
+	rm -f $PCP_LOG_DIR/pmlogger/.NeedRewrite
+    fi
 fi
 
 # optional end logging to $PCP_LOG_DIR/NOTICES
