@@ -1,12 +1,12 @@
 /*
- * Copyright (c) 2014-2018 Red Hat.
+ * Copyright (c) 2014-2018,2021 Red Hat.
  * Copyright (c) 1995-2001 Silicon Graphics, Inc.  All Rights Reserved.
- * 
+ *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
  * Free Software Foundation; either version 2 of the License, or (at your
  * option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
@@ -221,7 +221,7 @@ setavail(pmResult *resp)
  * It is a merger of __pmLogGetIndom and searchindom.
  */
 static int
-__localLogGetInDom(__pmLogCtl *lcp, pmInDom indom, pmTimeval *tp, int **instlist, char ***namelist)
+__localLogGetInDom(__pmLogCtl *lcp, pmInDom indom, __pmTimestamp *tsp, int **instlist, char ***namelist)
 {
     __pmHashNode	*hp;
     __pmLogInDom	*idp;
@@ -231,7 +231,7 @@ __localLogGetInDom(__pmLogCtl *lcp, pmInDom indom, pmTimeval *tp, int **instlist
 	fprintf(stderr, "__localLogGetInDom( ..., %s) -> ",
 	    pmInDomStr(indom));
 
-    if ((hp = __pmHashSearch((unsigned int)indom, &lcp->l_hashindom)) == NULL) {
+    if ((hp = __pmHashSearch((unsigned int)indom, &lcp->hashindom)) == NULL) {
 	sts = -1;
 	goto done;
     }
@@ -245,7 +245,7 @@ __localLogGetInDom(__pmLogCtl *lcp, pmInDom indom, pmTimeval *tp, int **instlist
 
     *instlist = idp->instlist;
     *namelist = idp->namelist;
-    *tp = idp->stamp;
+    *tsp = idp->stamp;
 
     sts = idp->numinst;
 
@@ -255,7 +255,7 @@ done:
 	    fprintf(stderr, "%d (__pmHashSearch failed)\n", sts);
 	else if (sts >= 0) {
 	    fprintf(stderr, "%d @ ", sts);
-	    __pmPrintTimeval(stderr, tp);
+	    __pmPrintTimestamp(stderr, tsp);
 	    fputc('\n', stderr);
 	}
 	else
@@ -316,7 +316,7 @@ check_inst(pmValueSet *vsp, int hint, pmResult *lrp)
 }
 
 static int
-putlabels(unsigned int type, unsigned int ident, const pmTimeval *tp)
+putlabels(unsigned int type, unsigned int ident, const __pmTimestamp *tsp)
 {
     int		len;
     pmLabelSet	*label;
@@ -338,10 +338,10 @@ putlabels(unsigned int type, unsigned int ident, const pmTimeval *tp)
 
     if (len > 0) {
 	int	sts;
-	sts = __pmLogPutLabel(&archctl, type, ident, len, label, tp);
+	sts = __pmLogPutLabels(&archctl, type, ident, len, label, tsp);
 	if (sts < 0)
 	    /*
-	     * on success, labels are stashed by __pmLogPutLabel(), otherwise
+	     * on success, labels are stashed by __pmLogPutLabels(), otherwise
 	     * we need to free labels
 	     */
 	    pmFreeLabelSets(label, len);
@@ -352,7 +352,7 @@ putlabels(unsigned int type, unsigned int ident, const pmTimeval *tp)
 }
 
 static int
-manageLabels(pmDesc *desc, const pmTimeval *tp, int only_instances)
+manageLabels(pmDesc *desc, const __pmTimestamp *tsp, int only_instances)
 {
     int		i = 0;
     int		sts = 0;
@@ -386,10 +386,10 @@ manageLabels(pmDesc *desc, const pmTimeval *tp, int only_instances)
 	    ident = PM_IN_NULL;
 
 	/* Lookup returns >= 0 when the key exists */
-	if (__pmLogLookupLabel(&archctl, type, ident, &label, tp) >= 0)
+	if (__pmLogLookupLabel(&archctl, type, ident, &label, tsp) >= 0)
 	    continue;
 
-	if ((sts = putlabels(type, ident, tp)) < 0)
+	if ((sts = putlabels(type, ident, tsp)) < 0)
 	    break;
     }
     return sts;
@@ -554,6 +554,73 @@ log_callback(int afid, void *data)
     }
 }
 
+/*
+ * Sort an indom into ascending external instance identifier order.
+ * Since an indom is often already sorted, and we need to sort both
+ * instlist[] and namelist[], it seems a bubble sort may be most
+ * efficient
+ */
+void
+sort_indom(int numinst, int *instlist, char **namelist)
+{
+    int		i;
+    int		j;
+    int		ti;
+    char	*tp;
+    int		nswap;
+
+    for (i = numinst-1; i >= 0; i--) {
+	nswap = 0;
+	for (j = 1; j <= i; j++) {
+	    if (instlist[j-1] > instlist[j]) {
+		nswap++;
+		ti = instlist[j-1];
+		instlist[j-1] = instlist[j];
+		instlist[j] = ti;
+		tp = namelist[j-1];
+		namelist[j-1] = namelist[j];
+		namelist[j] = tp;
+	    }
+	}
+	if (nswap == 0)
+	    break;
+    }
+}
+
+/*
+ * Test if two observations of the same indom are identical ...
+ * we know the indoms are both sorted.
+ */
+int
+same_indom(int numinst_a, int *instlist_a, char **namelist_a, int numinst_b, int *instlist_b, char **namelist_b)
+{
+    int		i;
+
+    if (numinst_a != numinst_b)
+	return 0;
+
+    /* internal instance identifiers */
+    for (i = 0; i < numinst_a; i++) {
+	if (instlist_a[i] != instlist_b[i])
+	    return 0;
+    }
+
+    /*
+     * external instance names (only bad PMDAs, like proc) should
+     * assign different names to the same instance
+     */
+    for (i = 0; i < numinst_a; i++) {
+	if (strcmp(namelist_a[i], namelist_b[i]) != 0)
+	    return 0;
+    }
+
+    if (pmDebugOptions.logmeta && pmDebugOptions.desperate) {
+	fprintf(stderr, "same_indom -> yes\n");
+    }
+
+    return 1;
+}
+
 /*
  * do real work from callback ...
  */
@@ -577,24 +644,22 @@ do_work(task_t *tp)
     int			needti;
     static off_t	flushsize = 100000;
     long		old_meta_offset;
+    long		label_offset;
     long		new_offset;
     long		new_meta_offset;
     int			pdu_bytes = 0;
     int			pdu_metrics = 0;
     int			numinst;
-    int			*instlist;
+    int			*instlist = NULL;
     char		**namelist;
-    pmTimeval		tmp;
-    pmTimeval		resp_tval;
+    __pmTimestamp	resp_stamp;
+    __pmTimestamp	stamp;
     unsigned long	peek_offset;
 
-    if ((pmDebugOptions.appl2) && (pmDebugOptions.desperate)) {
-	struct timeval	now;
+    label_offset = __pmLogLabelSize(archctl.ac_log);
 
-	pmtimevalNow(&now);
-	pmPrintStamp(stderr, &now);
-	fprintf(stderr, " do_work(tp=%p): afid=%d parse_done=%d exit_samples=%d\n", tp, tp->t_afid, parse_done, exit_samples);
-    }
+    if ((pmDebugOptions.appl2) && (pmDebugOptions.desperate))
+	pmNotifyErr(LOG_INFO, "do_work(tp=%p): afid=%d parse_done=%d exit_samples=%d", tp, tp->t_afid, parse_done, exit_samples);
 
     if (!parse_done)
 	/* ignore callbacks until all of the config file has been parsed */
@@ -678,14 +743,14 @@ do_work(task_t *tp)
 	    if (sts != -ETIMEDOUT) {
 		/* optionally report and disconnect() the first time thru */
 		if (pmDebugOptions.appl2)
-		    fprintf(stderr, "callback: disconnecting because myFetch failed: %s\n", pmErrStr(sts));
+		    pmNotifyErr(LOG_INFO, "callback: disconnecting because myFetch failed: %s", pmErrStr(sts));
 		disconnect(sts);
 	    }
 	    continue;
 	}
 
 	if (pmDebugOptions.appl2)
-	    fprintf(stderr, "callback: fetch group %p (%d metrics, 0x%x change)\n", fp, fp->f_numpmid, changed);
+	    pmNotifyErr(LOG_INFO, "callback: fetch group %p (%d metrics, 0x%x change)", fp, fp->f_numpmid, changed);
 
 	/*
 	 * hook to rewrite PDU buffer ...
@@ -712,7 +777,7 @@ do_work(task_t *tp)
 	peek_offset += ((__pmPDUHdr *)pb_in)->len - sizeof(__pmPDUHdr) + 2*sizeof(int);
 	if (peek_offset > 0x7fffffff) {
 	    if (pmDebugOptions.appl2)
-		fprintf(stderr, "callback: new volume based on max size, currently %ld\n", __pmFtell(archctl.ac_mfp));
+		pmNotifyErr(LOG_INFO, "callback: new volume based on max size, currently %ld", __pmFtell(archctl.ac_mfp));
 	    (void)newvolume(VOL_SW_MAX);
 	}
 
@@ -743,18 +808,18 @@ do_work(task_t *tp)
 	    exit(1);
 	}
 	setavail(resp);
-	resp_tval.tv_sec = resp->timestamp.tv_sec;
-	resp_tval.tv_usec = resp->timestamp.tv_usec;
+	resp_stamp.sec = resp->timestamp.tv_sec;
+	resp_stamp.nsec = resp->timestamp.tv_usec * 1000;
 
 	if (changed & PMCD_LABEL_CHANGE) {
 	    /*
 	     * Change to the context labels associated with logged host
 	     */
-	    putlabels(PM_LABEL_CONTEXT, PM_IN_NULL, &resp_tval);
+	    putlabels(PM_LABEL_CONTEXT, PM_IN_NULL, &resp_stamp);
 	}
 
 	needti = 0;
-	old_meta_offset = __pmFtell(logctl.l_mdfp);
+	old_meta_offset = __pmFtell(logctl.mdfp);
 	assert(old_meta_offset >= 0);
 
 	for (i = 0; i < resp->numpmid; i++) {
@@ -785,14 +850,12 @@ do_work(task_t *tp)
 		    fprintf(stderr, "__pmLogPutDesc: %s\n", pmErrStr(sts));
 		    exit(1);
 		}
-		manageLabels(&desc, &resp_tval, 0);
+		free(names);
+		manageLabels(&desc, &resp_stamp, 0);
 		manageText(&desc);
 		if (IS_DERIVED_LOGGED(desc.pmid))
 		    /* derived metric, restore cluster field ... */
 		    desc.pmid = CLEAR_DERIVED_LOGGED(desc.pmid);
-		if (numnames > 0) {
-		    free(names);
-		}
 	    }
 	    if (desc.type == PM_TYPE_EVENT) {
 		/*
@@ -810,9 +873,8 @@ do_work(task_t *tp)
 		 * returned. The timestamp is then used to decide if
 		 * the indom needs to be refreshed.
 		 */
-		pmTimeval indom_tval;
-		numinst = __localLogGetInDom(&logctl, desc.indom, &indom_tval, &instlist, &namelist);
-		if (numinst > 0 && __pmTimevalSub(&resp_tval, &indom_tval) <= 0) {
+		numinst = __localLogGetInDom(&logctl, desc.indom, &stamp, &instlist, &namelist);
+		if (numinst > 0 && __pmTimestampSub(&resp_stamp, &stamp) <= 0) {
 		    /*
 		     * Already have indom with the same (or later, in the
 		     * case of some time warp) timestamp compared to the
@@ -825,9 +887,18 @@ do_work(task_t *tp)
 		     *     if the indom is dynamic, like proc metrics
 		     */
 		    needindom = 0;
+		    if (pmDebugOptions.logmeta && pmDebugOptions.desperate) {
+			fprintf(stderr, "time warp: pmResult: % " FMT_INT64 ".%09d last %s indom: %" FMT_INT64 ".%09d\n",
+			    resp_stamp.sec, resp_stamp.nsec,
+			    pmInDomStr(desc.indom),
+			    stamp.sec, stamp.nsec);
+		    }
 		}
 		else if (numinst < 0) {
 		    needindom = 1;
+		    if (pmDebugOptions.logmeta && pmDebugOptions.desperate) {
+			fprintf(stderr, "numinst=%d => needindom %s\n", numinst, pmInDomStr(desc.indom));
+		    }
 		}
 		else {
 		    needindom = 0;
@@ -842,6 +913,10 @@ do_work(task_t *tp)
 			}
 			if (k == numinst) {
 			    needindom = 1;
+			    if (pmDebugOptions.logmeta && pmDebugOptions.desperate) {
+				fprintf(stderr, "inst %d in pmResult, not in cached indom => needindom %s\n",
+				    vsp->vlist[j].inst, pmInDomStr(desc.indom));
+			    }
 			    break;
 			}
 		    }
@@ -851,43 +926,77 @@ do_work(task_t *tp)
 		     * tests above, but still the indom still needs to
 		     * be refeshed.
 		     */
-		    if (needindom == 0 && lfp->lf_resp != NULL)
+		    if (needindom == 0 && lfp->lf_resp != NULL) {
 			needindom = check_inst(vsp, i, lfp->lf_resp);
+			if (pmDebugOptions.logmeta && pmDebugOptions.desperate) {
+			    if (needindom)
+				fprintf(stderr, "check_inst => needindom %s\n",
+				    pmInDomStr(desc.indom));
+			}
+		    }
 		}
 
 		if (needindom) {
 		    /*
-		     * Note.  We do NOT free() instlist and namelist allocated
-		     *	  here unless this indom is a duplicate.
-		     *    Look for magic below log{Put,Get}InDom.
+		     * Looks like the indom may have changed, in which case it needs
+		     * to be pushed to the .meta file.
+		     *
+		     * Note.  We do NOT free() new_instlist and new_namelist allocated
+		     *	  here unless this indom is a duplicate (look for magic below
+		     *	  log{Put,Get}InDom) or the indom really has not changed.
 		     */
-		    if ((numinst = pmGetInDom(desc.indom, &instlist, &namelist)) < 0) {
+		    int		new_numinst;
+		    int		*new_instlist;
+		    char	**new_namelist;
+		    if ((new_numinst = pmGetInDom(desc.indom, &new_instlist, &new_namelist)) < 0) {
 			fprintf(stderr, "pmGetInDom(%s): %s\n", pmInDomStr(desc.indom), pmErrStr(numinst));
 			exit(1);
 		    }
-		    tmp.tv_sec = (__int32_t)resp->timestamp.tv_sec;
-		    tmp.tv_usec = (__int32_t)resp->timestamp.tv_usec;
-		    if ((sts = __pmLogPutInDom(&archctl, desc.indom, &tmp, numinst, instlist, namelist)) < 0) {
-			fprintf(stderr, "__pmLogPutInDom: %s\n", pmErrStr(sts));
-			exit(1);
+		    /*
+		     * sort the indom based in internal instance identifier
+		     */
+		    sort_indom(new_numinst, new_instlist, new_namelist);
+		    /*
+		     * if this is the first time we've seen this indom,
+		     * or the current and previous indoms are different
+		     * we need to call __pmLogPutInDom()
+		     */
+		    if (instlist == NULL ||
+		        !same_indom(numinst, instlist, namelist, new_numinst, new_instlist, new_namelist)) {
+			numinst = new_numinst;
+			instlist = new_instlist;
+			namelist = new_namelist;
+			stamp.sec = (__int32_t)resp->timestamp.tv_sec;
+			stamp.nsec = (__int32_t)resp->timestamp.tv_usec * 1000;
+			if ((sts = __pmLogPutInDom(&archctl, desc.indom, &stamp, numinst, instlist, namelist)) < 0) {
+			    fprintf(stderr, "__pmLogPutInDom: %s\n", pmErrStr(sts));
+			    exit(1);
+			}
+			if (sts == PMLOGPUTINDOM_DUP) {
+			    if (pmDebugOptions.logmeta && pmDebugOptions.desperate) {
+				fprintf(stderr, "__pmLogPutInDom -> PMLOGPUTINDOM_DUP\n");
+			    }
+			    free(instlist);
+			    free(namelist);
+			}
+			manageLabels(&desc, &stamp, 1);
+			needti = 1;
+			if (pmDebugOptions.appl2)
+			    pmNotifyErr(LOG_INFO, "callback: indom (%s) changed", pmInDomStr(desc.indom));
 		    }
-		    if (sts == PMLOGPUTINDOM_DUP) {
-			free(instlist);
-			free(namelist);
+		    else {
+			free(new_instlist);
+			free(new_namelist);
 		    }
-		    manageLabels(&desc, &tmp, 1);
-		    needti = 1;
-		    if (pmDebugOptions.appl2)
-			fprintf(stderr, "callback: indom (%s) changed\n", pmInDomStr(desc.indom));
 		}
 	    }
 	}
 
-	if (last_log_offset == 0 || last_log_offset == sizeof(__pmLogLabel)+2*sizeof(int)) {
+	if (last_log_offset == 0 || last_log_offset == label_offset) {
 	    /* first result in this volume */
 	    needti = 1;
 	    if (pmDebugOptions.appl2)
-		fprintf(stderr, "callback: first result for this volume\n");
+		pmNotifyErr(LOG_INFO, "callback: first result for this volume");
 	}
 
 	if (tp->t_dm != 0) {
@@ -923,7 +1032,7 @@ do_work(task_t *tp)
 	if (__pmFtell(archctl.ac_mfp) > flushsize) {
 	    needti = 1;
 	    if (pmDebugOptions.appl2)
-		fprintf(stderr, "callback: file size (%d) reached flushsize (%ld)\n", (int)__pmFtell(archctl.ac_mfp), (long)flushsize);
+		pmNotifyErr(LOG_INFO, "callback: file size (%d) reached flushsize (%ld)", (int)__pmFtell(archctl.ac_mfp), (long)flushsize);
 	}
 
 	if (needti) {
@@ -934,18 +1043,18 @@ do_work(task_t *tp)
 	     */
 	    new_offset = __pmFtell(archctl.ac_mfp);
 	    assert(new_offset >= 0);
-	    new_meta_offset = __pmFtell(logctl.l_mdfp);
+	    new_meta_offset = __pmFtell(logctl.mdfp);
 	    assert(new_meta_offset >= 0);
 	    __pmFseek(archctl.ac_mfp, last_log_offset, SEEK_SET);
-	    __pmFseek(logctl.l_mdfp, old_meta_offset, SEEK_SET);
-	    tmp.tv_sec = (__int32_t)resp->timestamp.tv_sec;
-	    tmp.tv_usec = (__int32_t)resp->timestamp.tv_usec;
-	    __pmLogPutIndex(&archctl, &tmp);
+	    __pmFseek(logctl.mdfp, old_meta_offset, SEEK_SET);
+	    stamp.sec = (__int32_t)resp->timestamp.tv_sec;
+	    stamp.nsec = (__int32_t)resp->timestamp.tv_usec * 1000;
+	    __pmLogPutIndex(&archctl, &stamp);
 	    /*
 	     * ... and put them back
 	     */
 	    __pmFseek(archctl.ac_mfp, new_offset, SEEK_SET);
-	    __pmFseek(logctl.l_mdfp, new_meta_offset, SEEK_SET);
+	    __pmFseek(logctl.mdfp, new_meta_offset, SEEK_SET);
 	    flushsize = __pmFtell(archctl.ac_mfp) + 100000;
 	}
 
@@ -966,7 +1075,6 @@ do_work(task_t *tp)
     if (rflag && tp->t_size == 0 && pdu_metrics > 0) {
 	char	*name = NULL;
 	int	taskindex;
-        int     i;
 
 	tp->t_size = pdu_bytes;
 
@@ -1028,14 +1136,14 @@ do_work(task_t *tp)
 	++vol_samples_counter == vol_switch_samples) {
         (void)newvolume(VOL_SW_COUNTER);
 	if (pmDebugOptions.appl2)
-	    fprintf(stderr, "callback: new volume based on samples (%d)\n", vol_samples_counter);
+	    pmNotifyErr(LOG_INFO, "callback: new volume based on samples (%d)", vol_samples_counter);
     }
 
     if (vol_switch_bytes > 0 &&
         (__pmFtell(archctl.ac_mfp) >= vol_switch_bytes)) {
         (void)newvolume(VOL_SW_BYTES);
 	if (pmDebugOptions.appl2)
-	    fprintf(stderr, "callback: new volume based on size (%d)\n", (int)__pmFtell(archctl.ac_mfp));
+	    pmNotifyErr(LOG_INFO, "callback: new volume based on size (%d)", (int)__pmFtell(archctl.ac_mfp));
     }
 
 }
