@@ -19,13 +19,21 @@
 #include "libpcp.h"
 #include <assert.h>
 
+/*
+ * Pass 0 and name cache helpers
+ */
+extern FILE *pass0(FILE *);
+extern int cache_lookup(char *, pmID *, pmDesc *);
+extern void cache_free(void);
+
 /*
  * a task is a bundle of fetches to be done together - it
  * originally corresponded one-to-one with a configuration
  * file curly-brace-enclosed block, but no longer does.
  */
 typedef struct task_s {
-    struct task_s	*t_next;
+    struct task_s	*t_next;	/* linked list of all tasks */
+    struct task_s	*t_alarmed;	/* linked list of alarmed tasks */
     struct timeval	t_delta;
     int			t_state;	/* logging state */
     int			t_numpmid;
@@ -186,6 +194,7 @@ extern int		log_switch_flag; /* archive switch: set on SIGUSR2 */
 extern int		pmlogger_reexec;
 extern int		vol_samples_counter;
 extern int		archive_version; 
+extern int		pmlc_ipc_version;
 extern int		parse_done;
 extern __int64_t	exit_bytes;
 extern __int64_t	vol_bytes;
