@@ -5,30 +5,33 @@ dometric(const char *name)
 {
     int			sts;
     metric_t		*mp;
+    int			j;
+    int			numnames;
+    char		**names;
 
     if ((namelist = (char **)realloc(namelist, (numpmid+1)*sizeof(namelist[0]))) == NULL) {
 	fprintf(stderr,
-	    "%s: dometric: Error: cannot realloc space for %d names\n",
-		pmProgname, numpmid+1);
+	    "%s: dometric: Error: cannot realloc space for %d namelist\n",
+		pmGetProgname(), numpmid+1);
 	exit(1);
     }
     namelist[numpmid] = strdup(name);
     if ((pmidlist = (pmID *)realloc(pmidlist, (numpmid+1)*sizeof(pmidlist[0]))) == NULL) {
 	fprintf(stderr,
-	    "%s: dometric: Error: cannot realloc space for %d pmIDs\n",
-		pmProgname, numpmid+1);
+	    "%s: dometric: Error: cannot realloc space for %d pmidlist\n",
+		pmGetProgname(), numpmid+1);
 	exit(1);
     }
-    if ((sts = pmLookupName(1, (char **)&name, &pmidlist[numpmid])) < 0) {
+    if ((sts = pmLookupName(1, &name, &pmidlist[numpmid])) < 0) {
 	fprintf(stderr,
 	    "%s: dometric: Error: cannot lookup pmID for metric \"%s\": %s\n",
-		pmProgname, name, pmErrStr(sts));
+		pmGetProgname(), name, pmErrStr(sts));
 	exit(1);
     }
     if ((metriclist = (metric_t *)realloc(metriclist, (numpmid+1)*sizeof(metriclist[0]))) == NULL) {
 	fprintf(stderr,
 	    "%s: dometric: Error: cannot realloc space for %d metric_t's\n",
-		pmProgname, numpmid+1);
+		pmGetProgname(), numpmid+1);
 	exit(1);
     }
     mp = &metriclist[numpmid];
@@ -36,7 +39,7 @@ dometric(const char *name)
     if ((sts = pmLookupDesc(pmidlist[numpmid], &mp->idesc)) < 0) {
 	fprintf(stderr,
 	    "%s: dometric: Error: cannot lookup pmDesc for metric \"%s\": %s\n",
-		pmProgname, name, pmErrStr(sts));
+		pmGetProgname(), name, pmErrStr(sts));
 	exit(1);
     }
     mp->odesc = mp->idesc;	/* struct assignment */
@@ -51,10 +54,24 @@ dometric(const char *name)
         mp->idesc.type == PM_TYPE_EVENT) {
 	fprintf(stderr,
 	    "%s: %s: Warning: skipping %s metric\n",
-		pmProgname, name, pmTypeStr(mp->idesc.type));
+		pmGetProgname(), name, pmTypeStr(mp->idesc.type));
 	mp->mode = MODE_SKIP;
 	goto done;
     }
+    
+    /*
+     * if we've already seen this PMID it is a duplicate name in the
+     * PMNS, so remove this one from the fetch list as we only need
+     * to instantiate the value once in each pmFetch ... any duplicate
+     * PMNS names are added to the output archive metadata when
+     * __pmLogPutDesc() is called below.
+     */
+    for (j = 0; j < numpmid; j++) {
+	if (pmidlist[j] == pmidlist[numpmid]) {
+	    numpmid--;
+	    goto done;
+	}
+    }
 
     /*
      * input -> output descriptor mapping ... has to be the same
@@ -86,30 +103,41 @@ dometric(const char *name)
 	    }
 	    else {
 		fprintf(stderr, "Cannot rate convert \"%s\" yet,", namelist[numpmid]);
-		__pmPrintDesc(stderr, &mp->idesc);
+		pmPrintDesc(stderr, &mp->idesc);
 		exit(1);
 	    }
 	    break;
 #endif
     }
-#if PCP_DEBUG
-    if (pmDebug & DBG_TRACE_APPL0) {
-	fprintf(stderr, "metric: \"%s\" (%s)\n",
-	    namelist[numpmid], pmIDStr(pmidlist[numpmid]));
-	fprintf(stderr, "input descriptor:\n");
-	__pmPrintDesc(stderr, &mp->idesc);
-	fprintf(stderr, "output descriptor (added to archive):\n");
-	__pmPrintDesc(stderr, &mp->odesc);
-    }
-#endif
 
-    if ((sts = __pmLogPutDesc(&logctl, &mp->odesc, 1, &namelist[numpmid])) < 0) {
+    /* get all the names for this metric ... */
+    if ((numnames = pmNameAll(pmidlist[numpmid], &names)) < 0) {
 	fprintf(stderr,
-	    "%s: Error: failed to add pmDesc for %s (%s): %s\n",
-		pmProgname, namelist[numpmid], pmIDStr(pmidlist[numpmid]), pmErrStr(sts));
+	    "%s: Error: failed to get names for %s (%s): %s\n",
+		pmGetProgname(), namelist[numpmid], pmIDStr(pmidlist[numpmid]), pmErrStr(sts));
 	exit(1);
     }
 
+    if (pmDebugOptions.appl0) {
+	fprintf(stderr, "metric: \"");
+	__pmPrintMetricNames(stderr, numnames, names, " or ");
+	fprintf(stderr, "\" (%s)\n", pmIDStr(pmidlist[numpmid]));
+	fprintf(stderr, "input descriptor:\n");
+	pmPrintDesc(stderr, &mp->idesc);
+	fprintf(stderr, "output descriptor (added to archive):\n");
+	pmPrintDesc(stderr, &mp->odesc);
+    }
+
+    if ((sts = __pmLogPutDesc(&archctl, &mp->odesc, numnames, names)) < 0) {
+	fprintf(stderr,
+	    "%s: Error: failed to add pmDesc for", pmGetProgname());
+	__pmPrintMetricNames(stderr, numnames, names, " or ");
+	fprintf(stderr,
+	    " (%s): %s\n", pmIDStr(pmidlist[numpmid]), pmErrStr(sts));
+	exit(1);
+    }
+    free(names);
+
     /*
      * instance domain initialization
      */
@@ -117,7 +145,6 @@ dometric(const char *name)
 	/*
 	 * has an instance domain, check to see if it has already been seen
 	 */
-	int		j;
 
 	for (j = 0; j <= numpmid; j++) {
 	    if (metriclist[j].idp != NULL && 
@@ -131,7 +158,7 @@ dometric(const char *name)
 	    if ((mp->idp = (indom_t *)malloc(sizeof(indom_t))) == NULL) {
 		fprintf(stderr,
 		    "%s: dometric: Error: cannot malloc indom_t for %s\n",
-		    pmProgname, pmInDomStr(mp->idesc.indom));
+		    pmGetProgname(), pmInDomStr(mp->idesc.indom));
 		exit(1);
 	    }
 	    mp->idp->indom = mp->idesc.indom;
@@ -141,12 +168,10 @@ dometric(const char *name)
 	}
     }
 
-#if PCP_DEBUG
-    if (pmDebug & DBG_TRACE_APPL0) {
+    if (pmDebugOptions.appl0) {
 	if (mp->idp != NULL)
-	    fprintf(stderr, "    indom %s -> (%p)\n", pmInDomStr(mp->idp->indom), mp->idp);
+	    fprintf(stderr, "    indom %s -> (" PRINTF_P_PFX "%p)\n", pmInDomStr(mp->idp->indom), mp->idp);
     }
-#endif
 
 done:
 
