@@ -1,19 +1,19 @@
 /*
  * pmlogreduce - statistical reduction of a PCP archive log
  *
- * Copyright (c) 2014,2017 Red Hat.
+ * Copyright (c) 2014,2017,2021 Red Hat.
  * Copyright (c) 2004 Silicon Graphics, Inc.  All Rights Reserved.
- * 
+ *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
  * Free Software Foundation; either version 2 of the License, or (at your
  * option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  * for more details.
- * 
+ *
  * TODO (global list)
  * 	- check for counter overflow in doscan()
  * 	- optimization (maybe) for discrete and instantaneous metrics
@@ -202,12 +202,14 @@ int
 main(int argc, char **argv)
 {
     int		sts;
+    int		vers;
     char	*msg;
     pmResult	*irp;		/* input pmResult */
     pmResult	*orp;		/* output pmResult */
     __pmPDU	*pb;		/* pdu buffer */
     struct timeval	unused;
     unsigned long	peek_offset;
+    __pmTimestamp	stamp;
 
     /* no derived or anon metrics, please */
     __pmSetInternalState(PM_STATE_PMCS);
@@ -271,10 +273,8 @@ main(int argc, char **argv)
 	printf("Note: timezone set to \"TZ=%s\"\n\n", tz);
     }
     else {
-	char	*tz;
-        tz = __pmTimezone();
 	/* use TZ from local host */
-	if ((sts = pmNewZone(tz)) < 0) {
+	if ((sts = pmNewZone(__pmTimezone())) < 0) {
 	    fprintf(stderr, "%s: Cannot set local host's timezone: %s\n",
 		    pmGetProgname(), pmErrStr(sts));
 	    exit(1);
@@ -310,7 +310,8 @@ main(int argc, char **argv)
 
     /* create output log - must be done before writing label */
     archctl.ac_log = &logctl;
-    if ((sts = __pmLogCreate("", oname, PM_LOG_VERS02, &archctl)) < 0) {
+    vers = ilabel.ll_magic & 0xff;
+    if ((sts = __pmLogCreate("", oname, vers, &archctl)) < 0) {
 	fprintf(stderr, "%s: Error: __pmLogCreate: %s\n",
 		pmGetProgname(), pmErrStr(sts));
 	exit(1);
@@ -323,15 +324,17 @@ main(int argc, char **argv)
      *		- write labels
      */
     newlabel();
-    current.tv_sec = logctl.l_label.ill_start.tv_sec = winstart_tval.tv_sec;
-    current.tv_usec = logctl.l_label.ill_start.tv_usec = winstart_tval.tv_usec;
+    current.tv_sec = winstart_tval.tv_sec;
+    current.tv_usec = winstart_tval.tv_usec;
+    logctl.label.start.sec = winstart_tval.tv_sec;
+    logctl.label.start.nsec = winstart_tval.tv_usec * 1000;
     /* write label record */
     writelabel();
     /*
-     * Supress any automatic label creation in libpcp at the first
+     * Suppress any automatic label creation in libpcp at the first
      * pmResult write.
      */
-    logctl.l_state = PM_LOG_STATE_INIT;
+    logctl.state = PM_LOG_STATE_INIT;
 
     /*
      * Traverse the PMNS to get all the metrics and their metadata
@@ -345,8 +348,10 @@ main(int argc, char **argv)
     /*
      * All the initial metadata has been generated, add timestamp
      */
-    __pmFflush(logctl.l_mdfp);
-    __pmLogPutIndex(&archctl, &current);
+    __pmFflush(logctl.mdfp);
+    stamp.sec = current.tv_sec;
+    stamp.nsec = current.tv_usec * 1000;
+    __pmLogPutIndex(&archctl, &stamp);
 
     written = 0;
 
@@ -422,9 +427,9 @@ main(int argc, char **argv)
 	/* switch volumes if required */
 	if (varg > 0) {
 	    if (written > 0 && (written % varg) == 0) {
-		pmTimeval	next_stamp;
-		next_stamp.tv_sec = irp->timestamp.tv_sec;
-		next_stamp.tv_usec = irp->timestamp.tv_usec;
+		__pmTimestamp	next_stamp;
+		next_stamp.sec = irp->timestamp.tv_sec;
+		next_stamp.nsec = irp->timestamp.tv_usec * 1000;
 		newvolume(oname, &next_stamp);
 	    }
 	}
@@ -435,9 +440,9 @@ main(int argc, char **argv)
 	peek_offset = __pmFtell(archctl.ac_mfp);
 	peek_offset += ((__pmPDUHdr *)pb)->len - sizeof(__pmPDUHdr) + 2*sizeof(int);
 	if (peek_offset > 0x7fffffff) {
-	    pmTimeval	next_stamp;
-	    next_stamp.tv_sec = irp->timestamp.tv_sec;
-	    next_stamp.tv_usec = irp->timestamp.tv_usec;
+	    __pmTimestamp	next_stamp;
+	    next_stamp.sec = irp->timestamp.tv_sec;
+	    next_stamp.nsec = irp->timestamp.tv_usec * 1000;
 	    newvolume(oname, &next_stamp);
 	}
 
@@ -464,8 +469,10 @@ next:
 
     /* write the last time stamp */
     __pmFflush(archctl.ac_mfp);
-    __pmFflush(logctl.l_mdfp);
-    __pmLogPutIndex(&archctl, &current);
+    __pmFflush(logctl.mdfp);
+    stamp.sec = current.tv_sec;
+    stamp.nsec = current.tv_usec * 1000;
+    __pmLogPutIndex(&archctl, &stamp);
 
     exit(exit_status);
 
