@@ -1,12 +1,12 @@
 /*
- * Copyright (c) 2013,2018 Red Hat.
+ * Copyright (c) 2013,2018,2021 Red Hat.
  * Copyright (c) 2004 Silicon Graphics, Inc.  All Rights Reserved.
- * 
+ *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
  * Free Software Foundation; either version 2 of the License, or (at your
  * option) any later version.
- * 
+ *
  * This program is distributed in the hope that it will be useful, but
  * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
  * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
@@ -27,7 +27,7 @@ extern int	wflag;		/* -w from command line */
  */
 typedef struct {
     int			flags;		/* GLOBAL_* flags */
-    struct timeval	time;		/* timestamp shift */
+    __pmTimestamp	time;		/* timestamp shift */
     char		hostname[PM_LOG_MAXHOSTLEN];
     char		tz[PM_TZ_MAXLEN]; 
 } global_t;
@@ -165,11 +165,12 @@ extern labelspec_t	*label_root;
  */
 typedef struct {
     int		ctx;
+    int		version;	/* input log version (2/3) */
     __pmContext	*ctxp;
     char	*name;
     pmLogLabel	label;
-    __pmPDU	*metarec;
-    __pmPDU	*logrec;
+    __int32_t	*metarec;
+    __int32_t	*logrec;
     pmResult	*rp;
     int		mark;		/* need EOL marker */
 } inarch_t;
@@ -183,6 +184,7 @@ typedef struct {
     char	*name;		/* base name of output archive */
     __pmArchCtl	archctl;	/* libpcp archive control */
     __pmLogCtl	logctl;		/* libpcp log control */
+    int		version;	/* output log version (2/3) */
 } outarch_t;
 
 extern outarch_t	outarch;	/* output archive */
@@ -209,8 +211,8 @@ extern int	yyparse(void);
 #define W_NEXT	2
 #define W_NONE	3
 
-extern int	_pmLogGet(__pmArchCtl *, int, __pmPDU **);
-extern int	_pmLogPut(FILE *, __pmPDU *);
+extern int	_pmLogGet(__pmArchCtl *, int, __int32_t **);
+extern int	_pmLogPut(FILE *, __int32_t *);
 extern int	_pmLogRename(const char *, const char *);
 extern int	_pmLogRemove(const char *, int);
 extern pmUnits	ntoh_pmUnits(pmUnits);
