@@ -23,7 +23,7 @@
  * raw read of next log record - largely stolen from __pmLogRead in libpcp
  */
 int
-_pmLogGet(__pmArchCtl *acp, int vol, __pmPDU **pb)
+_pmLogGet(__pmArchCtl *acp, int vol, __int32_t **pb)
 {
     __pmLogCtl	*lcp = acp->ac_log;
     int		head;
@@ -31,11 +31,11 @@ _pmLogGet(__pmArchCtl *acp, int vol, __pmPDU **pb)
     int		sts;
     long	offset;
     char	*p;
-    __pmPDU	*lpb;
+    __int32_t	*lpb;
     __pmFILE	*f;
 
     if (vol == PM_LOG_VOL_META)
-	f = lcp->l_mdfp;
+	f = lcp->mdfp;
     else
 	f = acp->ac_mfp;
 
@@ -54,7 +54,7 @@ again:
 		fprintf(stderr, "AFTER end\n");
 	    __pmFseek(f, offset, SEEK_SET);
 	    if (vol != PM_LOG_VOL_META) {
-		if (acp->ac_curvol < lcp->l_maxvol) {
+		if (acp->ac_curvol < lcp->maxvol) {
 		    if (__pmLogChangeVol(acp, acp->ac_curvol+1) == 0) {
 			f = acp->ac_mfp;
 			goto again;
@@ -71,7 +71,7 @@ again:
 	    return -oserror();
     }
 
-    if ((lpb = (__pmPDU *)malloc(ntohl(head))) == NULL) {
+    if ((lpb = (__int32_t *)malloc(ntohl(head))) == NULL) {
 	if (pmDebugOptions.log)
 	    fprintf(stderr, "Error: _pmLogGet:(%d) %s\n",
 		(int)ntohl(head), osstrerror());
@@ -106,14 +106,18 @@ again:
     }
 
     if (pmDebugOptions.log) {
-	if (vol != PM_LOG_VOL_META || ntohl(lpb[1]) == TYPE_INDOM) {
+	if (vol != PM_LOG_VOL_META ||
+	    ntohl(lpb[1]) == TYPE_INDOM || ntohl(lpb[1]) == TYPE_INDOM_V2) {
 	    fprintf(stderr, "@");
 	    if (sts >= 0) {
-		struct timeval	stamp;
-		pmTimeval		*tvp = (pmTimeval *)&lpb[vol == PM_LOG_VOL_META ? 2 : 1];
-		stamp.tv_sec = ntohl(tvp->tv_sec);
-		stamp.tv_usec = ntohl(tvp->tv_usec);
-		pmPrintStamp(stderr, &stamp);
+		__pmTimestamp	stamp;
+		if (vol != PM_LOG_VOL_META)
+		    __pmLoadTimeval(&lpb[2], &stamp);
+		else if (ntohl(lpb[1]) == TYPE_INDOM)
+		    __pmLoadTimestamp(&lpb[1], &stamp);
+		else if (ntohl(lpb[1]) == TYPE_INDOM_V2)
+		    __pmLoadTimeval(&lpb[1], &stamp);
+		__pmPrintTimestamp(stderr, &stamp);
 	    }
 	    else
 		fprintf(stderr, "unknown time");
@@ -123,19 +127,23 @@ again:
 
     if (pmDebugOptions.pdu) {
 	int		i, j;
-	struct timeval	stamp;
-	pmTimeval	*tvp = (pmTimeval *)&lpb[vol == PM_LOG_VOL_META ? 2 : 1];
+	__pmTimestamp	stamp;
 	fprintf(stderr, "_pmLogGet");
-	if (vol != PM_LOG_VOL_META || ntohl(lpb[1]) == TYPE_INDOM) {
+	if (vol != PM_LOG_VOL_META ||
+	    ntohl(lpb[1]) == TYPE_INDOM || ntohl(lpb[1]) == TYPE_INDOM_V2) {
+	    if (vol != PM_LOG_VOL_META)
+		__pmLoadTimeval(&lpb[2], &stamp);
+	    else if (ntohl(lpb[1]) == TYPE_INDOM)
+		__pmLoadTimestamp(&lpb[1], &stamp);
+	    else if (ntohl(lpb[1]) == TYPE_INDOM_V2)
+		__pmLoadTimeval(&lpb[1], &stamp);
 	    fprintf(stderr, " timestamp=");
-	    stamp.tv_sec = ntohl(tvp->tv_sec);
-	    stamp.tv_usec = ntohl(tvp->tv_usec);
-	    pmPrintStamp(stderr, &stamp);
+	    __pmPrintTimestamp(stderr, &stamp);
 	}
-	fprintf(stderr, " " PRINTF_P_PFX "%p ... " PRINTF_P_PFX "%p", lpb, &lpb[ntohl(head)/sizeof(__pmPDU) - 1]);
+	fprintf(stderr, " " PRINTF_P_PFX "%p ... " PRINTF_P_PFX "%p", lpb, &lpb[ntohl(head)/sizeof(__int32_t) - 1]);
 	fputc('\n', stderr);
 	fprintf(stderr, "%03d: ", 0);
-	for (j = 0, i = 0; j < ntohl(head)/sizeof(__pmPDU); j++) {
+	for (j = 0, i = 0; j < ntohl(head)/sizeof(__int32_t); j++) {
 	    if (i == 8) {
 		fprintf(stderr, "\n%03d: ", j);
 		i = 0;
