@@ -1,7 +1,7 @@
 /*
  * pmResult rewrite methods for pmlogrewrite
  *
- * Copyright (c) 2017 Red Hat.
+ * Copyright (c) 2017,2021 Red Hat.
  * Copyright (c) 2011 Ken McDonell.  All Rights Reserved.
  * 
  * This program is free software; you can redistribute it and/or modify it
@@ -29,7 +29,7 @@
  * - changing type has the same implications as rescaling
  * - a single metric within a pmResult may have both rescaling and type
  *   change
- * - the initial pmResult contains pointers into a PDU buffer so the fast
+ * - the initial pmResult contains pointers into a record buffer so the fast
  *   track case in pmFreeresult() may not release any of the pmValueSet
  *   or pmValue or pmValueBlock allocations if pmFreeResult is given a
  *   rewritten pmResult, so we modify the pmResult in place but use save[]
@@ -392,7 +392,7 @@ rescale(int i, metricspec_t *mp)
 	    /*
 	     * current value uses pval that is from a previous call to
 	     * __pmStuffValue() during rewriting, not a pointer into a
-	     * PDU buffer
+	     * record buffer
 	     */
 	    if (pmDebugOptions.appl2) {
 		fprintf(stderr, "rescale free(" PRINTF_P_PFX "%p) pval pmid=%s inst=%d\n",
@@ -456,7 +456,7 @@ retype(int i, metricspec_t *mp)
 	    /*
 	     * current value uses pval that is from a previous call to
 	     * __pmStuffValue() during rewriting, not a pointer into a
-	     * PDU buffer
+	     * record buffer
 	     */
 	    if (pmDebugOptions.appl2) {
 		fprintf(stderr, "retype free(" PRINTF_P_PFX "%p) pval pmid=%s inst=%d\n",
@@ -587,11 +587,11 @@ do_result(void)
 		    if (pick >= 0) {
 			if (pick > 0) {
 			    /* swap vlist[0] and vlist[pick] */
-			    pmValue		save;
-			    save = inarch.rp->vset[i]->vlist[0];
+			    pmValue		saved_value;
+			    saved_value = inarch.rp->vset[i]->vlist[0];
 			    inarch.rp->vset[i]->vlist[0] = inarch.rp->vset[i]->vlist[pick];
 			    inarch.rp->vset[i]->vlist[0].inst = inarch.rp->vset[i]->vlist[pick].inst;
-			    inarch.rp->vset[i]->vlist[pick] = save;
+			    inarch.rp->vset[i]->vlist[pick] = saved_value;
 			}
 			if (mp->new_desc.indom == PM_INDOM_NULL)
 			    inarch.rp->vset[i]->vlist[0].inst = PM_IN_NULL;
@@ -633,7 +633,7 @@ do_result(void)
 				    /*
 				     * messy case ... last instance pval is
 				     * from calling __pmStuffValue() in
-				     * rewriting not a pointer into the PDU
+				     * rewriting not a pointer into the record
 				     * buffer, so free here because
 				     * clean_vset() won't find it
 				     */
@@ -666,7 +666,7 @@ do_result(void)
 	unsigned long	out_offset;
 	unsigned long	peek_offset;
 	peek_offset = __pmFtell(outarch.archctl.ac_mfp);
-	sts = __pmEncodeResult(PDU_OVERRIDE2, inarch.rp, &inarch.logrec);
+	sts = __pmEncodeResult(PDU_OVERRIDE2, inarch.rp, (__pmPDU **)&inarch.logrec);
 	if (sts < 0) {
 	    fprintf(stderr, "%s: Error: __pmEncodeResult: %s\n",
 		    pmGetProgname(), pmErrStr(sts));
@@ -682,13 +682,18 @@ do_result(void)
 	    newvolume(outarch.archctl.ac_curvol+1);
 	}
 	out_offset = __pmFtell(outarch.archctl.ac_mfp);
-	if ((sts = __pmLogPutResult2(&outarch.archctl, inarch.logrec)) < 0) {
+	if ((sts = __pmLogPutResult2(&outarch.archctl, (__pmPDU *)inarch.logrec)) < 0) {
 	    fprintf(stderr, "%s: Error: __pmLogPutResult2: log data: %s\n",
 		    pmGetProgname(), pmErrStr(sts));
 	    abandon();
 	    /*NOTREACHED*/
 	}
-	/* do not free inarch.logrec ... this is a libpcp PDU buffer */
+	/*
+	 * do not free inarch.logrec ... this is a libpcp record buffer,
+	 * so Unpin it
+	 */
+	__pmUnpinPDUBuf(inarch.logrec);
+	
 	if (pmDebugOptions.appl0) {
 	    struct timeval	stamp;
 	    fprintf(stderr, "Log: write ");
