@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2018-2019 Red Hat.
+ * Copyright (c) 2018-2019,2021 Red Hat.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -106,48 +106,26 @@ proxymetrics_close(struct proxy *proxy, enum proxy_registry prid)
     }
 }
 
-static void
-server_metrics_init(struct proxy *proxy)
-{
-    mmv_registry_t	*registry;
-    pmAtomValue		*value;
-    pmInDom		noindom = MMV_INDOM_NULL;
-    pmUnits		nounits = MMV_UNITS(0,0,0,0,0,0);
-    pid_t		pid = getpid();
-    char		buffer[64];
-    void		*map;
-
-    if ((registry = proxy->metrics[METRICS_SERVER]) == NULL)
-	return;
-
-    mmv_stats_add_metric(registry, "pid", SERVER_PID,
-		MMV_TYPE_U32, MMV_SEM_DISCRETE, nounits, noindom,
-		"PID for the current pmproxy invocation", NULL);
-    pmsprintf(buffer, sizeof(buffer), "%u", pid);
-    mmv_stats_add_metric_label(registry, SERVER_PID,
-		"pid", buffer, MMV_NUMBER_TYPE, 0);
-
-    if ((map = mmv_stats_start(registry)) == NULL) {
-	fprintf(stderr, "%s: instrumentation disabled\n", pmGetProgname());
-	return;
-    }
-
-    if ((value = mmv_lookup_value_desc(map, "pid", NULL)) != NULL)
-	mmv_set_value(map, value, pid);
-}
-
 static struct proxy *
 server_init(int portcount, const char *localpath)
 {
     struct server	*servers;
     struct proxy	*proxy;
     int			count;
+    mmv_registry_t	*registry;
+
+    if (pmWebTimerSetup() < 0) {
+	fprintf(stderr, "%s: error - failed to setup event timers\n",
+			pmGetProgname());
+	return NULL;
+    }
 
     if ((proxy = calloc(1, sizeof(struct proxy))) == NULL) {
 	fprintf(stderr, "%s: out-of-memory in proxy server setup\n",
 			pmGetProgname());
 	return NULL;
     }
+    uv_mutex_init(&proxy->write_mutex);
 
     count = portcount + (*localpath ? 1 : 0);
     if (count) {
@@ -168,11 +146,11 @@ server_init(int portcount, const char *localpath)
 
     proxy->config = config;
 
-    proxymetrics(proxy, METRICS_SERVER);
-    server_metrics_init(proxy);
+    if ((proxy->events = uv_default_loop()) != NULL)
+	pmWebTimerSetEventLoop(proxy->events);
 
-    proxy->events = uv_default_loop();
-    uv_loop_init(proxy->events);
+    if ((registry = proxymetrics(proxy, METRICS_SERVER)) != NULL)
+	pmWebTimerSetMetricRegistry(registry);
 
     return proxy;
 }
@@ -220,7 +198,7 @@ on_buffer_alloc(uv_handle_t *handle, size_t suggested_size, uv_buf_t *buf)
 	fprintf(stderr, "%s: handle %p buffer allocation of %lld bytes\n",
 			"on_buffer_alloc", handle, (long long)suggested_size);
 
-    if ((buf->base = sdsnewlen(SDS_NOINIT, suggested_size)) != NULL)
+    if ((buf->base = sdsnewlen(NULL, suggested_size)) != NULL)
 	buf->len = suggested_size;
     else
 	buf->len = 0;
@@ -258,20 +236,16 @@ client_put(struct client *client)
     uv_mutex_unlock(&client->mutex);
 
     if (refcount == 0) {
-	/* remove client from the doubly-linked list */
-	if (client->next != NULL)
-	    client->next->prev = client->prev;
-	*client->prev = client->next;
-
 	if (client->protocol & STREAM_PCP)
 	    on_pcp_client_close(client);
 	if (client->protocol & STREAM_HTTP)
 	    on_http_client_close(client);
 	if (client->protocol & STREAM_REDIS)
 	    on_redis_client_close(client);
-	if (client->protocol & STREAM_SECURE)
+	if ((client->protocol & STREAM_SECURE) && client->stream.secure)
 	    on_secure_client_close(client);
-
+	if (client->buffer)
+	    sdsfree(client->buffer);
 	memset(client, 0, sizeof(*client));
 	free(client);
     }
@@ -295,15 +269,15 @@ client_close(struct client *client)
 void
 on_client_write(uv_write_t *writer, int status)
 {
-    struct client	*client = (struct client *)writer->data;
-    stream_write_baton	*request = (stream_write_baton *)writer;
+    struct client		*client = (struct client *)writer->data;
+    struct stream_write_baton	*request = (struct stream_write_baton *)writer;
 
     if (pmDebugOptions.af)
 	fprintf(stderr, "%s: completed write [sts=%d] to client %p\n",
 			"on_client_write", status, client);
 
     if (status == 0) {
-	if (client->protocol & STREAM_SECURE)
+	if ((client->protocol & STREAM_SECURE) && client->stream.secure)
 	    on_secure_client_write(client);
 	if (client->protocol & STREAM_PCP)
 	    on_pcp_client_write(client);
@@ -332,9 +306,9 @@ on_client_write(uv_write_t *writer, int status)
 void *
 on_write_callback(uv_callback_t *handle, void *data)
 {
-    stream_write_baton	*request = (stream_write_baton *)data;
-    struct client	*client = (struct client *)request->writer.data;
-    int			sts;
+    struct stream_write_baton	*request = (struct stream_write_baton *)data;
+    struct client		*client = (struct client *)request->writer.data;
+    int				sts;
 
     if (pmDebugOptions.af)
 	fprintf(stderr, "%s: client=%p\n", "on_write_callback", client);
@@ -353,14 +327,14 @@ on_write_callback(uv_callback_t *handle, void *data)
 void
 client_write(struct client *client, sds buffer, sds suffix)
 {
-    stream_write_baton	*request;
-    struct proxy	*proxy = client->proxy;
-    unsigned int	nbuffers = 0;
+    struct stream_write_baton	*request;
+    struct proxy		*proxy = client->proxy;
+    unsigned int		nbuffers = 0;
 
     if (client_is_closed(client))
 	return;
 
-    if ((request = calloc(1, sizeof(stream_write_baton))) != NULL) {
+    if ((request = calloc(1, sizeof(struct stream_write_baton))) != NULL) {
 	if (pmDebugOptions.af)
 	    fprintf(stderr, "%s: sending %ld bytes [0] to client %p\n",
 			"client_write", (long)sdslen(buffer), client);
@@ -381,7 +355,7 @@ client_write(struct client *client, sds buffer, sds suffix)
     }
 }
 
-static stream_protocol
+static enum stream_protocol
 client_protocol(int key)
 {
     switch (key) {
@@ -453,10 +427,16 @@ on_client_read(uv_stream_t *stream, ssize_t nread, const uv_buf_t *buf)
     if (nread > 0) {
 	if (client->protocol == STREAM_UNKNOWN)
 	    client->protocol |= client_protocol(*buf->base);
-	if (client->protocol & STREAM_SECURE)
+
+#ifdef HAVE_OPENSSL
+	if ((client->protocol & STREAM_SECURE) && (proxy->ssl != NULL))
 	    on_secure_client_read(proxy, client, nread, buf);
 	else
 	    on_protocol_read(stream, nread, buf);
+#else
+	on_protocol_read(stream, nread, buf);
+#endif
+
     } else if (nread < 0) {
 	if (pmDebugOptions.af)
 	    fprintf(stderr, "%s: read error %ld "
@@ -513,12 +493,6 @@ on_client_connection(uv_stream_t *stream, int status)
     handle->data = (void *)proxy;
     client->proxy = proxy;
 
-    /* insert client into doubly-linked list at the head */
-    if ((client->next = proxy->first) != NULL)
-	proxy->first->prev = &client->next;
-    proxy->first = client;
-    client->prev = &proxy->first;
-
     status = uv_read_start((uv_stream_t *)&client->stream.u.tcp,
 			    on_buffer_alloc, on_client_read);
     if (status != 0) {
@@ -529,8 +503,9 @@ on_client_connection(uv_stream_t *stream, int status)
 }
 
 static int
-open_request_port(struct proxy *proxy, struct server *server, stream_family family,
-		const struct sockaddr *addr, int port, int maxpending)
+open_request_port(struct proxy *proxy, struct server *server,
+		stream_family_t family, const struct sockaddr *addr,
+		int port, int maxpending)
 {
     struct stream	*stream = &server->stream;
     uv_handle_t		*handle;
@@ -623,7 +598,7 @@ open_request_ports(char *localpath, size_t localpathlen, int maxpending)
     __pmSockAddr	*addr;
     struct proxyaddr	*addrlist;
     const struct sockaddr *sockaddr;
-    stream_family	family;
+    enum stream_family	family;
     struct server	*server;
     struct proxy	*proxy;
 
@@ -735,7 +710,7 @@ shutdown_ports(void *arg)
     struct proxy	*proxy = (struct proxy *)arg;
     struct server	*server;
     struct stream	*stream;
-    int			i;
+    unsigned int	i;
 
     for (i = 0; i < proxy->nservers; i++) {
 	server = &proxy->servers[i];
@@ -754,12 +729,11 @@ shutdown_ports(void *arg)
     proxy->nservers = 0;
 
     close_proxy(proxy);
-
     if (proxy->config) {
 	pmIniFileFree(proxy->config);
 	proxy->config = NULL;
     }
-
+    pmWebTimerClose();
     uv_loop_close(proxy->events);
     proxymetrics_close(proxy, METRICS_SERVER);
 
@@ -773,7 +747,8 @@ dump_request_ports(FILE *output, void *arg)
     struct proxy	*proxy = (struct proxy *)arg;
     struct stream	*stream;
     uv_os_fd_t		uv_fd;
-    int			i, fd;
+    unsigned int	i;
+    int			fd;
 
     fprintf(output, "%s request port(s):\n"
 		"  sts fd   port  family address\n"
